// Autogenerated by scripts/create_ganapatha.py

//! Implements rules from the *Gaṇapāṭha*.
//!
//! This module is largely auto-generated from the Ganapatha data on <https://ashtadhyayi.com>.
//! We have made a few minor corrections after the fact.
//!
//! For the source data, see <https://ashtadhyayi.com/ganapath>.

#![allow(unused)]

/// Models a *gaṇa-sūtra* from the *Gaṇapāṭha*.
#[derive(Copy, Clone, Debug, Hash, Eq, Ord, PartialEq, PartialOrd)]
pub struct GanapathaEntry {
    name: &'static str,
    number: u16,
    code: &'static str,
    items: &'static [&'static str],
    kind: GanaKind,
    varttika: Option<&'static str>,
}

/// Models the *gaṇa* type.
#[derive(Copy, Clone, Debug, Hash, Eq, Ord, PartialEq, PartialOrd)]
pub enum GanaKind {
    /// A basic *gaṇa* whose members are all listed.
    Basic,
    /// A *gaṇa* whose members form an incomplete set. The full set can be known only by
    /// observing actual usage.
    Akrti,
}

impl GanapathaEntry {
    pub(crate) const fn basic(
        name: &'static str,
        number: u16,
        code: &'static str,
        items: &'static [&'static str],
    ) -> Self {
        Self {
            name,
            number,
            code,
            items,
            kind: GanaKind::Basic,
            varttika: None,
        }
    }

    pub(crate) const fn akrti(
        name: &'static str,
        number: u16,
        code: &'static str,
        items: &'static [&'static str],
    ) -> Self {
        Self {
            name,
            number,
            code,
            items,
            kind: GanaKind::Akrti,
            varttika: None,
        }
    }

    pub(crate) const fn with_varttika(mut self: GanapathaEntry, varttika: &'static str) -> Self {
        self.varttika = Some(varttika);
        self
    }

    /// The name of this *gaṇa* in SLP1 transliteration.
    pub fn name(&self) -> &str {
        self.name
    }

    /// The number of this *gaṇa* relative to other *gaṇa*s in the Ganapatha.
    pub fn number(&self) -> u16 {
        self.number
    }

    /// The string ID of the Ashtadhyayi rule that first uses this *gaṇa*.
    pub fn code(&self) -> &str {
        self.code
    }

    /// All items in the *gaṇa*.
    pub fn items(&self) -> &[&str] {
        self.items
    }

    /// The type of this *gaṇa*.
    pub fn kind(&self) -> GanaKind {
        self.kind
    }

    /// A *vārttika* associated with this *gaṇa*, if one exists.
    pub fn varttika(&self) -> Option<&str> {
        self.varttika
    }
}

/// *sarvādi-gaṇa* (1), first used in the following *sūtra*:
///
/// > 1.1.27 *sarvādīni sarvanāmāni*
pub const SARVADI: GanapathaEntry = GanapathaEntry::basic(
    "sarvAdiH",
    1,
    "1.1.27",
    &[
        "sarva", "viSva", "uBa", "uBaya", "qatara", "qatama", "anya", "anyatara", "itara", "tvat",
        "tva", "nema", "sama", "sima",
        // <<pUrvaparAvaradakziRottarAparADarARivyavasTAyAmasaMjYAyAm>>
        "pUrva", "para", "avara", "dakziRa", "uttara", "apara", "aDara",
        // <<svamajYAtiDanAKyAyAm>>
        "sva", // <<antaraM bahiryogopasaMvyAnayoH>>
        "antara", "tyad", "tad", "yad", "etad", "idam", "adas", "eka", "dvi", "yuzmad", "asmad",
        "Bavatu", "kim",
    ],
);

/// *svarādi-gaṇa* (2), first used in the following *sūtra*:
///
/// > 1.1.37 *svarādinipātamavyayam*
pub(crate) const SVARADI: GanapathaEntry = GanapathaEntry::akrti(
    "svarAdiH",
    2,
    "1.1.37",
    &[
        "svar",
        "antar",
        "prAtar",
        // <i>antodAttAH</i>
        "punar",
        "sanutar",
        "uccEs",
        "nIcEs",
        "SanEs",
        "fDak",
        "fte",
        "yugapat",
        "ArAt",
        "antikAt",
        "pfTak",
        // <i>AdyudAttAH</i>
        "hyas",
        "Svas",
        "divA",
        "rAtrO",
        "sAyam",
        "ciram",
        "manAk",
        "Izat",
        "SaSvat",
        "jozam",
        "tUzRIm",
        "bahis",
        "aDas",
        "avas",
        "samayA",
        "nikazA",
        "svayam",
        "mfzA",
        "naktam",
        "naY",
        "hetO",
        "he",
        "hE",
        "idDA",
        "adDA",
        "sAmi",
        // <i>antodAttAH</i>
        // <=vat=> (gaRasUtram) , yaTA  brAhmaRavat, kzatriyavat
        "sanA",
        "sanat",
        "sanAt",
        "upaDA",
        "tiras",
        // <i>AdyudAttAH</i>
        "antarA",
        "antodAttAH",
        "antareRa",
        "mak",
        "jyok",
        "yok",
        "nak",
        "kam",
        "Sam",
        "sanA",
        "sahasA",
        "SradDA",
        "alam",
        "svaDA",
        "vazaw",
        "vinA",
        "nAnA",
        "svasti",
        "anyat",
        "asti",
        "upAMSu",
        "kzamA",
        "vihAyasA",
        "dozA",
        "mfzA",
        "muDA",
        "dizwyA",
        "vfTA",
        "miTyA",
        // <=ktavAtosunkasunaH=> (gaRasUtram)
        // <=kfnmakArasanDyakzarAnto'vyayIBAvaSca=> (gaRasUtram)
        "purA",
        "miTo",
        "miTas",
        "prAyas",
        "muhus",
        "pravAhukam",
        "pravAhikA",
        "Aryahalam",
        "aBIkzRam",
        "sAyam",
        "sAkam",
        "sArDam",
        "sArTam",
        "satram",
        "samam",
        "namas",
        "hiruk",
        // <=tasilAdayastadDita eDAccaparyantAH=> (gaRasUtram)
        // SastasI, kftvasuc, suc, AsTAlO
        // <=cvyarTASca=> (gaRasUtram)
        "aTa",
        "am",
        "Am",
        "pratAm",
        "pratAn",
        "praSAn",
        "taTAhi",
        "mA",
        "mAN",
        "Sram",
        "kAmam",
        "prakAmam",
        "BUyas",
        "param",
        "sAkzAt",
        "sAci",
        "sAvi",
        "satyam",
        "maMkzu",
        "saMvat",
        "avaSyam",
        "sapadi",
        "prAdus",
        "Avis",
        "aniSam",
        "nityam",
        "nityadA",
        "sadA",
        "ajasram",
        "santatam",
        "uzA",
        "om",
        "BUr",
        "Buvar",
        "Jawiti",
        "tarasA",
        "suzWu",
        "ku",
        "aYjasA",
        "a",
        "miTu",
        "amiTu",
        "viTak",
        "BAjak",
        "anvak",
        "cirAya",
        "ciram",
        "cirarAtrAya",
        "cirasya",
        "cireRa",
        "cirAt",
        "astam",
        "Anuzak",
        "anuzak",
        "anuzaw",
        "amnas",
        "amBas",
        "amnar",
        "amBar",
        "sTAne",
        "varam",
        "duzWu",
        "balAt",
        "Su",
        "arvAk",
        "Sudi",
        "vadi",
        "ityAdi",
        // <= tasilAdayaH prAkpASapAH=> (gaRasUtram)
        // <=SaspraBftayaH prAksamAsAnteByaH=> (gaRasUtram)
        // <=mAntaH kftvorTaH=>
        "nAnAYO",
    ],
);

/// *cādi-gaṇa* (3), first used in the following *sūtra*:
///
/// > 1.4.57 *cādayō'sattvē*
pub(crate) const CADI: GanapathaEntry = GanapathaEntry::akrti(
    "cAdiH",
    3,
    "1.4.57",
    &[
        "ca", "vA", "ha", "aha", "eva", "evam", "nUnam", "SaSvat", "yugapat", "BUyas", "sUpat",
        "kUpat", "kuvit", "net", "cet", "caR", "kaccit", "yatra", "tatra", "naha", "hanta",
        "mAkim", "mAkIm", "mAkir", "nakim", "nakIm", "nakir", "AkIm", "mANa", "naY", "tAvat",
        "yAvat", "tvA", "tvE", "dvE", "nvE", "rE", "re", "SrOzaw", "vOzaw", "svAhA", "vazaw",
        "svaDA", "om", "taTA", "taTAhi", "Kalu", "kila", "aTa", "suzwu", "sma", "a", "i", "u", "f",
        "x", "u", "e", "E", "o", "O", "Adaha", "uY", "ukaY", "velAyAm", "mAtrAyAm", "yaTA", "yat",
        "tat", "kim", "purA", "vaDA", "vaDvA", "Dik", "hAhA", "hehE", "hahe", "pAw", "pyAw", "Aho",
        "utAho", "ho", "aho", "no", "nO", "aTo", "nanu", "manye", "miTyA", "asi", "brUhi", "tu",
        "nu", "iti", "iva", "vat", "vAt", "vana", "bata", "sam", "vaSam", "Sikam", "dikam",
        "sanukaM", "CaMvaw", "CaMbaw", "SaNke", "Sukam", "Kam", "sanAt", "sanutar", "nahikam",
        "satyam", "ftam", "adDA", "idDA", "nocet", "nacet", "nahi", "jAtu", "kaTam", "kutaH",
        "kutra", "ava", "anu", "hA", "he", "hE", "Ahosvit", "Sam", "kam", "Kam", "dizwyA", "paSu",
        "naw", "saha", "Anuzaw", "Anuzak", "aNga", "Paw", "tAjak", "BAjak", "aye", "are", "vAw",
        "cAwu", "kum", "Kum", "Gum", "am", "Im", "sIm", "sim", "si",
        "vE",
        // <<upasargaviBaktisvarapratirUpakASca nipAtAH>>
    ],
);

/// *prādi-gaṇa* (4), first used in the following *sūtra*:
///
/// > 1.4.58 *prādayaḥ*
pub(crate) const PRADI: GanapathaEntry = GanapathaEntry::basic(
    "prAdiH",
    4,
    "1.4.58",
    &[
        "pra", "parA", "apa", "sam", "anu", "ava", "nis", "nir", "dus", "dur", "vi", "AN", "ni",
        "aDi", "api", "ati", "su", "ud", "aBi", "prati", "pari", "upa",
    ],
);

/// *ūryādi-gaṇa* (5), first used in the following *sūtra*:
///
/// > 1.4.61 *ūryādicviḍācaśca*
pub(crate) const URYADI: GanapathaEntry = GanapathaEntry::basic(
    "UryAdiH",
    5,
    "1.4.61",
    &[
        "UrI",
        "urarI",
        "tanTI",
        "tAlI",
        "AttAlI",
        "vetAlI",
        "DUlI",
        "DUsa",
        "SakalA",
        "saMSakalA",
        "DvaMsakalA",
        "BraMsakalA",
        "guluguDA",
        "sajUs",
        "Pala",
        "PalI",
        "viklI",
        "AklI",
        "AlozWI",
        "kevAlI",
        "kevAsI",
        "sevAsI",
        "paryAlI",
        "SevAlI",
        "varzAlI",
        "atyUmaSA",
        "maSmaSA",
        "masmasA",
        "masamasA",
        "Ozaw",
        "SrOzaw",
        "vOzaw",
        "vazaw",
        "svAhA",
        "svaDA",
        "vanDA",
        "prAdus",
        "Srat",
        "Avis",
    ],
);

/// *sākṣātprabhr̥ti-gaṇa* (6), first used in the following *sūtra*:
///
/// > 1.4.74 *sākṣātprabhr̥tīni ca*
pub(crate) const SAKSHATPRABHRTI: GanapathaEntry = GanapathaEntry::akrti(
    "sAkzAtpraBftiH",
    6,
    "1.4.74",
    &[
        "sAkzAt",
        "miTyA",
        "cintA",
        "BadrA",
        "rocanA",
        "AsTA",
        "amA",
        "SradDA",
        "prAjaryA",
        "prAjaruhA",
        "bIjaryA",
        "bIjaruhA",
        "saMsaryA",
        "arTe",
        "lavaRam",
        "uzRam",
        "SItam",
        "udakam",
        "Ardram",
        "agnO",
        "vaSe",
        "vihasane",
        "prasahane",
        "pratapane",
        "prAdus",
        "namas",
        "Avis",
    ],
);

/// *tiṣṭhadguprabhr̥ti-gaṇa* (7), first used in the following *sūtra*:
///
/// > 2.1.17 *tiṣṭhadguprabhr̥tīni ca*
pub(crate) const TISHTHADGUPRABHRTI: GanapathaEntry = GanapathaEntry::basic(
    "tizWadgupraBftiH",
    7,
    "2.1.17",
    &[
        "tizWadgu",
        "vahadgu",
        "AyatIgavam",
        "Kaleyavam",
        "Kalebusam",
        "lUnayavam",
        "lUyamAnayavam",
        "pUtayavam",
        "pUyamAnayavam",
        "saMhftayavam",
        "saMhriyamARAyavam",
        "saMhftabusam",
        "saMhriyamARAbusam",
        "samaBUmi",
        "samapadAti",
        "suzamam",
        "vizamam",
        "duHzamam",
        "niHzamam",
        "aparasamam",
        "AyatIsamam",
        "prOQam",
        "pApasamam",
        "puRyasamam",
        "prAhRam",
        "praraTam",
        "praBfgam",
        "pradakziRam",
        "aparadakziRam",
        "saMprati",
        "asaMprati",
        // <<itpratyayaH samAsAntaH>> [[5
        "4",
        // 127]] - [[5
        "4",
        "128]]",
    ],
);

/// *śauṇḍādi-gaṇa* (8), first used in the following *sūtra*:
///
/// > 2.1.40 *saptamī śauṇḍaiḥ*
pub(crate) const SHAUNDADI: GanapathaEntry = GanapathaEntry::basic(
    "SORqAdiH",
    8,
    "2.1.40",
    &[
        "SORqa", "DUrta", "kitava", "vyAqa", "pravIRa", "saMvIta", "antar", "aDi", "pawu",
        "paRqita", "kuSala", "capala", "nipuRa",
    ],
);

/// *pātrēsamitādi-gaṇa* (9), first used in the following *sūtra*:
///
/// > 2.1.48 *pātrēsamitādayaśca*
pub(crate) const PATRESAMITADI: GanapathaEntry = GanapathaEntry::akrti(
    "pAtresamitAdiH",
    9,
    "2.1.48",
    &[
        "pAtresamitAH",
        "pAtrebahulAH",
        "udumbaramaSakAH",
        "udarakfmiH",
        "kUpakacCapaH",
        "avawakacCapaH",
        "kUpamaRqUkaH",
        "kumBamaRqUkaH",
        "udapAnamaRqUkaH",
        "nagarakAkaH",
        "nagaravAyasaH",
        "mAtarizuruzaH",
        "piRqISUraH",
        "pitariSUraH",
        "geheSUraH",
        "gehenardI",
        "gehekzveqI",
        "gehevijitI",
        "gehevyAqaH",
        "gehemehI",
        "gehedAhI",
        "gehedfptaH",
        "geheDfzwaH",
        "garBetfptaH",
        "AKanikavakaH",
        "gozWeSUraH",
        "gozWevijitI",
        "gozWekzveqI",
        "gozWepawu",
        "gozWepaRqitaH",
        "gozWepragalBaH",
        "karRewiriwirA",
        "karRecuracurA",
    ],
);

/// *vyāghrādi-gaṇa* (10), first used in the following *sūtra*:
///
/// > 2.1.56 *upamitaṁ vyāghrādibhiḥ sāmānyāprayōgē*
pub(crate) const VYAGHRADI: GanapathaEntry = GanapathaEntry::akrti(
    "vyAGrAdiH",
    10,
    "2.1.56",
    &[
        "vyAGra",
        "siMha",
        "fkza",
        "fzaBa",
        "candana",
        "vfka",
        "vfza",
        "varAha",
        "hastin",
        "taru",
        "kuYjara",
        "ruru",
        "pfzat",
        "puRqarIka",
        "palASa",
        "kitava",
        "muKapadmam",
        "muKakamalam",
        "karakisalayam",
        "pArTivacandraH",
    ],
);

/// *śrēṇyādi-gaṇa* (11), first used in the following *sūtra*:
///
/// > 2.1.59 *śrēṇyādayaḥ kr̥tādibhiḥ*
pub(crate) const SHRENYADI: GanapathaEntry = GanapathaEntry::basic(
    "SreRyAdiH",
    11,
    "2.1.59",
    &[
        "SreRi",
        "eka",
        "pUga",
        "mukunda",
        "rASi",
        "nicaya",
        "vizaya",
        "niDana",
        "para",
        "indra",
        "deva",
        "muRqaBUta",
        "SravaRa",
        "vadAnya",
        "aDyApaka",
        "aBirUpaka",
        "brAhmaRa",
        "kzatriya",
        "viSizwa",
        "pawu",
        "paRqita",
        "kuSala",
        "capala",
        "nipuRa",
        "kfpaRa",
    ],
);

/// *kr̥tādi-gaṇa* (12), first used in the following *sūtra*:
///
/// > 2.1.59 *śrēṇyādayaḥ kr̥tādibhiḥ*
pub(crate) const KRTADI: GanapathaEntry = GanapathaEntry::basic(
    "kftAdiH",
    12,
    "2.1.59",
    &[
        "kfta",
        "mita",
        "mata",
        "BUta",
        "ukta",
        "yukta",
        "samAjYAta",
        "samAmnAta",
        "samAKyAta",
        "samBAvita",
        "saMsevita",
        "avaDArita",
        "avakalpita",
        "nirAkfta",
        "upakfta",
        "upAkfta",
        "dfzwa",
        "kalita",
        "dalita",
        "udAhfta",
        "viSruta",
        "udita",
    ],
);

/// *śākapārthivādi-gaṇa* (13), first used in the following *sūtra*:
///
/// > 2.1.60 *ktēna nañviśiṣṭēnānañ*
/// The *sūtra* has the following *vārttika*:
///
/// > *SAkapArTivAdInAmupasaNKyAnam*
pub(crate) const SHAKAPARTHIVADI: GanapathaEntry = GanapathaEntry::akrti(
    "SAkapArTivAdiH",
    13,
    "2.1.60",
    &[
        "SAkapArTivaH",
        "kutapasOSruta",
        "ajAtOlvali",
        "kftAkfta",
        "BuktaviBukta",
        "pItavipIta",
        "gatapratyAgata",
        "yAtAnuyAta",
        "krayAkrayikA",
        "puwApuwikA",
        "PalAPalikA",
        "mAnonmAnikA",
    ],
)
.with_varttika("SAkapArTivAdInAmupasaNKyAnam");

/// *śramaṇādi-gaṇa* (14), first used in the following *sūtra*:
///
/// > 2.1.70 *kumāraḥ śramaṇādibhiḥ*
pub(crate) const SHRAMANADI: GanapathaEntry = GanapathaEntry::basic(
    "SramaRAdiH",
    14,
    "2.1.70",
    &[
        "SramaRA",
        "pravrajitA",
        "kulawA",
        "garBiRI",
        "tApasI",
        "dAsI",
        "banDakI",
        "aDyApaka",
        "aBirUpaka",
        "paRqita",
        "pawu",
        "mfdu",
        "kuSala",
        "capala",
        "nipuRA",
    ],
);

/// *mayūravyaṁsakādi-gaṇa* (15), first used in the following *sūtra*:
///
/// > 2.1.72 *mayūravyaṁsakādayaśca*
pub(crate) const MAYURAVYAMSAKADI: GanapathaEntry = GanapathaEntry::akrti(
    "mayUravyaMsakAdiH",
    15,
    "2.1.72",
    &[
        "mayUravyaMsaka",
        "CAtravyaMsaka",
        "kambojamuRqa",
        "yavanamuRqa",
        // <<Candasi hastegfhya>>
        "hastagfhya",
        "pAdegfhya",
        "pAdagfhya",
        "lANgUlegfhya",
        "lANgUlagfhya",
        "punardAya",
        // <<ehIqAdayo'nyapadArTe>>
        "ehIqam",
        "ehiyavam",
        // <<ehivARijA kriyA>>
        "apehivARijA",
        "prehivARijA",
        "ehisvAgatA",
        "apohisvAgatA",
        "prehisvAgatA",
        "ehidvitIyA",
        "apehidvitIyA",
        "prehidvitIyA",
        "ehikawA",
        "apohakawA",
        "prohakawA",
        "AharakawA",
        "prohikardamA",
        "prohakardamA",
        "viDamacUqA",
        "udDamacUqA",
        "udDaracUqA",
        "AharacelA",
        "haravasana",
        "AharavanitA",
        "hatavinata",
        "kfntavicakzaRA",
        "udDarotsfjA",
        "udDarAvasfjA",
        "udDamaviDamA",
        "utpacanipacA",
        "utpatanipatA",
        "uccAvacam",
        "uccanIcam",
        "acopacam",
        "AparAcam",
        "Kapracama",
        "niScapracam",
        "akiMcana",
        "snAtvAkAlaka",
        "pItvAsTiraka",
        "BuktvAsuhita",
        "prozyapApIyAn",
        "utpatyapAkalA",
        "nipatyarohiRI",
        "nizaRRASyAmA",
        "aprehipraGasA",
        "prehipraGasA",
        "ihapaYcamI",
        "ihadvitIyA",
        // <<jahi karmaRA bahulamABIkzRye kartAraM cABidaDAti>>
        "jahijoqa",
        "jahijoqam",
        "jahistambaH",
        "jjahistambama",
        // <<AKyAtamAKyAtena kriyAsAtatye>>
        "aSnItapibatA",
        "pacataBfjjatA",
        "KAdatamodatA",
        "KAdatavamatA",
        "KAdatAcamatA",
        "AharanivapA",
        "AvapanizkirA",
        "utpacacvipacA",
        "BinDilavaRA",
        "kfnDivicakzanA",
        "pacalavaRA",
        "pacaprakUwA",
        "akutoBayaH",
        "kAndiSIkaH",
        "kAndoSIkaH",
        "strAhopuruzikA",
        "ahamahamikA",
        "yadfcCA",
        "ehireyAhirA",
        "unmfjAvamfjA",
        "dravyAntaram",
        "avaSyakAryam",
    ],
);

/// *yājakādi-gaṇa* (16), first used in the following *sūtra*:
///
/// > 2.2.9 *yājakādibhiśca*
pub(crate) const YAJAKADI: GanapathaEntry = GanapathaEntry::basic(
    "yAjakAdiH",
    16,
    "2.2.9",
    &[
        "yAjaka",
        "pUjaka",
        "paricAraka",
        "parivezaka",
        "parizecaka",
        "snAtaka",
        "aDyApaka",
        "utsAha",
        "udvartaka",
        "hotf",
        "Batf",
        "raTagaRaka",
        "pattigaRaka",
    ],
);

/// *rājadantādi-gaṇa* (17), first used in the following *sūtra*:
///
/// > 2.2.31 *rājadantādiṣu param*
pub(crate) const RAJADANTADI: GanapathaEntry = GanapathaEntry::basic(
    "rAjadantAdiH",
    17,
    "2.2.31",
    &[
        "rAjadantaH",
        "agrevaRam",
        "liptavAsitam",
        "nagnamuzitam",
        "siktasaMmfzwam",
        "mfzwaluYcitam",
        "avaklinnapakvam",
        "arpitoptam",
        "uptagAQam",
        "ulUKalamusalam",
        "taRqulakiRvam",
        "dfzadupalam",
        "AraqvAyani",
        "AragvAyanabanDakI",
        "citraraTavAhlIkam",
        "AvantyaSmakam",
        "SUdrAryam",
        "snAtakarAjAnO",
        "vizvaksenArjunO",
        "akziBruvam",
        "dAragavam",
        "SabdArTO",
        "DarmArTO",
        "kAmArTO",
        "arTaSabdO",
        "arTaDarmO",
        "arTakAmO",
        "vEkArimatam",
        "gAjavAjam",
        "gopAlaDAnapUlAsam",
        "gopAlaDAnIpUlAsam",
        "pUlAsakAraRqam",
        "pUlAsakakuraRqam",
        "sTUlAsama",
        "sTUlapUlAsam",
        "uSIrabIjam",
        "jijYAsTi",
        "siYjAsTam",
        "siYjASvatTam",
        "citrAsvAtI",
        "citrasvAtI",
        "BAryApatI",
        "daMpatI",
        "jaMpatI",
        "jAyApatI",
        "putrapatI",
        "putrapaSU",
        "keSaraSmaSrU",
        "Sirobiju",
        "SirobIjama",
        "SirojAnu",
        "sarpirmaDuna",
        "maDusarpizI",
        "AdyantO",
        "antAdI",
        "guRavfdDI",
        "vfdDiguRO",
    ],
);

/// *āhitāgnyādi-gaṇa* (18), first used in the following *sūtra*:
///
/// > 2.2.37 *vāhitāgnyādiṣu*
pub(crate) const AHITAGNYADI: GanapathaEntry = GanapathaEntry::akrti(
    "AhitAgnyAdiH",
    18,
    "2.2.37",
    &[
        "AhitAgniH",
        "jAtapUtraH",
        "jAtadanta",
        "jAtaSamaSru",
        "tElapIta",
        "GftapIta",
        "dyapI",
        "UQaBArya",
        "gatArTa",
        "gaqukaRWa",
        "asyudyata",
        "aramudyata",
        "daRqapARi",
    ],
);

/// *kaḍārādi-gaṇa* (19), first used in the following *sūtra*:
///
/// > 2.2.38 *kaḍārāḥ karmadhārayē*
pub(crate) const KADARADI: GanapathaEntry = GanapathaEntry::basic(
    "kaqArAdiH",
    19,
    "2.2.38",
    &[
        "kaqAra", "guqula", "KaYja", "Koqa", "kARa", "kuRWa", "Kalati", "gOra", "vfdDa", "Bikzuka",
        "piNgula", "piNgala", "taqa", "tanu", "Wa", "baDira", "maWara", "kaYja", "barbara",
    ],
);

/// *nāvādi-gaṇa* (20), first used in the following *sūtra*:
///
/// > 2.3.17 *manyakarmaṇyanādarē vibhāṣā'prāṇiṣu*
/// The *sūtra* has the following *vārttika*:
///
/// > *nOkAkAnnaSukaSfgAlavarjezu*
pub(crate) const NAVADI: GanapathaEntry = GanapathaEntry::basic(
    "nAvAdiH",
    20,
    "2.3.17",
    &["nO", "kAka", "anna", "Suka", "SfgAla"],
)
.with_varttika("nOkAkAnnaSukaSfgAlavarjezu");

/// *prakr̥tyādi-gaṇa* (21), first used in the following *sūtra*:
///
/// > 2.3.18 *kartr̥karaṇayōstr̥tīyā*
pub(crate) const PRAKRTYADI: GanapathaEntry = GanapathaEntry::basic(
    "prakftyAdiH",
    21,
    "2.3.18",
    &[
        "prakfti", "prAya", "gotra", "sama", "vizama", "dvidroRa", "paYcaka", "sAhasra",
    ],
);

/// *pratyādi-gaṇa* (22), first used in the following *sūtra*:
///
/// > 2.3.43 *sādhunipuṇābhyāmarcāyāṁ saptamyapratēḥ*
/// The *sūtra* has the following *vārttika*:
///
/// > *apratyAdiBiriti vaktavyam*
pub(crate) const PRATYADI: GanapathaEntry =
    GanapathaEntry::basic("pratyAdiH", 22, "2.3.43", &["prati", "pari", "anu"])
        .with_varttika("apratyAdiBiriti vaktavyam");

/// *gavāśvādi-gaṇa* (23), first used in the following *sūtra*:
///
/// > 2.4.11 *gavāśvaprabhr̥tīni ca*
pub(crate) const GAVASHVADI: GanapathaEntry = GanapathaEntry::basic(
    "gavASvAdiH",
    23,
    "2.4.11",
    &[
        "gavASvam",
        "gavAvikam",
        "gavEqakam",
        "ajAvikam",
        "ajEqakam",
        "kubjavAmanam",
        "kubjakirAtam",
        "putrapOtram",
        "SvacaRqAlam",
        "strIkumAram",
        "dAsImARavakam",
        "SAwIpawIram",
        "SAwIpacCakam",
        "SAwIpawwIkam",
        "uzwraKaram",
        "uzwraSaSam",
        "mUtraSakft",
        "mUtrapurIzam",
        "yakfnmedaH",
        "mAMsaSoRitam",
        "darBaSaram",
        "darBapUtIkam",
        "arjunaSirIzam",
        "arjunapuruzam",
        "tfRopalam",
        "tfRolapama",
        "dAsIdAsam",
        "kuwIkuwam",
        "BAgavatIBAgavatam",
    ],
);

/// *dadhipayasyādi-gaṇa* (24), first used in the following *sūtra*:
///
/// > 2.4.14 *na dadhipayaādīni*
pub(crate) const DADHIPAYASYADI: GanapathaEntry = GanapathaEntry::basic(
    "daDipayasyAdiH",
    24,
    "2.4.14",
    &[
        "daDipayasI",
        "sarpirmaDunI",
        "maDusapizI",
        "brahmaprajApatI",
        "SivavESravaRO",
        "skandaviSAKO",
        "parivrAjakakOSikO",
        // ( parivrAwkOSikO )
        "pravargyopasadO",
        "SuklakfzRO",
        "iDmAbarhizI",
        "dIkzAtapasI",
        "SradDAtapasI",
        "meDAtapasI",
        "aDyayanatapasI",
        "ulUKalamusale",
        "AdyAvasAne",
        "SradDAmeDe",
        "fksAme",
        "vANmanase",
    ],
);

/// *ardharcādi-gaṇa* (25), first used in the following *sūtra*:
///
/// > 2.4.31 *ardharcāḥ puṁsi ca*
pub(crate) const ARDHARCADI: GanapathaEntry = GanapathaEntry::basic(
    "arDarcAdiH",
    25,
    "2.4.31",
    &[
        "arDarca",
        "gomaya",
        "kazAya",
        "kArzApaRa",
        "kutapa",
        "kusapa",
        "kuRapa",
        "kapAwa",
        "SaNKa",
        "gUTa",
        "yUTa",
        "Dvaja",
        "kabanDa",
        "padma",
        "gfha",
        "saraka",
        "kaMsa",
        "divasa",
        "yUza",
        "anDakAra",
        "daRqa",
        "kamaRqalu",
        "manqa",
        "BUta",
        "dvIpa",
        "dyUta",
        "cakra",
        "Darma",
        "karman",
        "modaka",
        "SatamAna",
        "yAna",
        "naKa",
        "naKara",
        "caraRa",
        "pucCa",
        "dAqima",
        "hima",
        "rajata",
        "saktu",
        "piDAna",
        "sAra",
        "pAtra",
        "Gfta",
        "sEnDava",
        "OzaDa",
        "AQaka",
        "cazaka",
        "droRa",
        "KalIna",
        "pAtrIva",
        "zazwika",
        "vArabARa",
        "vAravAraRa",
        "proTa",
        "kapitTa",
        "Suzka",
        "SAla",
        "SIla",
        "Sukla",
        "Sulka",
        "SIDu",
        "kavaca",
        "reRu",
        "fRa",
        "kapawa",
        "SIkara",
        "musala",
        "suvarRa",
        "varRa",
        "pUrva",
        "camasa",
        "kzIra",
        "karza",
        "AkASa",
        "azwApada",
        "maNgala",
        "niDana",
        "niryAsa",
        "jfmBa",
        "vftta",
        "pusta",
        "busta",
        "kzveqita",
        "SfNga",
        "nigaqa",
        "Kala",
        "mUlaka",
        "maDu",
        "mUla",
        "sTUla",
        "SarAva",
        "nAla",
        "vapra",
        "vimAna",
        "muKa",
        "pragrIva",
        "SUla",
        "vajra",
        "kawaka",
        "kaRwaka",
        "karpawa",
        "SiKara",
        "kalka",
        "valkala",
        "nawamastaka",
        "nAwamastaka",
        "valaya",
        "kusuma",
        "tfRa",
        "paNka",
        "kuRqala",
        "kirIwa",
        "kumuda",
        "arbuda",
        "aNkuSa",
        "timira",
        "ASraya",
        "BUzaRa",
        "ikkasa",
        "izvAsa",
        "mukula",
        "vasanta",
        "tawAka",
        "taqAga",
        "piwaka",
        "viwaNka",
        "viqaNga",
        "piRyAka",
        "mAza",
        "koSa",
        "Palaka",
        "dina",
        "dEvata",
        "pinAka",
        "samara",
        "sTARu",
        "anIka",
        "upavAsa",
        "SAka",
        "karpAsa",
        "viSAla",
        "cazAla",
        "caKAla",
        "KaRqa",
        "dara",
        "viwapa",
        "raRa",
        "bala",
        "maka",
        "mfRAla",
        "hasta",
        "Ardra",
        "hala",
        "sUtra",
        "tARqava",
        "gARqIva",
        "maRqapa",
        "pawaha",
        "sODa",
        "yoDa",
        "pArSva",
        "SarIra",
        "Pala",
        "Cala",
        "pura",
        "purA",
        "rAzwra",
        "ambara",
        "bimba",
        "kuwwima",
        "maRqala",
        "kukkuwa",
        "kuqapa",
        "kakuda",
        "KaRqala",
        "tomara",
        "toraRa",
        "maYcaka",
        "puNKa",
        "maDya",
        "bAla",
        "CAla",
        "valmIka",
        "varza",
        "vastra",
        "vasu",
        "deha",
        "udyAna",
        "udyoga",
        "sneha",
        "stena",
        "Kana",
        "svara",
        "saNgama",
        "nizka",
        "kzema",
        "SUka",
        "kzatrapavitra",
        "yOvana",
        "kalaha",
        "mAlaka",
        "pAlaka",
        "mUzika",
        "maRqala",
        "valkala",
        "kuja",
        "kuYja",
        "vihAra",
        "lohita",
        "vizARa",
        "Bavana",
        "araRya",
        "pulina",
        "dfQa",
        "Asana",
        "ErAvata",
        "SUrpa",
        "tIrTa",
        "lomana",
        "lomaSa",
        "tamAla",
        "loha",
        "daRqaka",
        "SapaTa",
        "pratisara",
        "dAru",
        "Danus",
        "mAna",
        "varcaska",
        "kUrca",
        "taRqaka",
        "maWa",
        "sahasra",
        "odana",
        "pravAla",
        "Sakawa",
        "aparAhRa",
        "nIqa",
        "Sakala",
        "taRqula",
    ],
);

/// *pailādi-gaṇa* (26), first used in the following *sūtra*:
///
/// > 2.4.59 *pailādibhyaśca*
pub(crate) const PAILADI: GanapathaEntry = GanapathaEntry::akrti(
    "pElAdiH",
    26,
    "2.4.59",
    &[
        "pEla",
        "SAlaNki",
        "sAtyaki",
        "sAtyakAmi",
        "rAhavi",
        "rAvaRi",
        "OdaYco",
        "Odavraji",
        "OdameGi",
        "Odavyajri",
        "Odamajji",
        "Odamfjji",
        "dEvasTAni",
        "pENgalOdAyani",
        "rAhakzati",
        "BOliNgi",
        "Odanyi",
        "OdgAhamAni",
        "OjjihAni",
        "OdaSudDi",
        "tadrAjAccARaH",
        "tadrAja",
    ],
);

/// *taulvalyādi-gaṇa* (27), first used in the following *sūtra*:
///
/// > 2.4.61 *na taulvalibhyaḥ*
pub(crate) const TAULVALYADI: GanapathaEntry = GanapathaEntry::basic(
    "tOlvalyAdiH",
    27,
    "2.4.61",
    &[
        "tOlvali",
        "DAraRi",
        "pAraRi",
        "rAvaRi",
        "dElIpi",
        "dEvati",
        "vArkali",
        "nEvati",
        "nEvaki",
        "dEvamitri",
        "dEvamati",
        "dEvayajYi",
        "cAPawwaki",
        "bElvaki",
        "vEki",
        "vENki",
        "AnuhArati",
        "AnurAhati",
        "pOzkarasAdi",
        "Anurohati",
        "Anuti",
        "prAdohani",
        "nEmiSri",
        "prAqAhati",
        "bAnDaki",
        "vESIti",
        "AsinAsi",
        "AMhisi",
        "AsurI",
        "nEmizi",
        "AsibanDaki",
        "pOzpi",
        "kAreRupAli",
        "vEkarRi",
        "vEraki",
        "vEhati",
    ],
);

/// *yaskādi-gaṇa* (28), first used in the following *sūtra*:
///
/// > 2.4.63 *yaskādibhyō gōtrē*
pub(crate) const YASKADI: GanapathaEntry = GanapathaEntry::basic(
    "yaskAdiH",
    28,
    "2.4.63",
    &[
        "yaska",
        "lahya",
        "duhya",
        "ayasTURa",
        "ayaHsTURa",
        "tfRakar",
        "sadAmatta",
        "kambalahAra",
        "bahiryoga",
        "parRAQaka",
        "karRAQaka",
        "piRqIjaNGa",
        "bakasakTa",
        "vakasakTa",
        "voSri",
        "kudri",
        "ajavasti",
        "mitrayu",
        "rakzomuKa",
        "jaNGAraTa",
        "utkAsa",
        "kawukamaTaka",
        "manTaka",
        "puzkaraw",
        "puzkarasad",
        "vizapuwa",
        "uparimeKala",
        "krozwukamAna",
        "krozwumAna",
        "krozwupAda",
        "krozwumAya",
        "SIrzamAya",
        "Karapa",
        "padaka",
        "varzuka",
        "Balandana",
        "Baqila",
        "BaRqila",
        "Baqita",
    ],
);

/// *gōpavanādi-gaṇa* (29), first used in the following *sūtra*:
///
/// > 2.4.67 *na gōpavanādibhyaḥ*
pub(crate) const GOPAVANADI: GanapathaEntry = GanapathaEntry::basic(
    "gopavanAdiH",
    29,
    "2.4.67",
    &[
        "gOpavana",
        "Seyu",
        "Sipru",
        "bindu",
        "BAjana",
        "aSvAvatAna",
        "SyAmAka",
        "SyonAka",
        "SyAmaka",
        "SyAparRa",
        // (bidAdyantargaRo'yam [[4
        "1",
        "104]])",
    ],
);

/// *tikakitavādi-gaṇa* (30), first used in the following *sūtra*:
///
/// > 2.4.68 *tikakitavādibhyō dvandvē*
pub(crate) const TIKAKITAVADI: GanapathaEntry = GanapathaEntry::basic(
    "tikakitavAdiH",
    30,
    "2.4.68",
    &[
        "tikakitavAH",
        "vaNKaraBaRqIraTAH",
        "upakalamakAH",
        "paPakanarakAH",
        "bakanaKagudapariRadDAH",
        "ubjakakuBAH",
        "laNkaSAntamuKAH",
        "uttaraSalaNkawAH",
        "kfzRAjinakfzRasundarAH",
        "BrazwakakapizWalAH",
        "agniveSadaSerukAH",
    ],
);

/// *upakādi-gaṇa* (31), first used in the following *sūtra*:
///
/// > 2.4.69 *upakādibhyō'nyatarasyāmadvandvē*
pub(crate) const UPAKADI: GanapathaEntry = GanapathaEntry::basic(
    "upakAdiH",
    31,
    "2.4.69",
    &[
        "upakalamaka",
        "BrAzwraka",
        "kapizWala",
        "kfzRAjina",
        "kfzRasundara",
        "cUqAraka",
        "AqAraka",
        "gaquka",
        "udaNka",
        "suDAyuka",
        "avanDaka",
        "piNgalaka",
        "pizwaka",
        "supizwa",
        "supizWa",
        "mayUrakarRa",
        "KArIjaNGa",
        "SalATala",
        "pataYjala",
        "padaYjala",
        "kaWeraRi",
        "kuzItaka",
        "kASakftsna",
        "kASakftsna",
        "nidAGa",
        "kalaSIkaRWa",
        "dAmakaR",
        "kfzRapiNgala",
        "karRa",
        "parRaka",
        "jawiraka",
        "baDiraka",
        "jantuka",
        "anuloma",
        "anupada",
        "pratiloma",
        "apajagDa",
        "pratA",
        "anaBihita",
        "kamaka",
        "karAwaka",
        "leKAmra",
        "kamandaka",
        "piYjalaka",
        "karRaka",
        "mayUrakarRa",
        "nadAGa",
        "kavantaka",
        "kamantaka",
        "kadAmata",
        "dAmakaRWa",
    ],
);

/// *bhr̥śādi-gaṇa* (32), first used in the following *sūtra*:
///
/// > 3.1.12 *bhr̥śādibhyō bhuvyacvērlōpaśca halaḥ*
pub(crate) const BHRSHADI: GanapathaEntry = GanapathaEntry::basic(
    "BfSAdiH",
    32,
    "3.1.12",
    &[
        "BfSa",
        "SIGra",
        "capala",
        "manda",
        "paRqita",
        "utsuka",
        "sumanas",
        "durmanas",
        "aBimanas",
        "unmanas",
        "rahas",
        "rohat",
        "rehat",
        "saMScat",
        "tfpat",
        "SaSvat",
        "Bramat",
        "vehat",
        "Sucis",
        "Sucivarcas",
        "aRqara",
        "varcas",
        "ojas",
        "surajas",
        "arajas",
    ],
);

/// *lōhitādi-gaṇa* (33), first used in the following *sūtra*:
///
/// > 3.1.13 *lōhitādiḍājbhyaḥ kyaṣ*
pub(crate) const LOHITADI: GanapathaEntry = GanapathaEntry::akrti(
    "lohitAdiH",
    33,
    "3.1.13",
    &[
        "lohita", "carita", "nIla", "Pena", "madra", "hari", "dAsa", "man",
    ],
);

/// *sukhādi-gaṇa* (34), first used in the following *sūtra*:
///
/// > 3.1.18 *sukhādibhyaḥ kartr̥vēdanāyām*
pub(crate) const SUKHADI: GanapathaEntry = GanapathaEntry::basic(
    "suKAdiH",
    34,
    "3.1.18",
    &[
        "suKa", "duHKa", "tfpta", "kfcCra", "asra", "Asra", "aloka", "pratIpa", "karuRa", "kfpaRa",
        "so",
    ],
);

/// *kaṇḍvādi-gaṇa* (35), first used in the following *sūtra*:
///
/// > 3.1.27 *kaṇḍvādibhyō yak*
pub(crate) const KANDVADI: GanapathaEntry = GanapathaEntry::akrti(
    "kaRqvAdiH",
    35,
    "3.1.27",
    &[
        "kaRqUY", "mantu", "hfRIN", "valgu", "asu", "nasa", "mahIN", "low", "lew", "iras", "iraj",
        "iraY", "uvas", "uzas", "vew", "meDA", "kuzuBa", "namas", "magaDa", "tantas", "pampas",
        "papas", "suKa", "duHKa", "Bikza", "caraRa", "carama", "avara", "sapara", "arara", "arar",
        "Bizaj", "BizRuj", "apara", "Ara", "izuDa", "varaRa", "curaRa", "curaRa", "BuraRa",
        "gadgada", "elA", "kelA", "KelA", "velA", "SelA", "liw", "lAw", "leKA", "leKa", "reKA",
        "dravas", "tiras", "agada", "uras", "taraRa", "tariRa", "payas", "saMBUyas", "sambara",
    ],
);

/// *nandyādi-gaṇa* (36), first used in the following *sūtra*:
///
/// > 3.1.134 *nandigrahipacādibhyō lyuṇinyacaḥ*
pub(crate) const NANDYADI: GanapathaEntry = GanapathaEntry::basic(
    "nandyAdiH",
    36,
    "3.1.134",
    &[
        // <<nandivASimadidUzisADivarDiSociByo RyanteByaH saMjYAyAm>>
        "nandanaH",
        "vASanaH",
        "madanaH",
        "dUzaRaH",
        "sADanaH",
        "varDanaH",
        "SoBanaH",
        "rocanaH",
        // <<sahitapidameH saMjYAyAm>>
        "sahanaH",
        "tapanaH",
        "damanaH",
        "jalpanaH",
        "ramaRaH",
        "darpaRaH",
        "saNkrandanaH",
        "saNkarzaRaH",
        "saMharzaRaH",
        "janArdanaH",
        "yavanaH",
        "maDusUdanaH",
        "viBIzaRaH",
        "lavaRaH",
        "cittavinASanaH",
        "kuladamanaH",
        "SatrudamanaH",
    ],
);

/// *grahyādi-gaṇa* (37), first used in the following *sūtra*:
///
/// > 3.1.134 *nandigrahipacādibhyō lyuṇinyacaḥ*
pub(crate) const GRAHYADI: GanapathaEntry = GanapathaEntry::basic(
    "grahyAdiH",
    37,
    "3.1.134",
    &[
        "grAhI",
        "utsAhI",
        "uddAso",
        "udBAsI",
        "sTAyI",
        "mantrI",
        "saMmardI",
        // <<rakzaSruvasavapaSAM nO>>
        "nirakzI",
        "niSrAvI",
        "nivApO",
        "niSAyI",
        // <<yAcfvyAhfsaMvyAhfvrajavadavasAM pratizidDAnAm>>
        "ayAcI",
        "avyAhArI",
        "asaMvyAhArI",
        "avrAjI",
        "avAdI",
        "avAsI",
        // <<acAmacittakartfkARAM pratizidDAnAm>>
        "akArI",
        "ahArI",
        "avinAyI",
        "viSAyI",
        "vizAyI",
        // <<viSayI vizayI deSe>> viSayI  deSaH
        // vizayI  deSaH
        // <<aBiBAvI BUte>>
        "aparADI",
        "uparoDI",
        "pariBavI",
        "pariBAvI",
    ],
);

/// *pacādi-gaṇa* (38), first used in the following *sūtra*:
///
/// > 3.1.134 *nandigrahipacādibhyō lyuṇinyacaḥ*
pub(crate) const PACADI: GanapathaEntry = GanapathaEntry::akrti(
    "pacAdiH",
    38,
    "3.1.134",
    &[
        "paca", "vaca", "vapa", "vada", "cala", "pata", "nadaw", "Bazaw", "plavaw", "caraw",
        "garaw", "taraw", "coraw", "gAhaw", "saraw", "devaw", "dozaw", "jara", "raja", "mara",
        "mada", "kzama", "kzapa", "seva", "meza", "kopa", "koza", "meDA", "narta", "vraRa",
        "darSa", "sarpa", "damBa", "darpa", "jAraBara", "Svapaca",
    ],
);

/// *mūlavibhujādi-gaṇa* (39), first used in the following *sūtra*:
///
/// > 3.2.5 *tundaśōkayōḥ parimr̥jāpanudōḥ*
/// The *sūtra* has the following *vārttika*:
///
/// > *kaprakaraRe mUlaviBujAdiBya upasaMKyAnam*
pub(crate) const MULAVIBHUJADI: GanapathaEntry = GanapathaEntry::akrti(
    "mUlaviBujAdiH",
    39,
    "3.2.5",
    &[
        "mUlaviBuja",
        "naKamuca",
        "kAkaguha",
        "kumuda",
        "mahIDra",
        "kuDra",
        "giGra",
    ],
)
.with_varttika("kaprakaraRe mUlaviBujAdiBya upasaMKyAnam");

/// *pārśvādi-gaṇa* (40), first used in the following *sūtra*:
///
/// > 3.2.15 *adhikaraṇē śētēḥ*
/// The *sūtra* has the following *vārttika*:
///
/// > *pArSvAdizUpasaNKyAnam*
pub(crate) const PARSHVADI: GanapathaEntry = GanapathaEntry::basic(
    "pArSvAdiH",
    40,
    "3.2.15",
    &["pArSva", "udara", "pfzWa", "uttAna", "avamUrDana"],
)
.with_varttika("pArSvAdizUpasaNKyAnam");

/// *gamyādi-gaṇa* (41), first used in the following *sūtra*:
///
/// > 3.3.3 *bhaviṣyati gamyādayaḥ*
pub(crate) const GAMYADI: GanapathaEntry = GanapathaEntry::basic(
    "gamyAdiH",
    41,
    "3.3.3",
    &[
        "gamI",
        "AgamI",
        "BAvI",
        "prasTAyI",
        "pratiroDI",
        "pratiyoDI",
        "pratiboDI",
        "pratiyAyI",
        "pratiyogI",
    ],
);

/// *bhidādi-gaṇa* (42), first used in the following *sūtra*:
///
/// > 3.3.104 *ṣidbhidādibhyō'ṅ*
pub(crate) const BHIDADI: GanapathaEntry = GanapathaEntry::basic(
    "BidAdiH",
    42,
    "3.3.104",
    &[
        // <<BidA vidAraRe>>
        // <<CidA dveDIkaraRe>>
        "vidA", "kzipA", // <<guhA giryozaDyoH>>
        "SradDA", "meDA", "goDA",  // <<ArA SastryAm>>
        "hArA",  // <<kArA banDane>>
        "kziyA", // <<tArA jyotizi>>
        // <<DArA prapAtane>>
        "reKA", "cUqA", "pIqA", "vapA", "vasA", "sfjA", // <<krapeH saMprasAraRaM ca>>
        "kfpA",
    ],
);

/// *saṁpadādi-gaṇa* (43), first used in the following *sūtra*:
///
/// > 3.3.108 *rōgākhyāyāṁ ṇvul bahulam*
/// The *sūtra* has the following *vārttika*:
///
/// > *saMpadAdiByaH kvip*
pub(crate) const SAMPADADI: GanapathaEntry = GanapathaEntry::basic(
    "saMpadAdiH",
    43,
    "3.3.108",
    &["saMpad", "vipad", "Apad", "pratipad", "parizad"],
)
.with_varttika("saMpadAdiByaH kvip");

/// *bhīmādi-gaṇa* (44), first used in the following *sūtra*:
///
/// > 3.4.74 *bhīmādayō'pādānē*
pub(crate) const BHIMADI: GanapathaEntry = GanapathaEntry::akrti(
    "BImAdiH",
    44,
    "3.4.74",
    &[
        "BIma",
        "BIzma",
        "BayAnaka",
        "vahacara",
        "vahacaru",
        "praskandana",
        "pratapana",
        "prapatana",
        "samudra",
        "sruva",
        "sruk",
        "vfzwI",
        "vfzwI",
        "rakzaH",
        "saMkasuka",
        "SaNkusuka",
        "mUrKa",
        "Kalati",
    ],
);

/// *ajādi-gaṇa* (45), first used in the following *sūtra*:
///
/// > 4.1.4 *ajādyataṣṭāp*
pub(crate) const AJADI: GanapathaEntry = GanapathaEntry::basic(
    "ajAdiH",
    45,
    "4.1.4",
    &[
        "ajA",
        "eqakA",
        "kokilA",
        "cawakA",
        "aSvA",
        "mUzikA",
        "bAlA",
        "hoqA",
        "pAkA",
        "vatsA",
        "mandA",
        "vilAtA",
        "pUrvApihARA",
        "pUrvApahARA",
        "aparApahARA",
        // <<saMBastrAjinaSaRapiRqeByaH PalAt>>
        // <<sadackARqaprAntaSatEkeByaH puzpAt>>
        // <<SUdrA cAmahatpUrvA jAtiH>>
        "kruYcA",
        "uzRihA",
        "devaviSA",
        "jyezWA",
        "kanizWA",
        "maDyamA",
        "puMyoge'pi",
        // <<mUlAnnaYaH>>
        "daMzwrA",
    ],
);

/// *svasrādi-gaṇa* (46), first used in the following *sūtra*:
///
/// > 4.1.10 *na ṣaṭsvasrādibhyaḥ*
pub(crate) const SVASRADI: GanapathaEntry = GanapathaEntry::basic(
    "svasrAdiH",
    46,
    "4.1.10",
    &[
        "svasf", "duhitf", "nanAndf", "yAtf", "mAtf", "tisf", "catasf",
    ],
);

/// *samānādi-gaṇa* (47), first used in the following *sūtra*:
///
/// > 4.1.35 *nityaṁ sapatnyādiṣu*
pub(crate) const SAMANADI: GanapathaEntry = GanapathaEntry::basic(
    "samAnAdiH",
    47,
    "4.1.35",
    &[
        "samAna", "eka", "vIra", "piRqa", "Sva", "SirI", "BrAtf", "Badra",
        "putra",
        // <<dAsAcCandasi>>
    ],
);

/// *gaurādi-gaṇa* (48), first used in the following *sūtra*:
///
/// > 4.1.41 *ṣidgaurādibhyaśca*
pub(crate) const GAURADI: GanapathaEntry = GanapathaEntry::basic(
    "gOrAdiH",
    48,
    "4.1.41",
    &[
        "gOra",
        "matsya",
        "manuzya",
        "SfNga",
        "piNgala",
        "haya",
        "gavaya",
        "mukaya",
        "fzya",
        "puwa",
        "dURa",
        "druRa",
        "droRa",
        "hariRa",
        "kokaRa",
        "kAkaRa",
        "pawara",
        "uRaka",
        "Amala",
        "Amalaka",
        "kubala",
        "bimba",
        "badara",
        "Parkaraka",
        "karkaraka",
        "tarkAra",
        "SarkAra",
        "puzkara",
        "SiKaRqa",
        "sala",
        "SazkaRqa",
        "sananda",
        "suzama",
        "suzava",
        "alinda",
        "gaqu",
        "zARqaSa",
        "AQaka",
        "Ananda",
        "ASvatTa",
        "sfpAwa",
        "AKaka",
        "Apaccika",
        "Sazkula",
        "sUrya",
        "sUrma",
        "SUrpa",
        "sUpa",
        "vfsa",
        "atasa",
        "Ba",
        "BfNga",
        "maha",
        "maWa",
        "Ceda",
        "peSa",
        "meda",
        "Svan",
        "takzan",
        "anaquhI",
        "anaqvAhI",
        // <<ezaRaH karaRe>>
        "deha",
        "dehala",
        "kAkAdana",
        "gavAdana",
        "tejana",
        "rajana",
        "lavaRa",
        "OdgAhamAni",
        "AdgAhamAni",
        "gOtama",
        "gotama",
        "pAraka",
        "AyasTURa",
        "ayaHsTURa",
        "BOrika",
        "BOliki",
        "BOliNgi",
        "yAna",
        "meGa",
        "Alambi",
        "Alaji",
        "AlabDi",
        "Alakzi",
        "kevAla",
        "Apaka",
        "Arawa",
        "nawa",
        "wowa",
        "nowa",
        "mUlAwa",
        "SAtana",
        "potana",
        "pAtana",
        "pAWana",
        "pAnaWa",
        "AstaraRa",
        "aDikaraRa",
        "aDikAra",
        "AgrahAyaRI",
        "AgrahAyaRI",
        "pratyavarohiRI",
        "secana",
        // <<sumaNgalAtsaMjYAyAm>>
        "aRqara",
        "sundara",
        "maRqala",
        "manTara",
        "maNgala",
        "pawa",
        "piRwa",
        "zaRqa",
        "urda",
        "garda",
        "Sama",
        "sUd",
        "Oqa",
        "Ardra",
        "hfda",
        "hrada",
        "pARqa",
        "BARqala",
        "BARqa",
        "lohARqa",
        "kadara",
        "kandara",
        "kadala",
        "taruRa",
        "taluna",
        "kalmAza",
        "bfhat",
        "mahat",
        "soma",
        "sODarma",
        // <<rohiRI nakzatre>>
        // <<revatI nakzatre>>
        "vikala",
        "nizPala",
        "puzkala",
        // <<kawAcCroRivacane>>
        // <<pippalyAdayaSca>>
        "pippalI",
        "haritakI",
        "harItakI",
        "koSAtakI",
        "SamI",
        "varI",
        "SarI",
        "pfTivI",
        "krozwu",
        "mAtAmaha",
        "pitAmaha",
    ],
);

/// *bahvādi-gaṇa* (49), first used in the following *sūtra*:
///
/// > 4.1.45 *bahvādibhyaśca*
pub(crate) const BAHVADI: GanapathaEntry = GanapathaEntry::akrti(
    "bahvAdiH",
    49,
    "4.1.45",
    &[
        "bahu",
        "padDati",
        "aYcati",
        "aNkati",
        "aMhata",
        "Sakawi",
        "Sakati",
        "SaktiH",
        "Sastre",
        "SAri",
        "vAri",
        "rAti",
        "rADi",
        "SADi",
        "ahi",
        "kapi",
        "yazwi",
        "muni",
        // <<itaH prARyaNgAt>>
        // <<kfdikArAdaktinaH>>
        // <<sarvato'ktinnarTAdityeke>>
        "caRqa",
        "arAla",
        "kfpARa",
        "kamala",
        "vikawa",
        "viSAla",
        "viSaNkawa",
        "Baruja",
        "Dvaja",
        "candraBAgAnadyAm",
        // candraBAgA nadyAm
        "kalyARa",
        "udAra",
        "purARa",
        "ahan",
        "koqa",
        "naKa",
        "SiKA",
        "bAla",
        "SaPa",
        "guda",
        "Baga",
        "gala",
        "rAga",
    ],
);

/// *krōḍādi-gaṇa* (50), first used in the following *sūtra*:
///
/// > 4.1.56 *na krōḍādibahvacaḥ*
pub(crate) const KRODADI: GanapathaEntry = GanapathaEntry::akrti(
    "kroqAdiH",
    50,
    "4.1.56",
    &[
        "kroqA", "KurA", "UKA", "bAlA", "SaPA", "gudA", "GoRA", "naKA", "muKA", "suBagA", "sugalA",
    ],
);

/// *śārṅgaravādi-gaṇa* (51), first used in the following *sūtra*:
///
/// > 4.1.73 *śārṅgaravādyañō ṅīn*
pub(crate) const SHARNGARAVADI: GanapathaEntry = GanapathaEntry::basic(
    "SArNgaravAdiH",
    51,
    "4.1.73",
    &[
        "SArNgarava",
        "kApawava",
        "gOggulava",
        "brAhmaRa",
        "bEda",
        "gOta",
        "kAmaRqaleya",
        "brAhamakfteya",
        "nice",
        "AniDe",
        "ASokeya",
        "vAtsyAyana",
        "mOYjAyana",
        "kEkasa",
        "kApya",
        "kAvya",
        "SEvya",
        "ehi",
        "paryehi",
        "ASmaraTya",
        "OdapAna",
        "arAla",
        "caRqAla",
        "vataRqa",
        // <<BogavadgOrimatoH saMjYAyAM GAdizu ([[6
        "3",
        // 43]]) nityaM hrasvArTam>>
        // <<mfmarayorvfdDiSca>>
    ],
);

/// *krauḍyādi-gaṇa* (52), first used in the following *sūtra*:
///
/// > 4.1.80 *krauḍyādibhyaśca*
pub(crate) const KRAUDYADI: GanapathaEntry = GanapathaEntry::basic(
    "krOqyAdiH",
    52,
    "4.1.80",
    &[
        "kOqi",
        "lAqi",
        "vyAqi",
        "ApiSali",
        "Apakziti",
        "cOpayata",
        "cEwayata",
        "vEwayata",
        "sEkayata",
        "bElvayata",
        "sODAtaki",
        "sUtayuvatyAm",
        "Boja",
        "kzatriye",
        "yOtaki",
        "kOwi",
        "BOriki",
        "BOliki",
        "lmala",
        "SAlAsTali",
        "kApizWali",
        "gOkakzya",
    ],
);

/// *aśvapatyādi-gaṇa* (53), first used in the following *sūtra*:
///
/// > 4.1.84 *aśvapatyādibhyaśca*
pub(crate) const ASHVAPATYADI: GanapathaEntry = GanapathaEntry::basic(
    "aSvapatyAdiH",
    53,
    "4.1.84",
    &[
        "aSvapati",
        "jYAnapati",
        "Satapati",
        "Danapati",
        "gaRapati",
        "sTAnapati",
        "yajYapata",
        "rAzwrapati",
        "kulapati",
        "gfhapati",
        "Supata",
        "DAnyapati",
        "Danvapati",
        "nDupati",
        "Darmapata",
        "saBApati",
        "prARapati",
        "kzetrapati",
    ],
);

/// *utsādi-gaṇa* (54), first used in the following *sūtra*:
///
/// > 4.1.86 *utsādibhyō'ñ*
pub(crate) const UTSADI: GanapathaEntry = GanapathaEntry::basic(
    "utsAdiH",
    54,
    "4.1.86",
    &[
        "utsa",
        "udapAna",
        "vikara",
        "vinada",
        "mahAnada",
        "mahAnasa",
        "mahAprARa",
        "taruRa",
        "taluna",
        // <<bazkayAse>>
        "pfTivI",
        "Denu",
        "paNkti",
        "jagatI",
        "trizwup",
        "anuzwup",
        "janapada",
        "Barata",
        "uSInara",
        "grIzma",
        "pIlukuRa",
        // <<udasTAna deSe>>
        "pfzadaMSa",
        "BallakIya",
        "raTaMtara",
        "maDyaMdina",
        "bfhat",
        "mahat",
        "sattvat",
        "kuru",
        "paYcAla",
        "indrAvasAna",
        "uzRih",
        "kakuB",
        "suvarRa",
        "deva",
        // <<grIzmAdacCandasi>>
    ],
);

/// *bāhvādi-gaṇa* (55), first used in the following *sūtra*:
///
/// > 4.1.96 *bāhvādibhyaśca*
pub(crate) const BAAHVADI: GanapathaEntry = GanapathaEntry::akrti(
    "bAhvAdiH",
    55,
    "4.1.96",
    &[
        "bAhu",
        "upabAhu",
        "upavAku",
        "nivAku",
        "SivAku",
        "vawAku",
        "upanindu",
        "pabinda",
        "vfzalI",
        "vfkalA",
        "cUqA",
        "balAkA",
        "mUzikA",
        "kuSalA",
        "BagalA",
        "CagalA",
        "GruvakA",
        "DuvakA",
        "sumitrA",
        "durmitrA",
        "puzkarasad",
        "anuharat",
        "devaSarman",
        "agniSarman",
        "draSarmana",
        "suSarman",
        "kunAman",
        "sunAman",
        "paYcan",
        "saptan",
        "azwan",
        "amitOjasaH",
        "salopaSca",
        "suDAvat",
        "udaYcu",
        "Siras",
        "mAza",
        "SarAvin",
        "kzemavfdDin",
        "SfNKalatodin",
        "KaranAdin",
        "nagaramardin",
        "prAkAramardin",
        "loman",
        "ajIgarta",
        "kfzRa",
        "yuDizWira",
        "arjuna",
        "sAmba",
        "gada",
        "pradyumna",
        "rAma",
        "udaNka",
        // <<udakaH saMjYAyAm>>
        // <<saMBUyomBasoH salopaSca>>
        "sAttvakiH",
        "jANGiH",
        "EndaSarmiH",
        "AjaDenaviH",
    ],
);

/// *kuñjādi-gaṇa* (56), first used in the following *sūtra*:
///
/// > 4.1.98 *gōtrē kuñjādibhyaścphañ*
pub(crate) const KUNJADI: GanapathaEntry = GanapathaEntry::basic(
    "kuYjAdiH",
    56,
    "4.1.98",
    &[
        "kuYja", "braDna", "SaNKa", "Basman", "gaRa", "lomana", "SaWa", "SAka", "SuRqA", "SuBa",
        "vipASa", "skanda", "skamBa",
    ],
);

/// *naḍādi-gaṇa* (57), first used in the following *sūtra*:
///
/// > 4.1.99 *naḍādibhyaḥ phak*
pub(crate) const NADADI: GanapathaEntry = GanapathaEntry::basic(
    "naqAdiH",
    57,
    "4.1.99",
    &[
        "naqa",
        "cara",
        "vara",
        "baka",
        "muYja",
        "itika",
        "itiSa",
        "upaka",
        "eka",
        "lamaka",
        // <<SalaNku SalaNkaM ca>>
        "saptala",
        "vAjapya",
        "tika",
        // <<agniSarmanvfSagaRe>>
        "prARa",
        "nara",
        "sAyaka",
        "dAsa",
        "mitra",
        "dvIpa",
        "piNgara",
        "piNgala",
        "kiNkara",
        "kiNkala",
        "kAtara",
        "kAtala",
        "kASyapa",
        "kuSyapa",
        "kASya",
        "kAlya",
        "kAvya",
        "aja",
        "amuzya",
        "amuzma",
        // <<kfzRaraRO brAhmaRavAsizWe>>
        "amitra",
        "ligu",
        "citra",
        "kumAra",
        // <<krozwu krozwaM ca>>
        "loha",
        "durga",
        "stamBa",
        "SiMSapA",
        "agra",
        "tfRa",
        "Sakawa",
        "sumanas",
        "sumata",
        "mimata",
        "fc",
        "jalaMDara",
        "aDvara",
        "yugaMDara",
        "haMsaka",
        "daRqin",
        "hastin",
        "iR",
        "paYcAla",
        "camasin",
        "sukftya",
        "sTiraka",
        "brAhmaRa",
        "cawaka",
        "badara",
        "aSvala",
        "Karapa",
        "laNka",
        "inDa",
        "astra",
        "kAmuka",
        "vrahmadatta",
        "udumbara",
        "SoRa",
        "aloha",
        "daRqapa",
    ],
);

/// *bidādi-gaṇa* (58), first used in the following *sūtra*:
///
/// > 4.1.104 *anr̥ṣyānantaryē bidādibhyō'ñ*
pub(crate) const BIDADI: GanapathaEntry = GanapathaEntry::basic(
    "bidAdiH",
    58,
    "4.1.104",
    &[
        "bida",
        "urva",
        "kaSyapa",
        "kuSika",
        "BaradvAja",
        "upamanyu",
        "kilAta",
        "kandarpa",
        "kidarBa",
        "viSvAnara",
        "fzizeRa",
        "fzwizeRa",
        "ftaBAga",
        "haryaSva",
        "priyaka",
        "Apastamba",
        "kUcavAra",
        "Saradvat",
        "Sunaka",
        "Sunak",
        "Denu",
        "gopavana",
        "Sigru",
        "bindu",
        "oga",
        "BAjana",
        "Samika",
        "aSvAvatAna",
        "SyAmAka",
        "SyAmaka",
        "SyAvali",
        "SyAparRa",
        "harita",
        "kidAsa",
        "vahyaska",
        "arkalUza",
        "vaDyoga",
        "vizRuvfdDa",
        "pratiboDa",
        "racita",
        "raTantara",
        "gavizWira",
        "nizAda",
        "Sabra",
        "alasa",
        "maWara",
        "mfqAku",
        "sfpAku",
        "mfda",
        "punarBU",
        "putra",
        "duhitf",
        "nanAndf",
        // <<parastrI paraSuM ca>>
    ],
);

/// *gargādi-gaṇa* (59), first used in the following *sūtra*:
///
/// > 4.1.105 *gargādibhyō yañ*
pub(crate) const GARGADI: GanapathaEntry = GanapathaEntry::basic(
    "gargAdiH",
    59,
    "4.1.105",
    &[
        "garga",
        "vatsa",
        "vAjAse",
        "saMkfti",
        "aja",
        "vyAGrapAt",
        "vidaBft",
        "prAcInayoga",
        "agasti",
        "pulasti",
        "reBa",
        "agniveSa",
        "SaNKa",
        "Sawa",
        "Saka",
        "eka",
        "GUma",
        "avawa",
        "manas",
        "DanaMjaya",
        "viSvAvasu",
        "jaramARa",
        "lohita",
        "SaMsita",
        "baBru",
        "valgu",
        "maRqu",
        "SaNku",
        "ligu",
        "guhalu",
        "mantu",
        "maNkzu",
        "aligu",
        "jigIzu",
        "manu",
        "tantu",
        "manAyIsUnu",
        "kaTaka",
        "kanTaka",
        "fkza",
        "tfkza",
        "vfkza",
        "tanu",
        "tarukza",
        "talukza",
        "taRqa",
        "vataRqa",
        "kapikata",
        "kapi",
        "kata",
        "kurukata",
        "anaquh",
        "kaRva",
        "Sakala",
        "gokakza",
        "agastya",
        "kaRqinI",
        "yajYavalka",
        "parRavalka",
        "aBayajAta",
        "virohita",
        "vfzagaRa",
        "rahUgaRa",
        "SaRqila",
        "varRaka",
        "caRaka",
        "culuka",
        "mudgala",
        "musala",
        "jamadagni",
        "parASara",
        "jatUkarRa",
        "jAtUkarRa",
        "mahita",
        "mantrita",
        "aSmaraTa",
        "SarkarAkza",
        "pUtimAza",
        "sTUrA",
        "adaraka",
        "araraka",
        "elAka",
        "piNgala",
        "kfzRa",
        "golanda",
        "ulUka",
        "titikza",
        "Bizaja",
        "Bizaj",
        "BizRaja",
        "Baqita",
        "BaRqita",
        "dalBa",
        "cekita",
        "cikitsita",
        "devahU",
        "indrahU",
        "ekalu",
        "pippalu",
        "bfhadagni",
        "sulohina",
        "sulABin",
        "ukTa",
        "kuwIgu",
    ],
);

/// *aśvādi-gaṇa* (60), first used in the following *sūtra*:
///
/// > 4.1.110 *aśvādibhyaḥ phañ*
pub(crate) const ASHVADI: GanapathaEntry = GanapathaEntry::basic(
    "aSvAdiH",
    60,
    "4.1.110",
    &[
        "aSva",
        "aSman",
        "SaNKa",
        "SUdaka",
        "vida",
        "puwa",
        "rohiRa",
        "KarjUra",
        "KajUra",
        "YjAra",
        "vas",
        "pijUra",
        "Baqila",
        "BaRqila",
        "Baqita",
        "BaRqita",
        "prakfta",
        "rAmoda",
        "kzAnta",
        "kASa",
        "tIkzRa",
        "golANka",
        "arka",
        "svara",
        "sPuwa",
        "cakra",
        "Sraviz",
        "pavinda",
        "pavitra",
        "gomin",
        "SyAma",
        "DUma",
        "DUmra",
        "vAgmin",
        "viSvAnara",
        "kuwa",
        "Sapa",
        "atreye",
        "jana",
        "jaqa",
        "Kaqa",
        "grIzma",
        "arha",
        "kita",
        "viMSapa",
        "viSAla",
        "giri",
        "capala",
        "cupa",
        "dAsaka",
        "vElya",
        "bElva",
        "prAcya",
        "Darmya",
        "Anaquhya",
        // <<puMsi jAte>>
        "arjuna",
        "prahfta",
        "sumanas",
        "durmanas",
        "mana",
        "manana",
        "prAnta",
        "Dvana",
        "Atreya",
        "BaradvAje",
        "BaradvAja",
        "Atreye",
        "itsa",
        "Atava",
        "kitava",
        "vada",
        "Danya",
        "pAda",
        "Siva",
        "Kadira",
    ],
);

/// *śivādi-gaṇa* (61), first used in the following *sūtra*:
///
/// > 4.1.112 *śivādibhyō'ṇ*
pub(crate) const SHIVADI: GanapathaEntry = GanapathaEntry::akrti(
    "SivAdiH",
    61,
    "4.1.112",
    &[
        "Siva",
        "prozWa",
        "prozWika",
        "caRqa",
        "jamBa",
        "BUri",
        "daRqa",
        "kuWAra",
        "kakuB",
        "kakuBA",
        "anaBimlAna",
        "kohita",
        "suKa",
        "saMDi",
        "muni",
        "kakutsTa",
        "kahoqa",
        "kohaqa",
        "kahUya",
        "kahaya",
        "roda",
        "kapiYjala",
        "kupiYjala",
        "KaYjana",
        "vataRqa",
        "tfRakarRa",
        "kzIrahrada",
        "jalahrada",
        "parila",
        "paTika",
        "pizwa",
        "hEhaya",
        "Arzika",
        "gopikA",
        "kapilikA",
        "jawilikA",
        "aleKana",
        "viSravaRa",
        "ravaRa",
        "vartanAkza",
        "grIvAkza",
        "piwaka",
        "viwapa",
        "piwAka",
        "tfkzAka",
        "naBaka",
        "UrRanABa",
        "caratkAru",
        "pfTA",
        "utkzepa",
        "purohitikA",
        "surohitikA",
        "surohikA",
        "AryaSveta",
        "aryaSveta",
        "supizwa",
        "masurakarRa",
        "mayUrakarRa",
        "rjUrakar",
        "kadUraka",
        "takzan",
        "fzwizeRa",
        "gaNgA",
        "vipASa",
        "maska",
        "lahya",
        "druhya",
        "ayasTURa",
        "tfRakarRa",
        "tfRa",
        "karRa",
        "parRa",
        "Balandana",
        "virUpAkza",
        "BUmi",
        "ilA",
        "sapatnI",
        // <<dvyaco nadyAH>>
        // <<triveRI trivaRaM ca>>
    ],
);

/// *śubhrādi-gaṇa* (62), first used in the following *sūtra*:
///
/// > 4.1.123 *śubhrādibhyaśca*
pub(crate) const SHUBHRADI: GanapathaEntry = GanapathaEntry::akrti(
    "SuBrAdiH",
    62,
    "4.1.123",
    &[
        "SuBra",
        "vizwa",
        "pura",
        "vizwapura",
        "brahmakfta",
        "SatadvAra",
        "SalATala",
        "SalAkABrU",
        "leKABrU",
        "leKAmra",
        "vikasA",
        "vikAsa",
        "rohiRI",
        "rukmiRI",
        "DarmiRI",
        "diS",
        "SAlUka",
        "ajabasti",
        "SakaMDi",
        "vimAtf",
        "viDavA",
        "Suka",
        "viSa",
        "devatara",
        "Sakuni",
        "Suka",
        "amra",
        "jYAtala",
        "Satala",
        "banDakI",
        "sfkaRqu",
        "visri",
        "atiTi",
        "godanta",
        "kuSAmba",
        "makuRqa",
        "SAtAhara",
        "pavazwurika",
        "sunAman",
        "lakzmaRaSyAmayorvAsizWe",
        "goDA",
        "kfkalAsa",
        "aRIva",
        "pravAhaRa",
        "Barata",
        "BArata",
        "Barama",
        "mukaRqa",
        "karpUra",
        "itara",
        "anyatara",
        "AliQa",
        "sudanta",
        "sucakza",
        "sucakzas",
        "sudAmana",
        "kadru",
        "tuda",
        "akaSAya",
        "kumArIkA",
        "kuWArikA",
        "kiSorikA",
        "ambikA",
        "jihmASin",
        "pariDi",
        "vAyudatta",
        "Sakala",
        "SalAkA",
        "KaqUra",
        "kuberikA",
        "aSokA",
        "ganDapiNgalA",
        "KaqonmattA",
        "anudfzwin",
        "anudfzwi",
        "jaratin",
        "balIvardin",
        "vipra",
        "vIja",
        "jIva",
        "Svana",
        "aSman",
        "aSva",
        "ajira",
    ],
);

/// *kalyāṇyādi-gaṇa* (63), first used in the following *sūtra*:
///
/// > 4.1.126 *kalyāṇyādīnāminaṅ*
pub(crate) const KALYANYADI: GanapathaEntry = GanapathaEntry::basic(
    "kalyARyAdiH",
    63,
    "4.1.126",
    &[
        "kalyARI",
        "suBagA",
        "durBagA",
        "banDakI",
        "anudfzwi",
        "anusfzwi",
        "jaratI",
        "balIvardI",
        "jyezWA",
        "kanizWA",
        "maDyamA",
        "parastrI",
    ],
);

/// *gr̥ṣṭyādi-gaṇa* (64), first used in the following *sūtra*:
///
/// > 4.1.136 *gr̥ṣṭyādibhyaśca*
pub(crate) const GRSHTYADI: GanapathaEntry = GanapathaEntry::basic(
    "gfzwyAdiH",
    64,
    "4.1.136",
    &[
        "gfzwi", "hfzwi", "bali", "hali", "viSri", "kudri", "ajabasti", "mitrayu",
    ],
);

/// *rēvatyādi-gaṇa* (65), first used in the following *sūtra*:
///
/// > 4.1.146 *rēvatyādibhyaṣṭhak*
pub(crate) const REVATYADI: GanapathaEntry = GanapathaEntry::basic(
    "revatyAdiH",
    65,
    "4.1.146",
    &[
        "revatI",
        "aSvapAlI",
        "maRipAlI",
        "dvArapAlI",
        "vfkavaYcin",
        "vfkabanDu",
        "vfkagrAha",
        "karRagrAha",
        "daRqagrAha",
        "kukkuwAkza",
        "kakudAkza",
        "cAmaragrAha",
    ],
);

/// *kurvādi-gaṇa* (66), first used in the following *sūtra*:
///
/// > 4.1.151 *kurvādibhyō ṇyaḥ*
pub(crate) const KURVADI: GanapathaEntry = GanapathaEntry::basic(
    "kurvAdiH",
    66,
    "4.1.151",
    &[
        "kuru",
        "gargara",
        "maNguza",
        "ajamAra",
        "raTakAra",
        "vAvadUka",
        "samrAjaH",
        "kzatriye",
        "kavi",
        "mati",
        "vimati",
        "kApiYjalAdi",
        "vAk",
        "vAmaraTa",
        "pitfmat",
        "indrajAlA",
        "eji",
        "vAtaki",
        "dAmozRIzi",
        "gaRakAri",
        "kESori",
        "kuwa",
        "SAlAkA",
        "SalAkA",
        "mura",
        "pura",
        "erakA",
        "SuBra",
        "aBra",
        "darBa",
        "keSinI",
        // <<vinAcCandasi>>
        "SUrpaRAya",
        "SyAvanAya",
        "SyAvaraTa",
        "SyAvaputra",
        "satyaNkAra",
        "vaqaBIkAra",
        "paTikAra",
        "mU",
        "SakanDu",
        "SaNku",
        "SAka",
        "SAlin",
        "kartf",
        "hartf",
        "ina",
        "piRqI",
        "takzan",
        // <<vAmaraTasya kaRvAdivatsvaravarjam>>
    ],
);

/// *tikādi-gaṇa* (67), first used in the following *sūtra*:
///
/// > 4.1.154 *tikādibhyaḥ phiñ*
pub(crate) const TIKADI: GanapathaEntry = GanapathaEntry::basic(
    "tikAdiH",
    67,
    "4.1.154",
    &[
        "tika",
        "kitava",
        "saMjYA",
        "bAlA",
        "SiKA",
        "uras",
        "SAWya",
        "sEnDava",
        "yamunda",
        "rUpya",
        "grAmya",
        "nIla",
        "amitra",
        "gokakzya",
        "gOkakzya",
        "kuru",
        "devaraTa",
        "tEtila",
        "Orasa",
        "kOravya",
        "BEriki",
        "BOliki",
        "cOpayata",
        "cEtayata",
        "cEwayata",
        "SEkayata",
        "kzEtayata",
        "vAjavata",
        "candramas",
        "SuBa",
        "gaNgA",
        "vareRya",
        "supAman",
        "ArabDa",
        "vAhyaka",
        "svalpaka",
        "vfza",
        "lomaka",
        "udranya",
        "yajYa",
    ],
);

/// *vākinādi-gaṇa* (68), first used in the following *sūtra*:
///
/// > 4.1.158 *vākinādīnāṁ kuk ca*
pub(crate) const VAKINADI: GanapathaEntry = GanapathaEntry::basic(
    "vAkinAdiH",
    68,
    "4.1.158",
    &[
        "vAkina", "gADera", "kArkaSa", "kAka", "laMkA",
        // <<carmivarmiRornalopaSca>>
    ],
);

/// *kambōjādi-gaṇa* (69), first used in the following *sūtra*:
///
/// > 4.1.175 *kambōjālluk*
pub(crate) const KAMBOJADI: GanapathaEntry = GanapathaEntry::basic(
    "kambojAdiH",
    69,
    "4.1.175",
    &["kamboja", "cola", "kerala", "Saka", "yavana"],
);

/// *bhargādi-gaṇa* (70), first used in the following *sūtra*:
///
/// > 4.1.178 *na prācyabhargādiyaudhēyādibhyaḥ*
pub(crate) const BHARGADI: GanapathaEntry = GanapathaEntry::basic(
    "BargAdiH",
    70,
    "4.1.178",
    &[
        "Barga", "karUza", "kekaya", "kaSmIra", "sAlva", "susTAla", "uras", "kOravya",
    ],
);

/// *yaudhēyādi-gaṇa* (71), first used in the following *sūtra*:
///
/// > 4.1.178 *na prācyabhargādiyaudhēyādibhyaḥ*
pub(crate) const YAUDHEYADI: GanapathaEntry = GanapathaEntry::basic(
    "yODeyAdiH",
    71,
    "4.1.178",
    &[
        "yODeya",
        "SOkreya",
        "SOBreya",
        "jyAvAReya",
        "DOrteya",
        "DArteya",
        "trigarta",
        "Barata",
        "uSInara",
    ],
);

/// *bhikṣādi-gaṇa* (72), first used in the following *sūtra*:
///
/// > 4.2.38 *bhikṣādibhyō'ṇ*
pub(crate) const BHIKSHADI: GanapathaEntry = GanapathaEntry::basic(
    "BikzAdiH",
    72,
    "4.2.38",
    &[
        "BikzAgarBiRI",
        "kzetra",
        "karIza",
        "aNkAra",
        "aNgAra",
        "carmin",
        "Darmin",
        "sahasra",
        "yuvati",
        "padAti",
        "padDati",
        "aTarvan",
        "dakziRA",
        "BUta",
        "vizaya",
        "Srotra",
    ],
);

/// *khaṇḍikādi-gaṇa* (73), first used in the following *sūtra*:
///
/// > 4.2.45 *khaṇḍikādibhyaśca*
pub(crate) const KHANDIKADI: GanapathaEntry = GanapathaEntry::basic(
    "KaRqikAdiH",
    73,
    "4.2.45",
    &[
        "KaRqikA",
        "vaqavA",
        // <<kzudrakaqAlavAt senAsaMjYAyAm>>
        // <<kzudrakamAlavAt senAsaMjYAyAm>>
        "Bikzuka",
        "Suka",
        "ulUka",
        "Svan",
        "ahan",
        "yugavaratrA",
        "yugavaratra",
        "halabanDA",
        "halabanDa",
    ],
);

/// *pāśādi-gaṇa* (74), first used in the following *sūtra*:
///
/// > 4.2.49 *pāśādibhyō yaḥ*
pub(crate) const PASHADI: GanapathaEntry = GanapathaEntry::basic(
    "pASAdiH",
    74,
    "4.2.49",
    &[
        "pASa", "tfRa", "DUma", "vAta", "aNgAra", "pAwala", "pota", "gala", "piwaka", "piwAka",
        "Sakawa", "hala", "nawa", "vana",
    ],
);

/// *khalādi-gaṇa* (75), first used in the following *sūtra*:
///
/// > 4.2.51 *initrakaṭyacaśca*
/// The *sūtra* has the following *vārttika*:
///
/// > *KalAdiBya inirvaktavyaH*
pub(crate) const KHALADI: GanapathaEntry = GanapathaEntry::akrti(
    "KalAdiH",
    75,
    "4.2.51",
    &["Kala", "qAka", "kuwumba", "SAka", "kuRqalinI"],
)
.with_varttika("KalAdiBya inirvaktavyaH");

/// *rājanyādi-gaṇa* (76), first used in the following *sūtra*:
///
/// > 4.2.53 *rājanyādibhyō vuñ*
pub(crate) const RAJANYADI: GanapathaEntry = GanapathaEntry::akrti(
    "rAjanyAdiH",
    76,
    "4.2.53",
    &[
        "rAjanya",
        "Anfta",
        "bABravya",
        "SAlaNkAyana",
        "dEvayAtava",
        "dEvayAta",
        "vrIqa",
        "varatra",
        "jAlaMDarAyaRa",
        "rAjAyana",
        "telu",
        "AtmakAmeya",
        "ambarIzaputra",
        "vasAti",
        "bElvavana",
        "SElUza",
        "udumbara",
        "tIvra",
        "bElvala",
        "ArjunAyana",
        "saMpriya",
        "dakzi",
        "UrRanABa",
    ],
);

/// *bhaurikyādi-gaṇa* (77), first used in the following *sūtra*:
///
/// > 4.2.54 *bhaurikyādyaiṣukāryādibhyō vidhalbhaktalau*
pub(crate) const BHAURIKYADI: GanapathaEntry = GanapathaEntry::basic(
    "BOrikyAdiH",
    77,
    "4.2.54",
    &[
        "BOriki",
        "vEpeya",
        "BOliki",
        "cEpayata",
        "cEwayata",
        "kAReya",
        "vARijaka",
        "vARikAjya",
        "vAlikAjya",
        "sEkayata",
        "vEkayata",
    ],
);

/// *aiṣukāryādi-gaṇa* (78), first used in the following *sūtra*:
///
/// > 4.2.54 *bhaurikyādyaiṣukāryādibhyō vidhalbhaktalau*
pub(crate) const AISHUKARYADI: GanapathaEntry = GanapathaEntry::basic(
    "EzukAryAdiH",
    78,
    "4.2.54",
    &[
        "EzukAri",
        "sArasyAyana",
        "sArasAyana",
        "cAndrAyaRa",
        "dvyAkzAyaRa",
        "tryAkzAyaRa",
        "OqAyana",
        "jOlAyana",
        "KAqAyana",
        "dAsamitri",
        "dAsamitrAyaRa",
        "SOdrAyaRa",
        "dAkzAyaRa",
        "SayaRqAyana",
        "SAyaRqAyana",
        "tArkzyAyaRa",
        "SOBrAyaRa",
        "sOvIra",
        "sOvIrAyaRa",
        "SapaRqa",
        "SayaRqa",
        "SORqa",
        "SayARqi",
        "SayARqa",
        "vESvamAnava",
        "vESvaDyenava",
        "vESvaDenava",
        "naqa",
        "tuRqadeva",
        "viSvadeva",
        "sApiRqi",
    ],
);

/// *ukthādi-gaṇa* (79), first used in the following *sūtra*:
///
/// > 4.2.60 *kratūkthādisūtrāntāṭṭhak*
pub(crate) const UKTHADI: GanapathaEntry = GanapathaEntry::basic(
    "ukTAdiH",
    79,
    "4.2.60",
    &[
        "ukTa",
        "lokAyata",
        "nyAsa",
        "nyAya",
        "punarukta",
        "nirukta",
        "nimitta",
        "dvipadA",
        "jyotiza",
        "anupada",
        "anukalpa",
        "yajYa",
        "Darma",
        "carcA",
        "ametara",
        "Slakza",
        "SlakzRa",
        "saMhitA",
        "padakrama",
        "saGawa",
        "saMGawwa",
        "vftti",
        "parizad",
        "saMgraha",
        "gaRa",
        "guRa",
        "Ayurdeva",
        "Ayurveda",
    ],
);

/// *kramādi-gaṇa* (80), first used in the following *sūtra*:
///
/// > 4.2.61 *kramādibhyō vun*
pub(crate) const KRAMADI: GanapathaEntry = GanapathaEntry::basic(
    "kramAdiH",
    80,
    "4.2.61",
    &["krama", "pada", "SikzA", "mImAMsA", "sAman"],
);

/// *vasantādi-gaṇa* (81), first used in the following *sūtra*:
///
/// > 4.2.63 *vasantādibhyaṣṭhak*
pub(crate) const VASANTADI: GanapathaEntry = GanapathaEntry::basic(
    "vasantAdiH",
    81,
    "4.2.63",
    &[
        "vasanta", "pIzma", "varzA", "Sarad", "Sarat", "hemanta", "SiSira", "praTama", "guRa",
        "carama", "anuguRa", "aTarvan", "ATarvaR",
    ],
);

/// *saṅkalādi-gaṇa* (82), first used in the following *sūtra*:
///
/// > 4.2.75 *saṁkalādibhyaśca*
pub(crate) const SANKALADI: GanapathaEntry = GanapathaEntry::basic(
    "saNkalAdiH",
    82,
    "4.2.75",
    &[
        "saMkala",
        "puzkala",
        "uttama",
        "uqupa",
        "udvepa",
        "utpuwa",
        "kumBa",
        "viDAna",
        "sudakza",
        "sudatta",
        "suBUta",
        "sunetra",
        "sumaNgala",
        "supiNgala",
        "sUta",
        "sikata",
        "pUtIkA",
        "pUtika",
        "pUlAsa",
        "kUlAsa",
        "palASa",
        "niveSa",
        "gaveSa",
        "gamBIra",
        "itara",
        "An",
        "ahan",
        "loman",
        "veman",
        "caraRa",
        "varuRa",
        "bahula",
        "sadyoja",
        "aBizikta",
        "goBft",
        "rAjaBft",
        "Balla",
        "malla",
        "mAla",
    ],
);

/// *suvāstvādi-gaṇa* (83), first used in the following *sūtra*:
///
/// > 4.2.77 *suvāstvādibhyō'ṇ*
pub(crate) const SUVASTVADI: GanapathaEntry = GanapathaEntry::basic(
    "suvAstvAdiH",
    83,
    "4.2.77",
    &[
        "suvAstu",
        "suvastu",
        "varRu",
        "BaRqu",
        "KaRqu",
        "sevAnin",
        "karpUrin",
        "SiKaRqin",
        "garta",
        "karkaSa",
        "SakawIkarRa",
        "kfzRakarRa",
        "karka",
        "karkanDumatI",
        "goha",
        "ahisakTa",
    ],
);

/// *arīhaṇādi-gaṇa* (84), first used in the following *sūtra*:
///
/// > 4.2.80 *vuñchaṇkaṭhajilaśēniraḍhañṇyayaphakphiñiññyakakṭhakō'rīhaṇakr̥śāśvarśyakumudakāśatr̥ṇaprēkṣā'śmasakhisaṁkāśabalapakṣakarṇasutaṁgamapragadinvarāhakumudādibhyaḥ*
pub(crate) const ARIHANADI: GanapathaEntry = GanapathaEntry::basic(
    "arIhaRAdiH",
    84,
    "4.2.80",
    &[
        "arIhaRa",
        "ahIraRa",
        "druGaRa",
        "druhaRa",
        "Balaga",
        "Bagala",
        "ulanda",
        "kiraRa",
        "sAmparAyaRa",
        "krOzwrAyana",
        "krOzwrAyaRa",
        "OzwrAyaRa",
        "trEgartAyana",
        "mEtrAyaRa",
        "BAstrAyaRa",
        "vEmatAyaRa",
        "vEmatAyana",
        "gOmatAyana",
        "sOmatAyana",
        "sOsAyana",
        "DOmatAyana",
        "sOmAyana",
        "EndrAyaRa",
        "kOdrAyaRa",
        "kOndrAyaRa",
        "KAqAyana",
        "SARqilyAyana",
        "rAyaspoza",
        "vipaTa",
        "vipASa",
        "uddaRqa",
        "udaYcana",
        "KARqavIraRa",
        "vIraRa",
        "kASakftsna",
        "kaSakftsna",
        "jAmbavat",
        "SiMSapA",
        "rEvata",
        "revata",
        "bilva",
        "suyujYa",
        "SirIza",
        "baDira",
        "jambu",
        "suSarman",
        "saSarman",
        "halatf",
        "Balandana",
        "KaRqu",
        "kalana",
        "kanala",
        "yajYadatta",
    ],
);

/// *kr̥śāśvādi-gaṇa* (85), first used in the following *sūtra*:
///
/// > 4.2.80 *vuñchaṇkaṭhajilaśēniraḍhañṇyayaphakphiñiññyakakṭhakō'rīhaṇakr̥śāśvarśyakumudakāśatr̥ṇaprēkṣā'śmasakhisaṁkāśabalapakṣakarṇasutaṁgamapragadinvarāhakumudādibhyaḥ*
pub(crate) const KRSHASHVADI: GanapathaEntry = GanapathaEntry::basic(
    "kfSASvAdiH",
    85,
    "4.2.80",
    &[
        "kfSASva",
        "arizwa",
        "ArISma",
        "veSman",
        "viSAla",
        "lomaSa",
        "romaSa",
        "lomaka",
        "romaka",
        "Sabala",
        "kUwa",
        "varcala",
        "suvarcala",
        "sukara",
        "sUkara",
        "prAtara",
        "pratara",
        "sadfSa",
        "puraga",
        "purAga",
        "suKa",
        "DUma",
        "ajina",
        "vinatA",
        "avanata",
        "vikuGAsa",
        "kuviwyAsa",
        "parASara",
        "arus",
        "ayas",
        "mOdralyAkara",
        "mOdgalya",
        "yukara",
    ],
);

/// *r̥śyādi-gaṇa* (86), first used in the following *sūtra*:
///
/// > 4.2.80 *vuñchaṇkaṭhajilaśēniraḍhañṇyayaphakphiñiññyakakṭhakō'rīhaṇakr̥śāśvarśyakumudakāśatr̥ṇaprēkṣā'śmasakhisaṁkāśabalapakṣakarṇasutaṁgamapragadinvarāhakumudādibhyaḥ*
pub(crate) const RSHYADI: GanapathaEntry = GanapathaEntry::basic(
    "fSyAdiH",
    86,
    "4.2.80",
    &[
        "fSya",
        "fzya",
        "nyagroDa",
        "Sara",
        "nilIna",
        "nivAsa",
        "nivAta",
        "niDAna",
        "nibadDana",
        "nibanDa",
        "vibadDa",
        "parigUQa",
        "upagUQa",
        "asanI",
        "sita",
        "mat",
        "veSman",
        "uttarASman",
        "aSman",
        "sTUla",
        "bAhu",
        "Kadira",
        "SarkarA",
        "anaquha",
        "anaquh",
        "araqu",
        "parivaMSa",
        "veRu",
        "vIraRa",
        "KaRqa",
        "daRqa",
        "parivftta",
        "kardama",
        "aMSu",
    ],
);

/// *kumudādi-gaṇa* (87), first used in the following *sūtra*:
///
/// > 4.2.80 *vuñchaṇkaṭhajilaśēniraḍhañṇyayaphakphiñiññyakakṭhakō'rīhaṇakr̥śāśvarśyakumudakāśatr̥ṇaprēkṣā'śmasakhisaṁkāśabalapakṣakarṇasutaṁgamapragadinvarāhakumudādibhyaḥ*
pub(crate) const KUMUDADI: GanapathaEntry = GanapathaEntry::basic(
    "kumudAdiH",
    87,
    "4.2.80",
    &[
        "kumuda",
        "SarkarA",
        "nyagroDa",
        "itkawa",
        "saMkawa",
        "kaMkawa",
        "garta",
        "bIja",
        "parivApa",
        "niryAsa",
        "Sakawa",
        "kaca",
        "maDu",
        "SirIza",
        "aSva",
        "aSvatTa",
        "balvaja",
        "yavAsa",
        "kUpa",
        "vikaNkata",
        "daSagrAma",
    ],
);

/// *kāśādi-gaṇa* (88), first used in the following *sūtra*:
///
/// > 4.2.80 *vuñchaṇkaṭhajilaśēniraḍhañṇyayaphakphiñiññyakakṭhakō'rīhaṇakr̥śāśvarśyakumudakāśatr̥ṇaprēkṣā'śmasakhisaṁkāśabalapakṣakarṇasutaṁgamapragadinvarāhakumudādibhyaḥ*
pub(crate) const KASHADI: GanapathaEntry = GanapathaEntry::basic(
    "kASAdiH",
    88,
    "4.2.80",
    &[
        "kASa", "pASa", "aSvatTa", "palASa", "pIyUkza", "varaRa", "vAsa", "naqa", "vana",
        "kardama", "kacCUla", "vaNkawa", "gfha", "bisa", "tfRa", "karpUra", "barbara", "maDura",
        "graha", "kapitTa", "jatu", "sIpAla",
    ],
);

/// *tr̥ṇādi-gaṇa* (89), first used in the following *sūtra*:
///
/// > 4.2.80 *vuñchaṇkaṭhajilaśēniraḍhañṇyayaphakphiñiññyakakṭhakō'rīhaṇakr̥śāśvarśyakumudakāśatr̥ṇaprēkṣā'śmasakhisaṁkāśabalapakṣakarṇasutaṁgamapragadinvarāhakumudādibhyaḥ*
pub(crate) const TRNADI: GanapathaEntry = GanapathaEntry::basic(
    "tfRAdiH",
    89,
    "4.2.80",
    &[
        "tfRa", "naqa", "mUla", "vana", "parRa", "varRa", "varARa", "bila", "pula", "Pala",
        "arjuna", "arRa", "suvarRa", "bala", "caraRa", "busa",
    ],
);

/// *prēkṣādi-gaṇa* (90), first used in the following *sūtra*:
///
/// > 4.2.80 *vuñchaṇkaṭhajilaśēniraḍhañṇyayaphakphiñiññyakakṭhakō'rīhaṇakr̥śāśvarśyakumudakāśatr̥ṇaprēkṣā'śmasakhisaṁkāśabalapakṣakarṇasutaṁgamapragadinvarāhakumudādibhyaḥ*
pub(crate) const PREKSHADI: GanapathaEntry = GanapathaEntry::basic(
    "prekzAdiH",
    90,
    "4.2.80",
    &[
        "prekzA", "PalakA", "halakA", "banDukA", "DruvakA", "kzipakA", "nyagroDa", "ikkawa",
        "kaNkaka", "saMkawa", "kawa", "kUpa", "buka", "puka", "puwa", "maha", "parivApa", "yavAza",
        "DruvakA", "garta", "kUpaka", "hiraRya",
    ],
);

/// *aśmādi-gaṇa* (91), first used in the following *sūtra*:
///
/// > 4.2.80 *vuñchaṇkaṭhajilaśēniraḍhañṇyayaphakphiñiññyakakṭhakō'rīhaṇakr̥śāśvarśyakumudakāśatr̥ṇaprēkṣā'śmasakhisaṁkāśabalapakṣakarṇasutaṁgamapragadinvarāhakumudādibhyaḥ*
pub(crate) const ASHMADI: GanapathaEntry = GanapathaEntry::basic(
    "aSmAdiH",
    91,
    "4.2.80",
    &[
        "aSman", "yUTa", "Uza", "mIna", "mada", "darBa", "vfnda", "guda", "KaRqa", "naga", "SiKA",
        "kowa", "pAma", "kanda", "kAnda", "kula", "gahva", "guqa", "kuRqala", "pIna", "guha",
    ],
);

/// *sakhyādi-gaṇa* (92), first used in the following *sūtra*:
///
/// > 4.2.80 *vuñchaṇkaṭhajilaśēniraḍhañṇyayaphakphiñiññyakakṭhakō'rīhaṇakr̥śāśvarśyakumudakāśatr̥ṇaprēkṣā'śmasakhisaṁkāśabalapakṣakarṇasutaṁgamapragadinvarāhakumudādibhyaḥ*
pub(crate) const SAKHYADI: GanapathaEntry = GanapathaEntry::basic(
    "saKyAdiH",
    92,
    "4.2.80",
    &[
        "saKi",
        "agnidatta",
        "vAyudatta",
        "saKidatta",
        "gopila",
        "BallapAla",
        "Balla",
        "pAla",
        "cakta",
        "cakravAka",
        "Cagala",
        "aSoka",
        "karavIra",
        "vAsava",
        "vIra",
        "pUra",
        "vajra",
        "kuSIraka",
        "SIhara",
        "sIhara",
        "saraka",
        "sarasa",
        "saMmara",
        "samala",
        "surasa",
        "roha",
        "tamAla",
        "kadala",
        "saptala",
    ],
);

/// *saṅkāśādi-gaṇa* (93), first used in the following *sūtra*:
///
/// > 4.2.80 *vuñchaṇkaṭhajilaśēniraḍhañṇyayaphakphiñiññyakakṭhakō'rīhaṇakr̥śāśvarśyakumudakāśatr̥ṇaprēkṣā'śmasakhisaṁkāśabalapakṣakarṇasutaṁgamapragadinvarāhakumudādibhyaḥ*
pub(crate) const SANKASHADI: GanapathaEntry = GanapathaEntry::basic(
    "saNkASAdiH",
    93,
    "4.2.80",
    &[
        "saMkASa",
        "kapila",
        "kaSmIra",
        "samIra",
        "surasena",
        "saraka",
        "sUra",
        // <<supaTinpanTa ca>>
        "yUpa",
        "yUTa",
        "aMSa",
        "aNka",
        "nAsA",
        "palita",
        "anu",
        "nASa",
        "aSman",
        "kUwa",
        "malina",
        "daSa",
        "kumBa",
        "SIrza",
        "ciranta",
        "virata",
        "samala",
        "sIra",
        "paYjara",
        "manTa",
        "nala",
        "roman",
        "loman",
        "pulina",
        "supari",
        "kawipa",
        "sakarRaka",
        "vfzwi",
        "tIrTa",
        "agasti",
        "vikara",
        "nAsikA",
    ],
);

/// *balādi-gaṇa* (94), first used in the following *sūtra*:
///
/// > 4.2.80 *vuñchaṇkaṭhajilaśēniraḍhañṇyayaphakphiñiññyakakṭhakō'rīhaṇakr̥śāśvarśyakumudakāśatr̥ṇaprēkṣā'śmasakhisaṁkāśabalapakṣakarṇasutaṁgamapragadinvarāhakumudādibhyaḥ*
pub(crate) const BALADI: GanapathaEntry = GanapathaEntry::basic(
    "balAdiH",
    94,
    "4.2.80",
    &[
        "bala", "cula", "nala", "dala", "vawa", "lakula", "urala", "puKa", "pula", "mUla",
        "ulaqula", "ula", "qula", "vana", "kula",
    ],
);

/// *pakṣādi-gaṇa* (95), first used in the following *sūtra*:
///
/// > 4.2.80 *vuñchaṇkaṭhajilaśēniraḍhañṇyayaphakphiñiññyakakṭhakō'rīhaṇakr̥śāśvarśyakumudakāśatr̥ṇaprēkṣā'śmasakhisaṁkāśabalapakṣakarṇasutaṁgamapragadinvarāhakumudādibhyaḥ*
pub(crate) const PAKSHADI: GanapathaEntry = GanapathaEntry::basic(
    "pakzAdiH",
    95,
    "4.2.80",
    &[
        "pakza",
        "tukza",
        "tuza",
        "kuRqa",
        "aRqa",
        "kambalikA",
        "valika",
        "citra",
        "asti",
        // <<supaTinpanTa ca>>
        "kumBa",
        "sIraka",
        "saraka",
        "sakala",
        "sarasa",
        "samala",
        "atiSvan",
        "roman",
        "loman",
        "hastin",
        "maka",
        "lomaka",
        "SIrza",
        "nivAta",
        "pAka",
        "sahaka",
        "siMhaka",
        "aNkuSa",
        "suvarRaka",
        "haMsaka",
        "hiMsaka",
        "kutsa",
        "bila",
        "Kila",
        "yamala",
        "hasta",
        "kalA",
        "sakarRaka",
    ],
);

/// *karṇādi-gaṇa* (96), first used in the following *sūtra*:
///
/// > 4.2.80 *vuñchaṇkaṭhajilaśēniraḍhañṇyayaphakphiñiññyakakṭhakō'rīhaṇakr̥śāśvarśyakumudakāśatr̥ṇaprēkṣā'śmasakhisaṁkāśabalapakṣakarṇasutaṁgamapragadinvarāhakumudādibhyaḥ*
pub(crate) const KARNADI: GanapathaEntry = GanapathaEntry::basic(
    "karRAdiH",
    96,
    "4.2.80",
    &[
        "karRa",
        "vasizWa",
        "arka",
        "arkalUza",
        "drupada",
        "anaquhya",
        "pAYcajanya",
        "sPiga",
        "sPij",
        "kumBI",
        "kuntI",
        "jitvan",
        "jIvanta",
        "kuliSa",
        "aRqIvat",
        "ARaqIvata",
        "java",
        "jEtra",
        "Akana",
        "anaka",
    ],
);

/// *sutaṅgamādi-gaṇa* (97), first used in the following *sūtra*:
///
/// > 4.2.80 *vuñchaṇkaṭhajilaśēniraḍhañṇyayaphakphiñiññyakakṭhakō'rīhaṇakr̥śāśvarśyakumudakāśatr̥ṇaprēkṣā'śmasakhisaṁkāśabalapakṣakarṇasutaṁgamapragadinvarāhakumudādibhyaḥ*
pub(crate) const SUTANGAMADI: GanapathaEntry = GanapathaEntry::basic(
    "sutaNgamAdiH",
    97,
    "4.2.80",
    &[
        "suMtagama",
        "municita",
        "vipracitta",
        "mahAcitta",
        "mahAputra",
        "svana",
        "Sveta",
        "gaqika",
        "Kaqika",
        "Sukra",
        "vigra",
        "bIjAvApin",
        "bIjavApin",
        "arjuna",
        "Svan",
        "ajira",
        "jIva",
        "KaRqina",
        "karRa",
        "vigraha",
    ],
);

/// *pragadyādi-gaṇa* (98), first used in the following *sūtra*:
///
/// > 4.2.80 *vuñchaṇkaṭhajilaśēniraḍhañṇyayaphakphiñiññyakakṭhakō'rīhaṇakr̥śāśvarśyakumudakāśatr̥ṇaprēkṣā'śmasakhisaṁkāśabalapakṣakarṇasutaṁgamapragadinvarāhakumudādibhyaḥ*
pub(crate) const PRAGADYADI: GanapathaEntry = GanapathaEntry::basic(
    "pragadyAdiH",
    98,
    "4.2.80",
    &[
        "pragadin", "magadin", "maMdadin", "kavila", "KaRqita", "gadita", "cUqAra", "mandAra",
        "kovidAra",
    ],
);

/// *varāhādi-gaṇa* (99), first used in the following *sūtra*:
///
/// > 4.2.80 *vuñchaṇkaṭhajilaśēniraḍhañṇyayaphakphiñiññyakakṭhakō'rīhaṇakr̥śāśvarśyakumudakāśatr̥ṇaprēkṣā'śmasakhisaṁkāśabalapakṣakarṇasutaṁgamapragadinvarāhakumudādibhyaḥ*
pub(crate) const VARAHADI: GanapathaEntry = GanapathaEntry::basic(
    "varAhAdiH",
    99,
    "4.2.80",
    &[
        "varAha", "palASA", "palASa", "Seriza", "SirIza", "pinadDa", "nibadDa", "balAha", "sTUla",
        "vidagDa", "vijagDa", "viBagna", "nimagna", "bAhu", "Kadira", "SarkarA",
    ],
);

/// *kumudādi-gaṇa* (100), first used in the following *sūtra*:
///
/// > 4.2.80 *vuñchaṇkaṭhajilaśēniraḍhañṇyayaphakphiñiññyakakṭhakō'rīhaṇakr̥śāśvarśyakumudakāśatr̥ṇaprēkṣā'śmasakhisaṁkāśabalapakṣakarṇasutaṁgamapragadinvarāhakumudādibhyaḥ*
pub(crate) const KUMUDADI_4: GanapathaEntry = GanapathaEntry::basic(
    "kumudAdiH",
    100,
    "4.2.80",
    &[
        "kumuda",
        "gomaTa",
        "raTakAra",
        "daSagrAma",
        "aSvatTa",
        "SAlmalI",
        "SirIza",
        "munisTala",
        "kuRqala",
        "kUwa",
        "muDukarRa",
        "GAsakunda",
        "SucikarRa",
    ],
);

/// *varaṇādi-gaṇa* (101), first used in the following *sūtra*:
///
/// > 4.2.82 *varaṇādibhyaśca*
pub(crate) const VARANADI: GanapathaEntry = GanapathaEntry::basic(
    "varaRAdiH",
    101,
    "4.2.82",
    &[
        "varaRA",
        "SfNgI",
        "SAlmali",
        "SuRqi",
        "SayARqI",
        "parRI",
        "tAmraparRI",
        "goda",
        "AliNgyAyana",
        "jAlapadI",
        "jAnapadI",
        "jambU",
        "puzkara",
        "sampA",
        "pampA",
        "valgu",
        "ujjayinI",
        "gayA",
        "maTurA",
        "takzaSilA",
        "urasA",
        "gomatI",
        "valaBI",
    ],
);

/// *madhvādi-gaṇa* (102), first used in the following *sūtra*:
///
/// > 4.2.86 *madhvādibhyaśca*
pub(crate) const MADHVADI: GanapathaEntry = GanapathaEntry::basic(
    "maDvAdiH",
    102,
    "4.2.86",
    &[
        "maDu",
        "bisa",
        "sTARu",
        "veRu",
        "karkanDu",
        "SamI",
        "karIra",
        "hima",
        "kiSarA",
        "SaryARA",
        "marut",
        "vArdAlI",
        "Sara",
        "izwakA",
        "Asuti",
        "Sakti",
        "AsandI",
        "Sakala",
        "SalAkA",
        "AmizI",
        "ikzu",
        "roman",
        "ruzwi",
        "ruzya",
        "takzaSilA",
        "kaqa",
        "vawa",
        "vewa",
    ],
);

/// *utkarādi-gaṇa* (103), first used in the following *sūtra*:
///
/// > 4.2.90 *utkarādibhyaśchaḥ*
pub(crate) const UTKARADI: GanapathaEntry = GanapathaEntry::basic(
    "utkarAdiH",
    103,
    "4.2.90",
    &[
        "utkara",
        "saMPala",
        "SaPara",
        "pippala",
        "pippalImUla",
        "aSman",
        "suvarRa",
        "svalAjina",
        "tika",
        "kitava",
        "aRaka",
        "trEvaRa",
        "picuka",
        "aSvatTa",
        "kASa",
        "kzudra",
        "BastrA",
        "SAla",
        "janyA",
        "ajira",
        "carman",
        "utkroSa",
        "kzAnta",
        "Kadira",
        "SUrpaRAya",
        "SyAvanAya",
        "nEvAkava",
        "tfRa",
        "vfkza",
        "SAka",
        "palASa",
        "vijigIzA",
        "aneka",
        "Atapa",
        "Pala",
        "saMpara",
        "arka",
        "garta",
        "agni",
        "vErARaka",
        "iqA",
        "araRya",
        "niSAnta",
        "paRa",
        "nIcAyaka",
        "SaMkara",
        "avarohita",
        "kzAra",
        "viSAla",
        "vetra",
        "arIhaRa",
        "KaRqa",
        "vAtAgAra",
        "indravfkza",
        "nitAntavfkza",
        "nitAntAvfkza",
        "Ardravfkza",
    ],
);

/// *naḍādi-gaṇa* (104), first used in the following *sūtra*:
///
/// > 4.2.91 *naḍādīnāṁ kuk ca*
pub(crate) const NADADI_4: GanapathaEntry = GanapathaEntry::basic(
    "naqAdiH",
    104,
    "4.2.91",
    &[
        "naqa", "plakza", "bilva", "veRu", "vetra", "vetasa", "ikzu", "kAzWa", "kapota",
        "tfRa",
        // <<kruYcA hrasvatvaM ca>>
        // <<takzannalopaSca>>
    ],
);

/// *katryādi-gaṇa* (105), first used in the following *sūtra*:
///
/// > 4.2.95 *katryādibhyō ḍhakañ*
pub(crate) const KATRYADI: GanapathaEntry = GanapathaEntry::basic(
    "katryAdiH",
    105,
    "4.2.95",
    &[
        "katri",
        "umBi",
        "puzkara",
        "puzkala",
        "modana",
        "kumBI",
        "kuRqina",
        "nagarI",
        "mAhizmatI",
        "varmatI",
        "uKyA",
        "grAma",
        "kuqyAyA",
        "yalopaSca",
    ],
);

/// *nadyādi-gaṇa* (106), first used in the following *sūtra*:
///
/// > 4.2.97 *nadyādibhyō ḍhak*
pub(crate) const NADYADI: GanapathaEntry = GanapathaEntry::basic(
    "nadyAdiH",
    106,
    "4.2.97",
    &[
        "nadI",
        "mahI",
        "vArARasI",
        "SrAvastI",
        "kOSAmbI",
        "vanakOSAmbI",
        "vanakoSAmbI",
        "kASaparI",
        "kASaPArI",
        "kASaParI",
        "KAdirI",
        "pUrvanagarI",
        "pAWA",
        "mAyA",
        "SAlvA",
        "dArvA",
        "setakI",
        "vaqavAyA",
        "vfze",
    ],
);

/// *paladyādi-gaṇa* (107), first used in the following *sūtra*:
///
/// > 4.2.110 *prasthōttarapadapaladyādikōpadhādaṇ*
pub(crate) const PALADYADI: GanapathaEntry = GanapathaEntry::basic(
    "paladyAdiH",
    107,
    "4.2.110",
    &[
        "paladI",
        "parizad",
        "romaka",
        "vAhIka",
        "kalakIwa",
        "bahukIwa",
        "jAlakIwa",
        "kamalakIwa",
        "kamalakIkara",
        "kamalaBidA",
        "gozWI",
        "nEkatI",
        "pariKA",
        "SUrasena",
        "gomatI",
        "pawaccara",
        "udapAna",
        "yakflloma",
    ],
);

/// *kāśyādi-gaṇa* (108), first used in the following *sūtra*:
///
/// > 4.2.116 *kāśyādibhyaṣṭhaññiṭhau*
pub(crate) const KASHYADI: GanapathaEntry = GanapathaEntry::basic(
    "kASyAdiH",
    108,
    "4.2.116",
    &[
        "kASi",
        "cedi",
        "vedi",
        "sAMyAti",
        "saMvAha",
        "acyuta",
        "modamAna",
        "SakulAda",
        "hastikarzU",
        "kunAman",
        "hiraRyA",
        "karaRa",
        "govAsana",
        "BAraNgI",
        "ariMdama",
        "aritra",
        "devadatta",
        "daSagrAma",
        "SOvAvatAna",
        "yuvarAja",
        "uparAja",
        "devarAja",
        "modana",
        "sinDumitra",
        "dAsamitra",
        "suDAmitra",
        "somamitra",
        "CAgamitra",
        "sADamitra",
        "saDamitra",
        // <<ApadAdipUrvapadAtkAlAntAt>>
        "ApatkAla",
        "UrDvakAla",
        "tatkAla",
    ],
);

/// *dhūmādi-gaṇa* (109), first used in the following *sūtra*:
///
/// > 4.2.127 *dhūmādibhyaśca*
pub(crate) const DHUMADI: GanapathaEntry = GanapathaEntry::basic(
    "DUmAdiH",
    109,
    "4.2.127",
    &[
        "DUma",
        "zaqaRqa",
        "SaSAdana",
        "arjunAva",
        "mAhakasTalI",
        "AnakasTalI",
        "mAhizasTalI",
        "mAnasTalI",
        "awwasTalI",
        "madrakasTalI",
        "samudrasTalI",
        "dARqAyanasTalI",
        "rAjasTalI",
        "videha",
        "rAjagfha",
        "sAtrAsAha",
        "Sazpa",
        "mitravarDra",
        "mitravarDa",
        "BakzAlI",
        "madrakUla",
        "AjIkUla",
        "dvyAhava",
        "dvyAhAva",
        "tryahava",
        "tryAhAva",
        "saMsPAya",
        "barbara",
        "varjya",
        "garta",
        "Anarta",
        "mAWara",
        "pATeya",
        "Goza",
        "pallI",
        "ArAjYI",
        "DArtarAjYI",
        "Avaya",
        "tIrTa",
        // <<kUlAtsOvIrezu>>
        // <<samudAnnAvi manuzye ca>>
        "kukzi",
        "antarIpa",
        "dvIpa",
        "aruRa",
        "ujjayanI",
        "pawwAra",
        "dakziRApaTa",
        "sAketa",
    ],
);

/// *kacchādi-gaṇa* (110), first used in the following *sūtra*:
///
/// > 4.2.133 *kacchādibhyaśca*
pub(crate) const KACCHADI: GanapathaEntry = GanapathaEntry::basic(
    "kacCAdiH",
    110,
    "4.2.133",
    &[
        "kacCa", "sinDu", "varRu", "ganDAra", "maDumat", "kamboja", "kaSmIra", "sAlva", "kuru",
        "anuSaRqa", "dvIpa", "anUpa", "ajavAha", "vijApaka", "kalUtara", "raNku",
    ],
);

/// *gahādi-gaṇa* (111), first used in the following *sūtra*:
///
/// > 4.2.138 *gahādibhyaśca*
pub(crate) const GAHADI: GanapathaEntry = GanapathaEntry::akrti(
    "gahAdiH",
    111,
    "4.2.138",
    &[
        "gaha",
        "antaHsTa",
        "sama",
        "vizama",
        "maDya",
        "maDyaMdina",
        "caraRe",
        "uttama",
        "aNga",
        "vaNga",
        "magaDa",
        "pUrvapakza",
        "aparapakza",
        "aDamaSAKa",
        "uttamaSAKa",
        "samAnagrAma",
        "ekagrAma",
        "ekavfkza",
        "ekapalASa",
        "izvagra",
        "izvanIka",
        "avasyandan",
        "kAmaprasTa",
        "SAqikAqAyani",
        "KAqAyana",
        "kAWeraRi",
        "lAveraRi",
        "sOmitri",
        "SESiri",
        "Ayuta",
        "dEvaSarmi",
        "SrOti",
        "AhiMsi",
        "Amitri",
        "vyAqi",
        "vEji",
        "ADyaSvi",
        "sOmitri",
        "vyAqi",
        "vEji",
        "ADyaSvi",
        "AnfSasi",
        "AnfSaMsi",
        "agniSarmi",
        "BOji",
        "vArAwaki",
        "vAlmiki",
        "vAlmIki",
        "kzEmavfdDi",
        "ASvatTi",
        "OdgAhamAni",
        "ekavindavi",
        "dantAgra",
        "haMsa",
        "tatvagra",
        "tantvagra",
        "uttara",
        "antara",
        "anantara",
        // <<muKapArSvatasorlopaH>>
        // <<janaparayoH kukca devasya ca>>
        // <<veRukAdiByaSCaR>>
    ],
);

/// *sandhivēlādi-gaṇa* (112), first used in the following *sūtra*:
///
/// > 4.3.16 *saṁdhivēlādyr̥tunakṣatrēbhyō'ṇ*
pub(crate) const SANDHIVELADI: GanapathaEntry = GanapathaEntry::basic(
    "sanDivelAdiH",
    112,
    "4.3.16",
    &[
        "sanDivelA",
        "saMDyA",
        "amAvAsyA",
        "trayodaSI",
        "caturdaSI",
        "paYcadaSI",
        "pOrRamasI",
        "pratipat",
        // <<saMvatsarAtPalaparvaRoH>>
    ],
);

/// *digādi-gaṇa* (113), first used in the following *sūtra*:
///
/// > 4.3.54 *digādibhyō yat*
pub(crate) const DIGADI: GanapathaEntry = GanapathaEntry::basic(
    "digAdiH",
    113,
    "4.3.54",
    &[
        "diS", "varga", "pUga", "gaRa", "pakza", "DAyyA", "mitra", "meDA", "antara", "paTin",
        "rahas", "alIka", "uKA", "sAkzin", "Adi", "deSa", "Adi", "anta", "muKa", "jaGana", "meGa",
        "yUTa", // <<udakAtsaMjYayAm>>
        "jYAya", "nyAya", "vaMSa", "veSa", "kAla", "AkASa",
    ],
);

/// *parimukhādi-gaṇa* (114), first used in the following *sūtra*:
///
/// > 4.3.59 *avyayībhāvācca*
/// The *sūtra* has the following *vārttika*:
///
/// > *parimuKAdiByaSca*
pub(crate) const PARIMUKHADI: GanapathaEntry = GanapathaEntry::basic(
    "parimuKAdiH",
    114,
    "4.3.59",
    &[
        "parimuKa",
        "parihanu",
        "paryozWa",
        "paryulUKala",
        "parisIra",
        "upasIra",
        "upasTURa",
        "upakalApa",
        "anupaTa",
        "anugaqga",
        "anutila",
        "anusIta",
        "aDusAya",
        "anusIra",
        "anumAza",
        "anuyava",
        "anuyUpa",
        "anuvaMSa",
        "pratiSAKa",
    ],
)
.with_varttika("parimuKAdiByaSca");

/// *adhyātmādi-gaṇa* (115), first used in the following *sūtra*:
///
/// > 4.3.60 *antaḥpūrvapadāṭṭhañ*
/// The *sūtra* has the following *vārttika*:
///
/// > *aDyAtmAdiByaSca*
pub(crate) const ADHYATMADI: GanapathaEntry = GanapathaEntry::akrti(
    "aDyAtmAdiH",
    115,
    "4.3.60",
    &["aDyAtma", "aDideva", "aDiBUta", "ihaloka", "paraloka"],
)
.with_varttika("aDyAtmAdiByaSca");

/// *r̥gayanādi-gaṇa* (116), first used in the following *sūtra*:
///
/// > 4.3.73 *aṇr̥gayanādibhyaḥ*
pub(crate) const RGAYANADI: GanapathaEntry = GanapathaEntry::basic(
    "fgayanAdiH",
    116,
    "4.3.73",
    &[
        "fgayana",
        "padavyAKyAna",
        "CandomAna",
        "CagdoBAzA",
        "Candoviciti",
        "nyAya",
        "punarukta",
        "vyAkaraRa",
        "nigama",
        "vAstuvidyA",
        "kzatravidyA",
        "aNgavidyA",
        "vidyA",
        "utpAda",
        "utpAta",
        "saMvatsara",
        "muhUrta",
        "upanizad",
        "nimitta",
        "SikzA",
        "BikzA",
    ],
);

/// *śuṇḍikādi-gaṇa* (117), first used in the following *sūtra*:
///
/// > 4.3.76 *śuṇḍikādibhyō'ṇ*
pub(crate) const SHUNDIKADI: GanapathaEntry = GanapathaEntry::basic(
    "SuRqikAdiH",
    117,
    "4.3.76",
    &[
        "SuRqika", "kfkaRa", "kfpaRa", "sTaRqila", "udapAna", "upala", "tIrTa", "BUmi", "tfRa",
        "parRa",
    ],
);

/// *śaṇḍikādi-gaṇa* (118), first used in the following *sūtra*:
///
/// > 4.3.92 *śaṇḍikādibhyō ñyaḥ*
pub(crate) const SHANDIKADI: GanapathaEntry = GanapathaEntry::basic(
    "SaRqikAdiH",
    118,
    "4.3.92",
    &[
        "SaRqika",
        "sarvasena",
        "sarvakeSa",
        "Saka",
        "Sawa",
        "raka",
        "SaNKa",
        "boDa",
    ],
);

/// *sindhvādi-gaṇa* (119), first used in the following *sūtra*:
///
/// > 4.3.93 *sindhutakṣaśilādibhyō'ṇañau*
pub(crate) const SINDHVADI: GanapathaEntry = GanapathaEntry::basic(
    "sinDvAdiH",
    119,
    "4.3.93",
    &[
        "sinDu", "varRu", "maDumat", "kamboja", "sAlva", "kaSmIra", "ganDAra", "kizkinDA", "urasA",
        "darada", "darad", "gandikA",
    ],
);

/// *takṣaśilādi-gaṇa* (120), first used in the following *sūtra*:
///
/// > 4.3.93 *sindhutakṣaśilādibhyō'ṇañau*
pub(crate) const TAKSHASHILADI: GanapathaEntry = GanapathaEntry::basic(
    "takzaSilAdiH",
    120,
    "4.3.93",
    &[
        "takzaSilA",
        "vatsodDaraRa",
        "kOmedura",
        "grAmaRI",
        "Cagala",
        "kozwukarRa",
        "siMhakarRa",
        "saMkucita",
        "kinnara",
        "kARqaDAra",
        "parvata",
        "avasAna",
        "barbara",
        "kaMsa",
    ],
);

/// *śaunakādi-gaṇa* (121), first used in the following *sūtra*:
///
/// > 4.3.106 *śaunakādibhyaśchandasi*
pub(crate) const SHAUNAKADI: GanapathaEntry = GanapathaEntry::basic(
    "SOnakAdiH",
    121,
    "4.3.106",
    &[
        "SOnaka",
        "vAjasaneya",
        "SArNgarava",
        "SAzpeya",
        "KAqAyana",
        "stamBa",
        "skanDa",
        "devadarSana",
        "rajjuBAra",
        "rajjukaRWa",
        "kaWaSAWa",
        "kazAya",
        "tala",
        "daRqa",
        "puruzAMsaka",
        "aSvapeja",
    ],
);

/// *kulālādi-gaṇa* (122), first used in the following *sūtra*:
///
/// > 4.3.118 *kulālādibhyō vuñ*
pub(crate) const KULALADI: GanapathaEntry = GanapathaEntry::basic(
    "kulAlAdiH",
    122,
    "4.3.118",
    &[
        "kulAla",
        "varuqa",
        "cARqAla",
        "nizAda",
        "karmAra",
        "senA",
        "sirinDra",
        "siriGri",
        "sErinDra",
        "devarAja",
        "parzat",
        "parizat",
        "vaDU",
        "maDU",
        "ruhU",
        "rudra",
        "anaquH",
        "brahman",
        "kumBakAra",
        "SvapAka",
    ],
);

/// *raivatikādi-gaṇa* (123), first used in the following *sūtra*:
///
/// > 4.3.131 *raivatikādibhyaśchaḥ*
pub(crate) const RAIVATIKADI: GanapathaEntry = GanapathaEntry::basic(
    "rEvatikAdiH",
    123,
    "4.3.131",
    &[
        "rEvatika",
        "svApiSi",
        "gOragrIva",
        "gOragrIvi",
        "OdameGi",
        "OdavApi",
        "bejavApi",
    ],
);

/// *bilvādi-gaṇa* (124), first used in the following *sūtra*:
///
/// > 4.3.136 *bilvādibhyō'ṇ*
pub(crate) const BILVADI: GanapathaEntry = GanapathaEntry::basic(
    "bilvAdiH",
    124,
    "4.3.136",
    &[
        "bilva", "vrIhi", "kARqa", "mudga", "masUra", "goDUma", "ikzu", "veRu", "gaveDukA",
        "karpAsI", "pAwalI", "karkanDU", "kuwIra",
    ],
);

/// *palāśādi-gaṇa* (125), first used in the following *sūtra*:
///
/// > 4.3.141 *palāśādibhyō vā*
pub(crate) const PALASHADI: GanapathaEntry = GanapathaEntry::basic(
    "palASAdiH",
    125,
    "4.3.141",
    &[
        "palASa",
        "Kadira",
        "SiMSapA",
        "spandana",
        "pUlAka",
        "karIra",
        "SirIza",
        "yavAsa",
        "vikaNkata",
    ],
);

/// *śarādi-gaṇa* (126), first used in the following *sūtra*:
///
/// > 4.3.144 *nityaṁ vr̥ddhaśarādibhyaḥ*
pub(crate) const SHARADI: GanapathaEntry = GanapathaEntry::basic(
    "SarAdiH",
    126,
    "4.3.144",
    &[
        "Sara", "darBa", "mfd", "mft", "kuwI", "tfRa", "soma", "balvaja",
    ],
);

/// *tālādi-gaṇa* (127), first used in the following *sūtra*:
///
/// > 4.3.152 *tālādibhyō'ṇ*
pub(crate) const TALADI: GanapathaEntry = GanapathaEntry::basic(
    "tAlAdiH",
    127,
    "4.3.152",
    &[
        "tAlAdDanuzi",
        "bArhiRa",
        "indrAliSa",
        "indrAdfSa",
        "indrASiza",
        "indrAyuDa",
        "caya",
        "SyAmAka",
        "pIyUkzA",
    ],
);

/// *rajatādi-gaṇa* (128), first used in the following *sūtra*:
///
/// > 4.3.154 *prāṇirajatādibhyō'ñ*
pub(crate) const RAJATADI: GanapathaEntry = GanapathaEntry::basic(
    "rajatAdiH",
    128,
    "4.3.154",
    &[
        "rajata",
        "sIsa",
        "loha",
        "udumbara",
        "nIpa",
        "dAru",
        "rohitaka",
        "biBItaka",
        "pItadAru",
        "tIvradAru",
        "trikaRwaka",
        "kaRwakAra",
    ],
);

/// *plakṣādi-gaṇa* (129), first used in the following *sūtra*:
///
/// > 4.3.164 *plakṣādibhyō'ṇ*
pub(crate) const PLAKSHADI: GanapathaEntry = GanapathaEntry::basic(
    "plakzAdiH",
    129,
    "4.3.164",
    &[
        "plakza", "nyagroDa", "aSvatTa", "iNgudI", "Sigru", "ruru", "kakzatu", "bfhatI",
    ],
);

/// *harītakyādi-gaṇa* (130), first used in the following *sūtra*:
///
/// > 4.3.167 *harītakyādibhyaśca*
pub(crate) const HARITAKYADI: GanapathaEntry = GanapathaEntry::basic(
    "harItakyAdiH",
    130,
    "4.3.167",
    &[
        "harItakI",
        "koSAtakI",
        "naKaraYjanI",
        "SazkaRqI",
        "dAqI",
        "doqI",
        "SvetapAkI",
        "arjunapAkI",
        "drAkzA",
        "kAlA",
        "DvAkzA",
        "gamIkA",
        "kaRwakArikA",
        "pippalI",
        "cimpA",
        "ciYcA",
        "SePAlikA",
    ],
);

/// *māśabdādi-gaṇa* (131), first used in the following *sūtra*:
///
/// > 4.4.1 *prāgvahatēṣṭhak*
/// The *sūtra* has the following *vārttika*:
///
/// > *mASabdAdiBya upasaMKyAnam*
pub(crate) const MASHABDADI: GanapathaEntry = GanapathaEntry::basic(
    "mASabdAdiH",
    131,
    "4.4.1",
    &["mASabdaH", "nityaSabdaH", "kAryaSabdaH"],
)
.with_varttika("mASabdAdiBya upasaMKyAnam");

/// *prabhr̥tādi-gaṇa* (132), first used in the following *sūtra*:
///
/// > 4.4.1 *prāgvahatēṣṭhak*
/// The *sūtra* has the following *vārttika*:
///
/// > *AhO praBUtAdiByaH*
pub(crate) const PRABHRTADI: GanapathaEntry =
    GanapathaEntry::basic("praBftAdiH", 132, "4.4.1", &["praBUta", "paryApta"])
        .with_varttika("AhO praBUtAdiByaH");

/// *susnātādi-gaṇa* (133), first used in the following *sūtra*:
///
/// > 4.4.1 *prāgvahatēṣṭhak*
/// The *sūtra* has the following *vārttika*:
///
/// > *pfcCatO susnAtAdiByaH*
pub(crate) const SUSNATADI: GanapathaEntry = GanapathaEntry::basic(
    "susnAtAdiH",
    133,
    "4.4.1",
    &["susnAta", "suKarAtri", "suKaSayana"],
)
.with_varttika("pfcCatO susnAtAdiByaH");

/// *paradārādi-gaṇa* (134), first used in the following *sūtra*:
///
/// > 4.4.1 *prāgvahatēṣṭhak*
/// The *sūtra* has the following *vārttika*:
///
/// > *gacCatO paradArAdiByaH*
pub(crate) const PARADARADI: GanapathaEntry =
    GanapathaEntry::basic("paradArAdiH", 134, "4.4.1", &["paradAra", "gurutalpa"])
        .with_varttika("gacCatO paradArAdiByaH");

/// *parpādi-gaṇa* (135), first used in the following *sūtra*:
///
/// > 4.4.10 *parpādibhyaḥ ṣṭhan*
pub(crate) const PARPADI: GanapathaEntry = GanapathaEntry::basic(
    "parpAdiH",
    135,
    "4.4.10",
    &[
        "parpa", "aSva", "aSvatTa", "raTa", "jAla", "nyAsa", "vyAla", "pAda", "paYca",
    ],
);

/// *vētanādi-gaṇa* (136), first used in the following *sūtra*:
///
/// > 4.4.12 *vētanādibhyō jīvati*
pub(crate) const VETANADI: GanapathaEntry = GanapathaEntry::basic(
    "vetanAdiH",
    136,
    "4.4.12",
    &[
        "vetana",
        "vAhana",
        "arDavAhana",
        "DanurdaRqa",
        "jAla",
        "veSa",
        "upaveSa",
        "prezaRa",
        "upavasti",
        "suKa",
        "SayyA",
        "Sakti",
        "upanizat",
        "upadeSa",
        "sPij",
        "sPija",
        "pAda",
        "upasTa",
        "upasTAna",
        "upahasta",
    ],
);

/// *utsaṅgādi-gaṇa* (137), first used in the following *sūtra*:
///
/// > 4.4.15 *haratyutsaṅgādibhyaḥ*
pub(crate) const UTSANGADI: GanapathaEntry = GanapathaEntry::basic(
    "utsaNgAdiH",
    137,
    "4.4.15",
    &[
        "utsaNga", "uqupa", "utpUta", "utpanna", "utpuwa", "piwaka", "piwAka",
    ],
);

/// *bhastrādi-gaṇa* (138), first used in the following *sūtra*:
///
/// > 4.4.16 *bhastrādibhyaḥ ṣṭhan*
pub(crate) const BHASTRADI: GanapathaEntry = GanapathaEntry::basic(
    "BastrAdiH",
    138,
    "4.4.16",
    &[
        "BastrA",
        "Barawa",
        "BaraRa",
        "SIrza",
        "SIrzeBAra",
        "aMsaBAra",
        "aMseBAra",
    ],
);

/// *akṣadyūtādi-gaṇa* (139), first used in the following *sūtra*:
///
/// > 4.4.19 *nirvr̥ttē'kṣadyūtādibhyaḥ*
pub(crate) const AKSHADYUTADI: GanapathaEntry = GanapathaEntry::basic(
    "akzadyUtAdiH",
    139,
    "4.4.19",
    &[
        "akzadyUta",
        "jAnuprahfta",
        "jaNGAprahfta",
        "jaNGAprahata",
        "pAdaKedana",
        "kaRwakamardana",
        "gatAnugata",
        "gatAgata",
        "yAtopayAta",
        "anugata",
    ],
);

/// *mahiṣyādi-gaṇa* (140), first used in the following *sūtra*:
///
/// > 4.4.48 *aṇ mahiṣyādibhyaḥ*
pub(crate) const MAHISHYADI: GanapathaEntry = GanapathaEntry::basic(
    "mahizyAdiH",
    140,
    "4.4.48",
    &[
        "mahizI",
        "prajApatI",
        "prajAvatI",
        "pralepikA",
        "vilepikA",
        "anulepikA",
        "purohitA",
        "maRipAlI",
        "anuvAraka",
        "nucAra",
        "hotf",
        "yajamAna",
    ],
);

/// *kisarādi-gaṇa* (141), first used in the following *sūtra*:
///
/// > 4.4.53 *kiśarādibhyaḥ ṣṭhan*
pub(crate) const KISARADI: GanapathaEntry = GanapathaEntry::basic(
    "kisarAdiH",
    141,
    "4.4.53",
    &[
        "kisara", "narada", "nalada", "sTAgala", "tagara", "guggulu", "uSIra", "haridrA",
        "haridru", "pargI", "parRI",
    ],
);

/// *chatrādi-gaṇa* (142), first used in the following *sūtra*:
///
/// > 4.4.62 *chatrādibhyō ṇaḥ*
pub(crate) const CHATRADI: GanapathaEntry = GanapathaEntry::basic(
    "CatrAdiH",
    142,
    "4.4.62",
    &[
        "Catra",
        "SikzA",
        "prarohasTA",
        "buBukzA",
        "carA",
        "titikzA",
        "upasTAna",
        "fzi",
        "karman",
        "viSvaDA",
        "tapas",
        "satya",
        "anfta",
        "viSiKA",
        "viSikA",
        "BakzA",
        "udasTAna",
        "paroqA",
        "vikzA",
        "cukzA",
        "mandra",
    ],
);

/// *pratijanādi-gaṇa* (143), first used in the following *sūtra*:
///
/// > 4.4.99 *pratijanādibhyaḥ khañ*
pub(crate) const PRATIJANADI: GanapathaEntry = GanapathaEntry::basic(
    "pratijanAdiH",
    143,
    "4.4.99",
    &[
        "pratijana",
        "idaMyuga",
        "saMyuga",
        "samayuga",
        "parayuga",
        "parakula",
        "parasyakula",
        "amuzyakula",
        "sarvajana",
        "viSvajana",
        "mahAjana",
        "paYcajana",
    ],
);

/// *kathādi-gaṇa* (144), first used in the following *sūtra*:
///
/// > 4.4.102 *kathādibhyaṣṭhak*
pub(crate) const KATHADI: GanapathaEntry = GanapathaEntry::basic(
    "kaTAdiH",
    144,
    "4.4.102",
    &[
        "kaTA",
        "vikaTA",
        "viSvakaTA",
        "saMkaTA",
        "vitaRqA",
        "kuzwavid",
        "kuzWavid",
        "janavAda",
        "janevAda",
        "janovAda",
        "vftti",
        "saMgfha",
        "guRagaRa",
        "Ayurveda",
    ],
);

/// *guḍādi-gaṇa* (145), first used in the following *sūtra*:
///
/// > 4.4.103 *guḍādibhyaṣṭhañ*
pub(crate) const GUDADI: GanapathaEntry = GanapathaEntry::basic(
    "guqAdiH",
    145,
    "4.4.103",
    &[
        "guqa",
        "kulmAza",
        "saktu",
        "apUpa",
        "mAMsOdana",
        "ikzu",
        "veRu",
        "saNgrAma",
        "saMGAta",
        "saMkAma",
        "saMvAha",
        "pravAsa",
        "nivAsa",
        "upavAsa",
    ],
);

/// *gavādi-gaṇa* (146), first used in the following *sūtra*:
///
/// > 5.1.2 *ugavādibhyō yat*
pub(crate) const GAVADI: GanapathaEntry = GanapathaEntry::basic(
    "gavAdiH",
    146,
    "5.1.2",
    &[
        "go", "havis", "akzara", "viza", "barhis", "azwakA", "svadA", "yuga", "meDA", "srac",
        // <<nABi naBaM ca>>
        // <<SunaH saMprasAraRaM vA ca dIrGatvaM, tatsaMniyogena cAntodAttatvam>>
        // <<UDaso'naN ca>>
        "kUpa", "Kada", "dara", "asura", "aDvan", "aDvana", "kzara", "veda", "bIja", "dIsa",
        "dIpta",
    ],
);

/// *apūpādi-gaṇa* (147), first used in the following *sūtra*:
///
/// > 5.1.4 *vibhāṣā havirapūpādibhyaḥ*
pub(crate) const APUPADI: GanapathaEntry = GanapathaEntry::basic(
    "apUpAdiH",
    147,
    "5.1.4",
    &[
        "apUpa",
        "taRqula",
        "aByuza",
        "aByUza",
        "aByoza",
        "avoza",
        "aByeza",
        "pfTuka",
        "odana",
        "sUpa",
        "pUpa",
        "kiRva",
        "pradIpa",
        "musala",
        "kawaka",
        "karRavezwaka",
        "irgala",
        "argala",
        // <<annavikAreByaSca>>
        "yUpa",
        "sTURA",
        "dIpa",
        "aSva",
        "patra",
    ],
);

/// *niṣkādi-gaṇa* (148), first used in the following *sūtra*:
///
/// > 5.1.20 *asamāsē niṣkādibhyaḥ*
pub(crate) const NISHKADI: GanapathaEntry = GanapathaEntry::basic(
    "nizkAdiH",
    148,
    "5.1.20",
    &["nizka", "paRa", "pAda", "mAza", "vAha", "droRa", "zazwi"],
);

/// *aśvādi-gaṇa* (149), first used in the following *sūtra*:
///
/// > 5.1.39 *gōdvyacō'saṁkhyāparimāṇāśvādēryat*
pub(crate) const ASHVADI_5: GanapathaEntry = GanapathaEntry::basic(
    "aSvAdiH",
    149,
    "5.1.39",
    &[
        "aSva", "aSman", "gaRa", "UrRA", "urma", "umA", "BaNgA", "gaNgA", "varzA", "vasu",
    ],
);

/// *vaṁśādi-gaṇa* (150), first used in the following *sūtra*:
///
/// > 5.1.50 *taddharati vahatyāvahati bhārādvaṁśādibhyaḥ*
pub(crate) const VAMSHADI: GanapathaEntry = GanapathaEntry::basic(
    "vaMSAdiH",
    150,
    "5.1.50",
    &[
        "vaMSa", "kuwaja", "balvaja", "mUla", "sTURA", "sTURa", "akza", "aSman", "aSva",
        "SlakzaRa", "ikzu", "KawvA",
    ],
);

/// *chēdādi-gaṇa* (151), first used in the following *sūtra*:
///
/// > 5.1.64 *chēdādibhyō nityam*
pub(crate) const CHEDADI: GanapathaEntry = GanapathaEntry::basic(
    "CedAdiH",
    151,
    "5.1.64",
    &[
        "Ceda",
        "Beda",
        "droha",
        "doha",
        "narti",
        "narta",
        "karza",
        "tIrTa",
        "saMprayoga",
        "viprayoga",
        "prayoga",
        "viprakarza",
        "prezaRa",
        "saMpraSna",
        "vipraSna",
        "vikarza",
        "virAga",
        "viraNgaM",
        "ca",
    ],
);

/// *daṇḍādi-gaṇa* (152), first used in the following *sūtra*:
///
/// > 5.1.66 *daṇḍādibhyaḥ*
pub(crate) const DANDADI: GanapathaEntry = GanapathaEntry::basic(
    "daRqAdiH",
    152,
    "5.1.66",
    &[
        "daRqa",
        "musala",
        "maDuparka",
        "kaSA",
        "arGa",
        "meGa",
        "meDA",
        "suvarRa",
        "udaka",
        "vaDa",
        "yuga",
        "guhA",
        "BAga",
        "iBa",
        "BaNga",
    ],
);

/// *mahānāmnyādi-gaṇa* (153), first used in the following *sūtra*:
///
/// > 5.1.94 *tadasya brahmacaryam*
/// The *sūtra* has the following *vārttika*:
///
/// > *mahAnAmnyAdiByaH zazWayanteBya upasaMKyAnam*
pub(crate) const MAHANAMNYADI: GanapathaEntry = GanapathaEntry::basic(
    "mahAnAmnyAdiH",
    153,
    "5.1.94",
    &["mahAnAmnI", "Adityavrata", "godAna"],
)
.with_varttika("mahAnAmnyAdiByaH zazWayanteBya upasaMKyAnam");

/// *avāntaradīkṣādi-gaṇa* (154), first used in the following *sūtra*:
///
/// > 5.1.94 *tadasya brahmacaryam*
/// The *sūtra* has the following *vārttika*:
///
/// > *avAntaradIkzAdiByo qernirvaktavyaH*
pub(crate) const AVANTARADIKSHADI: GanapathaEntry = GanapathaEntry::basic(
    "avAntaradIkzAdiH",
    154,
    "5.1.94",
    &["avAntaradIkzA", "tilavrata", "devavrata"],
)
.with_varttika("avAntaradIkzAdiByo qernirvaktavyaH");

/// *vyuṣṭādi-gaṇa* (155), first used in the following *sūtra*:
///
/// > 5.1.97 *vyuṣṭādibhyō'ṇ*
pub(crate) const VYUSHTADI: GanapathaEntry = GanapathaEntry::basic(
    "vyuzwAdiH",
    155,
    "5.1.97",
    &[
        "vyuzwa",
        "nitya",
        "nizkramaRa",
        "praveSana",
        "upasaNkramaRa",
        "tIrTa",
        "AstaraRa",
        "saNgrAma",
        "saNGAta",
    ],
);

/// *agnipadādi-gaṇa* (156), first used in the following *sūtra*:
///
/// > 5.1.97 *vyuṣṭādibhyō'ṇ*
pub(crate) const AGNIPADADI: GanapathaEntry = GanapathaEntry::akrti(
    "agnipadAdiH",
    156,
    "5.1.97",
    &["agnipada", "pIlupada", "pIlumUla", "pravAsa", "upavAsa"],
);

/// *saṁtāpādi-gaṇa* (157), first used in the following *sūtra*:
///
/// > 5.1.101 *tasmai prabhavati saṁtāpādibhyaḥ*
pub(crate) const SAMTAPADI: GanapathaEntry = GanapathaEntry::basic(
    "saMtApAdiH",
    157,
    "5.1.101",
    &[
        "saMtApa",
        "saMnAha",
        "saMgrAma",
        "saMyoga",
        "saMparAya",
        "saMveSana",
        "saMpeza",
        "nizpeza",
        "sarga",
        "nisarga",
        "visarga",
        "upasarga",
        "pravAsa",
        "upavAsa",
        "saNGAsa",
        "saMveza",
        "saMvAsa",
        "samodana",
        "saktu",
        // <<mAMsOdanAdvigfhItAdapi>>
    ],
);

/// *upavastrādi-gaṇa* (158), first used in the following *sūtra*:
///
/// > 5.1.105 *r̥tōraṇ*
/// The *sūtra* has the following *vārttika*:
///
/// > *ftoraR prakaraRe upavastrAdiBya upasaMKyAnam*
pub(crate) const UPAVASTRADI: GanapathaEntry = GanapathaEntry::basic(
    "upavastrAdiH",
    158,
    "5.1.105",
    &["upavastf", "prASitf", "cUqA", "SradDA"],
)
.with_varttika("ftoraR prakaraRe upavastrAdiBya upasaMKyAnam");

/// *anupravacanādi-gaṇa* (159), first used in the following *sūtra*:
///
/// > 5.1.111 *anupravacanādibhyaśchaḥ*
pub(crate) const ANUPRAVACANADI: GanapathaEntry = GanapathaEntry::basic(
    "anupravacanAdiH",
    159,
    "5.1.111",
    &[
        "anupravacana",
        "utTApana",
        "upasTApana",
        "saMveSana",
        "praveSana",
        "anupraveSana",
        "anuvAsana",
        "anuvacana",
        "anuvAcana",
        "anvArohaRa",
        "prAramBaRa",
        "AramBaRa",
        "ArohaRa",
    ],
);

/// *svargādi-gaṇa* (160), first used in the following *sūtra*:
///
/// > 5.1.111 *anupravacanādibhyaśchaḥ*
/// The *sūtra* has the following *vārttika*:
///
/// > *svargAdiByo yadvaktavyaH*
pub(crate) const SVARGADI: GanapathaEntry = GanapathaEntry::basic(
    "svargAdiH",
    160,
    "5.1.111",
    &["svarRa", "yaSas", "Ayus", "kAma", "Dana"],
)
.with_varttika("svargAdiByo yadvaktavyaH");

/// *puṇyāhavācanādi-gaṇa* (161), first used in the following *sūtra*:
///
/// > 5.1.111 *anupravacanādibhyaśchaḥ*
/// The *sūtra* has the following *vārttika*:
///
/// > *puRyAhavAcanAdiByo lugvaktavyaH*
pub(crate) const PUNYAHAVACANADI: GanapathaEntry = GanapathaEntry::basic(
    "puRyAhavAcanAdiH",
    161,
    "5.1.111",
    &["puRyAhavAcana", "svastivAcana", "SAntivAcana"],
)
.with_varttika("puRyAhavAcanAdiByo lugvaktavyaH");

/// *pr̥thvādi-gaṇa* (162), first used in the following *sūtra*:
///
/// > 5.1.122 *pr̥thvādibhya imanijvā*
pub(crate) const PRTHVADI: GanapathaEntry = GanapathaEntry::basic(
    "pfTvAdiH",
    162,
    "5.1.122",
    &[
        "pfTu", "mfdu", "mahat", "pawu", "tanu", "laGu", "bahu", "sADu", "ASu", "uru", "guru",
        "bahula", "KaRqa", "daRqa", "caRqa", "akiMcana", "bAla", "hoqa", "pAka", "vatsa", "manda",
        "svAdu", "hrasva", "dIrGa", "priya", "vfza", "fju", "kzipra", "kzupra", "kzudra", "aRu",
    ],
);

/// *varṇadr̥ḍhādi-gaṇa* (163), first used in the following *sūtra*:
///
/// > 5.1.123 *varṇadr̥ḍhādibhyaḥ ṣyañ ca*
pub(crate) const VARNADRDHADI: GanapathaEntry = GanapathaEntry::basic(
    "varRadfQAdiH",
    163,
    "5.1.123",
    &[
        "dfQa", "vfQa", "parivfQa", "BfSa", "kfSa", "vakra", "Amra", "kfzwa", "lavaRa", "tAmra",
        "SIta", "SIta", "uzRa", "jaqa", "baDira", "paRqita", "maDura", "mUrKa", "mUka", "sTira",
        // <<veryAtalAtamatimanaHSAradAnAm samo matimanasoH>>
        "javana",
    ],
);

/// *brāhmaṇādi-gaṇa* (164), first used in the following *sūtra*:
///
/// > 5.1.124 *guṇavacanabrāhmaṇādibhyaḥ karmaṇi ca*
pub(crate) const BRAHMANADI: GanapathaEntry = GanapathaEntry::akrti(
    "brAhmaRAdiH",
    164,
    "5.1.124",
    &[
        "brAhmaRa",
        "vAqava",
        "mARava",
        // <<arhato num ca>>
        "cora",
        "DUrta",
        "ArADaya",
        "virADaya",
        "aparADaya",
        "uparADaya",
        "ekaBAva",
        "dviBAva",
        "triBAva",
        "anyaBAva",
        "akzetrajYa",
        "saMvAdin",
        "saMveSin",
        "saMBAzin",
        "bahuBAzin",
        "SIrzapAtin",
        "viGAtin",
        "samasTa",
        "vizamasTa",
        "paramasTa",
        "maDyamasTa",
        "anISvara",
        "kuSala",
        "capala",
        "nipuRa",
        "piSuna",
        "kutUhala",
        "kzetrajYa",
        "niSna",
        "bAliSa",
        "alasa",
        "duHpuruza",
        "kApuruza",
        "rAjan",
        "gaRapati",
        "aDipati",
        "gaqula",
        "dAyAda",
        "viSasti",
        "vizama",
        "vipAta",
        // <<sarvavedAdiByaH svArTe>>
        // <<caturvedasyoBayapadavfdDiSca>>
        "SOwIra",
    ],
);

/// *caturvēdādi-gaṇa* (165), first used in the following *sūtra*:
///
/// > 5.1.124 *guṇavacanabrāhmaṇādibhyaḥ karmaṇi ca*
/// The *sūtra* has the following *vārttika*:
///
/// > *caturvedAdiBya uBayapadavfdDiSca*
pub(crate) const CATURVEDADI: GanapathaEntry = GanapathaEntry::basic(
    "caturvedAdiH",
    165,
    "5.1.124",
    &[
        "caturveda",
        "caturvarRa",
        "caturASrama",
        "sarvaviDa",
        "triloka",
        "trisvara",
        "zaqguRa",
        "senA",
        "anantara",
        "saMniDi",
        "samIpa",
        "upamA",
        "muKa",
        "tadarTa",
        "itiha",
        "maRika",
    ],
)
.with_varttika("caturvedAdiBya uBayapadavfdDiSca");

/// *purōhitādi-gaṇa* (166), first used in the following *sūtra*:
///
/// > 5.1.128 *patyantapurōhitādibhyō yak*
pub(crate) const PUROHITADI: GanapathaEntry = GanapathaEntry::basic(
    "purohitAdiH",
    166,
    "5.1.128",
    &[
        "purohita",
        // <<rAjAse>>
        "grAmika",
        "piRqika",
        "suhita",
        "vAlamanda",
        "bAlamanda",
        "KaRqika",
        "daRqika",
        "varmika",
        "karmika",
        "Carmika",
        "Sitika",
        "mUtika",
        "mUlika",
        "tilaka",
        "aYjalika",
        "antalika",
        "rUpika",
        "fzika",
        "patrika",
        "paTika",
        "carmika",
        "pratika",
        "sAraTi",
        "AsTika",
        "sUcika",
        "saMrakza",
        "sUcaka",
        "saMrakzasUcaka",
        "nAstika",
        "ajAnika",
        "SAkvara",
        "nAgara",
        "cUqika",
    ],
);

/// *udgātrādi-gaṇa* (167), first used in the following *sūtra*:
///
/// > 5.1.129 *prāṇabhr̥jjātivayōvacanōdgātrādibhyō'ñ*
pub(crate) const UDGATRADI: GanapathaEntry = GanapathaEntry::basic(
    "udgAtrAdiH",
    167,
    "5.1.129",
    &[
        "udgAtf",
        "unnetf",
        "pratihartf",
        "praSastf",
        "hotf",
        "potf",
        "hartf",
        "raTagaRaka",
        "pakzigaRaka",
        "suzWu",
        "duzWu",
        "aDvaryu",
        "vaDU",
        "suBaga",
        "mantre",
    ],
);

/// *yuvādi-gaṇa* (168), first used in the following *sūtra*:
///
/// > 5.1.130 *hāyanāntayuvādibhyō'ṇ*
pub(crate) const YUVADI: GanapathaEntry = GanapathaEntry::basic(
    "yuvAdiH",
    168,
    "5.1.130",
    &[
        "yuvan",
        "sTavira",
        "hotf",
        "yajamAna",
        "BrAtf",
        "kutuka",
        "SramaRa",
        "SravaRa",
        "kawuka",
        "kamaRqalu",
        "kustrI",
        "sustri",
        "daHstrI",
        "suhfdaya",
        "durhfdaya",
        "suhfd",
        "durhft",
        "suBrAtf",
        "durBrAtf",
        "vfzala",
        "parivrAjaka",
        "sabrahmacArin",
        "anfSaMsa",
        "hfdayAse",
        "kuSala",
        "capala",
        "nipuRa",
        "piSuna",
        "kutUhala",
        "kzetrajYa",
        // <<Srotriyasya yalopaSca>>
    ],
);

/// *manōjñādi-gaṇa* (169), first used in the following *sūtra*:
///
/// > 5.1.133 *dvandvamanōjñādibhyaśca*
pub(crate) const MANOJNADI: GanapathaEntry = GanapathaEntry::basic(
    "manojYAdiH",
    169,
    "5.1.133",
    &[
        "manojYa",
        "priyarUpa",
        "aBirUpa",
        "kalyARa",
        "meDAvin",
        "AQaya",
        "kulaputra",
        "CAndasa",
        "CAtra",
        "Srotriya",
        "cora",
        "DUrta",
        "viSvadeva",
        "yuvan",
        "kuputra",
        "grAmaputra",
        "grAmakulAla",
        "grAmaqa",
        "grAmazaRqa",
        "grAmakumAra",
        "sukumAra",
        "bahula",
        "avaSyaputra",
        "amuzyaputra",
        "amuzyakula",
        "sAraputra",
        "Sataputra",
    ],
);

/// *pīlvādi-gaṇa* (170), first used in the following *sūtra*:
///
/// > 5.2.24 *tasya pākamūlē pīlvādikarṇādibhyaḥ kuṇabjāhacau*
pub(crate) const PILVADI: GanapathaEntry = GanapathaEntry::basic(
    "pIlvAdiH",
    170,
    "5.2.24",
    &[
        "pIlu", "karkanDU", "karkanDu", "SamI", "karIra", "bala", "kuvala", "badara", "aSvatTa",
        "Kadira",
    ],
);

/// *karṇādi-gaṇa* (171), first used in the following *sūtra*:
///
/// > 5.2.24 *tasya pākamūlē pīlvādikarṇādibhyaḥ kuṇabjāhacau*
pub(crate) const KARNADI_5: GanapathaEntry = GanapathaEntry::basic(
    "karRAdiH",
    171,
    "5.2.24",
    &[
        "karRa", "akzi", "naKa", "muKa", "maKa", "keSa", "pAda", "gulPa", "BrUBaNga", "danta",
        "ozWa", "pfzWa", "aNguzWa",
    ],
);

/// *tārakādi-gaṇa* (172), first used in the following *sūtra*:
///
/// > 5.2.36 *tadasya saṁjātaṁ tārakādibhya itac*
pub(crate) const TARAKADI: GanapathaEntry = GanapathaEntry::basic(
    "tArakAdiH",
    172,
    "5.2.36",
    &[
        "tArakA",
        "puzpa",
        "karRaka",
        "maYjarI",
        "fjIza",
        "kzaRa",
        "sUtra",
        "mUtra",
        "nizkramaRa",
        "purIza",
        "uccAra",
        "pracAra",
        "vicAra",
        "kuqmala",
        "kaRwaka",
        "musala",
        "mukula",
        "kusuma",
        "kutUhala",
        "stabaka",
        "stavaka",
        "kisalaya",
        "pallava",
        "KaRqa",
        "vega",
        "nidrA",
        "mudrA",
        "buBukzA",
        "DenuzyA",
        "pipAsA",
        "SradDA",
        "aBra",
        "pulaka",
        "aNgAraka",
        "varRaka",
        "droha",
        "doha",
        "suKa",
        "duHKa",
        "utkaRWA",
        "Bara",
        "vyADi",
        "varman",
        "vraRa",
        "gOrava",
        "SAstra",
        "taraNga",
        "tilaka",
        "candraka",
        "anDakAra",
        "garva",
        "kumura",
        "mukura",
        "harza",
        "utkarza",
        "raRa",
        "kuvalaya",
        "garGa",
        "kzuD",
        "sImanta",
        "jvara",
        "gara",
        "roga",
        "romAYca",
        "paRqA",
        "kujjala",
        "tfz",
        "koraka",
        "kallola",
        "sTapuwa",
        "Pala",
        "kaYcuka",
        "SfNgAra",
        "aNkura",
        "SEvala",
        "bakula",
        "Svamra",
        "ArAla",
        "kalaNka",
        "kardama",
        "kandala",
        "mUrcCA",
        "aNgAra",
        "hastaka",
        "pribimba",
        "viGnatantra",
        "pratyaya",
        "dIkzA",
        "garja",
        // <<garBAdaprARini>>
    ],
);

/// *vimuktādi-gaṇa* (173), first used in the following *sūtra*:
///
/// > 5.2.61 *vimuktādibhyō'ṇ*
pub(crate) const VIMUKTADI: GanapathaEntry = GanapathaEntry::basic(
    "vimuktAdiH",
    173,
    "5.2.61",
    &[
        "vimukta",
        "devAsura",
        "rakzosura",
        "upasad",
        "suvarRa",
        "parisAraka",
        "sadasat",
        "vasu",
        "marut",
        "patnIvat",
        "vasumat",
        "mahIyas",
        "satvat",
        "barhavat",
        "daSArRa",
        "daSArha",
        "vayas",
        "havirDAna",
        "patatrin",
        "mahitrI",
        "asyahatya",
        "somApUzan",
        "iqA",
        "agnAvizRU",
        "urvaSI",
        "vftrahan",
    ],
);

/// *gōṣadādi-gaṇa* (174), first used in the following *sūtra*:
///
/// > 5.2.62 *gōṣadādibhyō vun*
pub(crate) const GOSHADADI: GanapathaEntry = GanapathaEntry::basic(
    "gozadAdiH",
    174,
    "5.2.62",
    &[
        "gozada",
        "gozad",
        "izetvA",
        "mAtariSvan",
        "devasyatvA",
        "devIrApaH",
        // <<kfzRo'syAKyarezwaH>>
        "devIMDiyA",
        "devIMDiya",
        "rakzohaRa",
        "yuYjAna",
        "aYjana",
        "prasUta",
        "praBUta",
        "pratUrta",
        "kfSAnu",
        "kfSAku",
    ],
);

/// *ākarṣādi-gaṇa* (175), first used in the following *sūtra*:
///
/// > 5.2.64 *ākarṣādibhyaḥ kan*
pub(crate) const AKARSHADI: GanapathaEntry = GanapathaEntry::basic(
    "AkarzAdiH",
    175,
    "5.2.64",
    &[
        "Akarza",
        "Akaza",
        "tsaru",
        "piSAca",
        "picaRqa",
        "aSani",
        "aSman",
        "nicaya",
        "caya",
        "vijaya",
        "jaya",
        "Acaya",
        "naya",
        "pAda",
        "dIpahrada",
        "hrAda",
        "hlAda",
        "gadgada",
        "Sakuni",
    ],
);

/// *iṣṭādi-gaṇa* (176), first used in the following *sūtra*:
///
/// > 5.2.88 *iṣṭādibhyaśca*
pub(crate) const ISHTADI: GanapathaEntry = GanapathaEntry::basic(
    "izwAdiH",
    176,
    "5.2.88",
    &[
        "izwa",
        "pUrta",
        "upAsAdita",
        "nigadita",
        "parigAdita",
        "parivAdita",
        "nikaTita",
        "nizAdita",
        "nipaWita",
        "sakalita",
        "parikalita",
        "saMrakzita",
        "parirakzita",
        "Arcita",
        "gaRita",
        "avakIrRa",
        "Ayukta",
        "gfhIta",
        "AAmnAta",
        "Sruta",
        "aDIta",
        "avaDAna",
        "Asevita",
        "avaDArita",
        "avakalpita",
        "nirAkfta",
        "upakfta",
        "upAkfta",
        "anuyukta",
        "anugaRita",
        "anupaWita",
        "vyAkulita",
    ],
);

/// *rasādi-gaṇa* (177), first used in the following *sūtra*:
///
/// > 5.2.95 *rasādibhyaśca*
pub(crate) const RASADI: GanapathaEntry = GanapathaEntry::basic(
    "rasAdiH",
    177,
    "5.2.95",
    &[
        "rasa", "rUpa", "varRa", "ganDa", "sparSa", "Sabda", "sneha", "BAva", "guRAt", "ekAcaH",
    ],
);

/// *sidhmādi-gaṇa* (178), first used in the following *sūtra*:
///
/// > 5.2.97 *sidhmādibhyaśca*
pub(crate) const SIDHMADI: GanapathaEntry = GanapathaEntry::basic(
    "siDmAdiH",
    178,
    "5.2.97",
    &[
        "siDma", "gaqu", "maRi", "nABi", "bIja", "vIRA", "kfzRa", "nizpAva", "pAMsu", "pArSva",
        "parSu", "hanu", "saktu", "mAsa", "mAMsa",
        // <<pArzRiDamanyordIrGaSca>>
        // <<vAtadantabalalalAwAnAmUN ca>>
        // <<jawAGawAkawAkAlAH kzepe>>
        "parRa", "udaka", "prajA", "sakTi", "karRa", "sneha", "SIta", "SyAma", "piNga", "pitta",
        "puzPa", "pfTu", "mfdu", "maYju", "gaRqa", "patra", "cawu", "kapi", "kaRqu", "granTi",
        "SrI", "kuSa", "DArA", "varzman", "pakzman", "Slezman", "peSa", "nizpAd",
        "kuRqa",
        // <<kzudrajantUpatApayoSca>>
    ],
);

/// *lōmādi-gaṇa* (179), first used in the following *sūtra*:
///
/// > 5.2.100 *lōmādipāmādipicchādibhyaḥ śanēlacaḥ*
pub(crate) const LOMADI: GanapathaEntry = GanapathaEntry::basic(
    "lomAdiH",
    179,
    "5.2.100",
    &[
        "loman", "roman", "baBru", "hari", "giri", "karka", "kapi", "muni", "taru",
    ],
);

/// *pāmādi-gaṇa* (180), first used in the following *sūtra*:
///
/// > 5.2.100 *lōmādipāmādipicchādibhyaḥ śanēlacaḥ*
pub(crate) const PAMADI: GanapathaEntry = GanapathaEntry::basic(
    "pAmAdiH",
    180,
    "5.2.100",
    &[
        "pAman", "vAman", "veman", "heman", "Slezman", "kadru", "kadrU", "bali", "sAman", "Uzman",
        "kfmi",
        // <<aNgAtkalyARe>>
        // <<SAkIpalAlIdadrURAM hrasvatvaM ca>>
        // <<vizvagityuttarapadalopaScAkftasaMDeH>>
        // <<lakzmyA acca>>
    ],
);

/// *picchādi-gaṇa* (181), first used in the following *sūtra*:
///
/// > 5.2.100 *lōmādipāmādipicchādibhyaḥ śanēlacaḥ*
pub(crate) const PICCHADI: GanapathaEntry = GanapathaEntry::basic(
    "picCAdiH",
    181,
    "5.2.100",
    &[
        "picCa", "uras", "Duvaka", "Druvaka", // <<jawAGawAkAlAH kzepe>>
        "varRa", "udaka", "paNka", "prajYA",
    ],
);

/// *jyōtsnādi-gaṇa* (182), first used in the following *sūtra*:
///
/// > 5.2.103 *aṇ ca*
/// The *sūtra* has the following *vārttika*:
///
/// > *jyotsnAdiBya upasaMKyAnam*
pub(crate) const JYOTSNADI: GanapathaEntry = GanapathaEntry::basic(
    "jyotsnAdiH",
    182,
    "5.2.103",
    &[
        "jyotsnA", "tamisnA", "kuRqala", "kutapa", "visarpa", "vipAdikA",
    ],
)
.with_varttika("jyotsnAdiBya upasaMKyAnam");

/// *vrīhyādi-gaṇa* (183), first used in the following *sūtra*:
///
/// > 5.2.116 *vrīhyādibhyaśca*
pub(crate) const VRIHYADI: GanapathaEntry = GanapathaEntry::basic(
    "vrIhyAdiH",
    183,
    "5.2.116",
    &[
        "vrIhI", "mAyA", "SAlA", "SiKA", "mAlA", "meKalA", "kekA", "azwakA", "patAkA", "carman",
        "karman", "varmin", "daMzwrA", "saMjYA", "vaqavA", "kumArI", "nO", "vIRA", "balAkA",
        "yavaKada",
        // <<SIrzAnnaYaH>>
    ],
);

/// *tundādi-gaṇa* (184), first used in the following *sūtra*:
///
/// > 5.2.117 *tundādibhya ilac ca*
pub(crate) const TUNDADI: GanapathaEntry = GanapathaEntry::basic(
    "tundAdiH",
    184,
    "5.2.117",
    &[
        "tunda", "udara", "picaRqa", "yava", "vrIhi",
        // <<svANgAdvivfdDO>>
    ],
);

/// *arśaādi-gaṇa* (185), first used in the following *sūtra*:
///
/// > 5.2.127 *arśaādibhyō'c*
pub(crate) const ARSHAADI: GanapathaEntry = GanapathaEntry::basic(
    "arSaAdiH",
    185,
    "5.2.127",
    &[
        "arSas", "uras", "tunda", "catura", "kalita", "jawA", "GawA", "GAwA", "aBra", "aGa",
        "kardama", "amla", "lavaRa", // <<svANgAdDInAt>>
        "varRAt",
    ],
);

/// *sukhādi-gaṇa* (186), first used in the following *sūtra*:
///
/// > 5.2.131 *sukhādibhyaśca*
pub(crate) const SUKHADI_5: GanapathaEntry = GanapathaEntry::basic(
    "suKAdiH",
    186,
    "5.2.131",
    &[
        "suKa", "duHKa", "tfpta", "tfpra", "kfcCra", "asna", "ASra", "Asna", "alIka", "kaWiRa",
        "soQa", "pratIpa", "SIla", "hala", "mAlA", "kzepe", "kfpaRa", "praRAya", "praRaya", "dala",
        "kakza",
    ],
);

/// *puṣkarādi-gaṇa* (187), first used in the following *sūtra*:
///
/// > 5.2.135 *puṣkarādibhyō dēśē*
pub(crate) const PUSHKARADI: GanapathaEntry = GanapathaEntry::basic(
    "puzkarAdiH",
    187,
    "5.2.135",
    &[
        "puzkara", "padma", "utpala", "tamAla", "kumuda", "naqa", "kapitTa", "bisa", "mfRAla",
        "kardama", "SAlUka", "vigarha", "karIza", "SirIza", "yavAsa", "pravAsa", "hiraRya",
        "kErava", "kallola", "tawa", "taraNga", "paNkaja", "saroja", "rAjIva", "saroruha",
        "puwaka", "aravinda", "amBoja", "abja", "kamala", "kallola", "payas",
    ],
);

/// *balādi-gaṇa* (188), first used in the following *sūtra*:
///
/// > 5.2.136 *balādibhyō matubanyatarasyām*
pub(crate) const BALADI_5: GanapathaEntry = GanapathaEntry::basic(
    "balAdiH",
    188,
    "5.2.136",
    &[
        "bala", "utsAha", "udBAva", "udvAsa", "udvAma", "SiKA", "kula", "cUqA", "sula", "kUla",
        "AyAma", "vyAyAma", "upayAma", "Aroha", "avaroha", "pariRAha", "yudDa",
    ],
);

/// *bhavadādi-gaṇa* (189), first used in the following *sūtra*:
///
/// > 5.3.14 *itarābhyō'pi dr̥śyantē*
/// The *sūtra* has the following *vārttika*:
///
/// > *dfSigrahaRAdBavadAdiyoga eva*
pub(crate) const BHAVADADI: GanapathaEntry = GanapathaEntry::basic(
    "BavadAdiH",
    189,
    "5.3.14",
    &["BavAn", "dIrGAyuH", "devAnAMpriyaH", "AyuzmAn"],
)
.with_varttika("dfSigrahaRAdBavadAdiyoga eva");

/// *dēvapathādi-gaṇa* (190), first used in the following *sūtra*:
///
/// > 5.3.100 *dēvapathādibhyaśca*
pub(crate) const DEVAPATHADI: GanapathaEntry = GanapathaEntry::basic(
    "devapaTAdiH",
    190,
    "5.3.100",
    &[
        "devapaTa",
        "haMsapaTa",
        "vAripaTa",
        "raTapaTa",
        "sTalapaTa",
        "karipaTa",
        "ajapaTa",
        "rAjapaTa",
        "SatapaTa",
        "SaNkupaTa",
        "minDupaTa",
        "sidDagati",
        "uzwragrIvA",
        "vAmarajju",
        "hasta",
        "indra",
        "daRqa",
        "puzpa",
        "matsya",
    ],
);

/// *śākhādi-gaṇa* (191), first used in the following *sūtra*:
///
/// > 5.3.103 *śākhādibhyō yat*
pub(crate) const SHAKHADI: GanapathaEntry = GanapathaEntry::basic(
    "SAKAdiH",
    191,
    "5.3.103",
    &[
        "SAKA", "muKa", "jaGana", "SfNga", "meGa", "aBra", "caraRa", "skanDa", "skada", "skanda",
        "uras", "Siras", "agra", "SARa", "SaraRa",
    ],
);

/// *śarkarādi-gaṇa* (192), first used in the following *sūtra*:
///
/// > 5.3.107 *śarkarādibhyō'ṇ*
pub(crate) const SHARKARADI: GanapathaEntry = GanapathaEntry::basic(
    "SarkarAdiH",
    192,
    "5.3.107",
    &[
        "SarkarA",
        "kapAlikA",
        "kapAwikA",
        "kapizWikA",
        "kanizWikA",
        "puRqarIka",
        "Satapatra",
        "goloman",
        "loman",
        "gopucCa",
        "narAcI",
        "nakula",
        "sikatA",
    ],
);

/// *aṅgulyādi-gaṇa* (193), first used in the following *sūtra*:
///
/// > 5.3.108 *aṅgulyādibhyaṣṭhak*
pub(crate) const ANGULYADI: GanapathaEntry = GanapathaEntry::basic(
    "aNgulyAdiH",
    193,
    "5.3.108",
    &[
        "aNguli", "Baruja", "baBru", "valgu", "maRqara", "maRqala", "SazkulI", "hari", "kapi",
        "muni", "ruha", "Kala", "udaSvit", "goRI", "uras", "kuliSa",
    ],
);

/// *dāmanyādi-gaṇa* (194), first used in the following *sūtra*:
///
/// > 5.3.116 *dāmanyāditrigartaṣaṣṭhācchaḥ*
pub(crate) const DAMANYADI: GanapathaEntry = GanapathaEntry::basic(
    "dAmanyAdiH",
    194,
    "5.3.116",
    &[
        "dAmanI",
        "Olapi",
        "bEjavApi",
        "Odaki",
        "OdaNki",
        "acyutanti",
        "Acyutanti",
        "acyutadanti",
        "Acyutadanti",
        "SAkuntaki",
        "AkidantI",
        "Akvidanti",
        "Oqavi",
        "kAkadantaki",
        "SatruMtapi",
        "sArvaseni",
        "bindu",
        "bEndavi",
        "tulaBa",
        "mOYjAyana",
        "kAkandi",
        "ulaBa",
        "sAvitrIputra",
    ],
);

/// *parśvādi-gaṇa* (195), first used in the following *sūtra*:
///
/// > 5.3.117 *parśvādiyaudhēyādibhyāmaṇañau*
pub(crate) const PARSHVADI_5: GanapathaEntry = GanapathaEntry::basic(
    "parSvAdiH",
    195,
    "5.3.117",
    &[
        "parSu",
        "asura",
        "rakzas",
        "bAhlIka",
        "vayas",
        "vasu",
        "marut",
        "satvat",
        "daSArha",
        "piSAca",
        "aSani",
        "kArzApaRa",
    ],
);

/// *yaudhēyādi-gaṇa* (196), first used in the following *sūtra*:
///
/// > 5.3.117 *parśvādiyaudhēyādibhyāmaṇañau*
pub(crate) const YAUDHEYADI_5: GanapathaEntry = GanapathaEntry::basic(
    "yODeyAdiH",
    196,
    "5.3.117",
    &[
        "yODeya",
        "krOSeya",
        "SOkeya",
        "SOBreya",
        "DArteya",
        "GArteya",
        "jyAvAReya",
        "trigarta",
        "Barata",
        "uSInara",
    ],
);

/// *sthūlādi-gaṇa* (197), first used in the following *sūtra*:
///
/// > 5.4.3 *sthūlādibhyaḥ prakāravacanē kan*
pub(crate) const STHULADI: GanapathaEntry = GanapathaEntry::basic(
    "sTUlAdiH",
    197,
    "5.4.3",
    &[
        "sTUla",
        "aRu",
        "mAza",
        "izu",
        // <<kfzRa tilezu>>
        // <<yava vrIhizu>>
        // <<pAdyakAlAvadAtasurAyAm>>
        // <<gomUtra AcCAdane>>
        // <<surAyA ahO>>
        // <<jIrRaSAlizu>>
        // <<patramUla  samastavyaste>>
        "kumArIputra",
        "kumArISvaSura",
        "maRi",
    ],
);

/// *yāvādi-gaṇa* (198), first used in the following *sūtra*:
///
/// > 5.4.29 *yāvādibhyaḥ kan*
pub(crate) const YAVADI: GanapathaEntry = GanapathaEntry::basic(
    "yAvAdiH",
    198,
    "5.4.29",
    &[
        "yAva", "maRi", "asTi", "tAlu", "jAnu", "sAndra", "pIta", "stamba",
        // <<ftA uzRaSIte>>
        // <<paSO lUnavipAte>>
        // <<aRu nipuRe>>
        // <<putra kftrime>>
        // <<snAta vedasamAptO>>
        // <<SUnya rikte>>
        // <<dAna kutsite>>
        // <<tanu sUtre>>
        // <<IyasaSca>>
        "jYAta", "ajYAta",
        // <<kumArIkrIqanakAni ca>>
        // <<kumArakrIqanakAni ca>>
    ],
);

/// *vinayādi-gaṇa* (199), first used in the following *sūtra*:
///
/// > 5.4.34 *vinayādibhyaṣṭhak*
pub(crate) const VINAYADI: GanapathaEntry = GanapathaEntry::basic(
    "vinayAdiH",
    199,
    "5.4.34",
    &[
        "vinaya",
        "samaya",
        "upAyo",
        "hrasvatvaM",
        "ca",
        "saMprati",
        "saMgati",
        "kaTaMcit",
        "akasmAt",
        "samAcAra",
        "upacAra",
        "samAya",
        "samayAcAra",
        "vyavahAra",
        "saMpradAna",
        "samutkarza",
        "samUha",
        "viSeza",
        "atyaya",
    ],
);

/// *prajñādi-gaṇa* (200), first used in the following *sūtra*:
///
/// > 5.4.38 *prajñādibhyaśca*
pub(crate) const PRAJNADI: GanapathaEntry = GanapathaEntry::basic(
    "prajYAdiH",
    200,
    "5.4.38",
    &[
        "prajYa",
        "vaRij",
        "uSij",
        "uzRij",
        "pratyakza",
        "vidvas",
        "vidan",
        "zoqan",
        "vidyA",
        "manas",
        "Srotra",
        "SarIre",
        "juhvat",
        "kfzRa",
        "mfge",
        "cikIrzat",
        "cora",
        "Satra",
        "yoDa",
        "cakzus",
        "vasu",
        "enas",
        "marut",
        "kruYca",
        "sattvat",
        "daSArha",
        "vayas",
        "vyAkfta",
        "asura",
        "rakzas",
        "piSAca",
        "aSani",
        "kArzApaRa",
        "devatA",
        "banDu",
    ],
);

/// *ādyādi-gaṇa* (201), first used in the following *sūtra*:
///
/// > 5.4.44 *pratiyōgē pañcamyāstasiḥ*
/// The *sūtra* has the following *vārttika*:
///
/// > *AdyAdiBya upasaMKyAnam*
pub(crate) const ADYADI: GanapathaEntry = GanapathaEntry::basic(
    "AdyAdiH",
    201,
    "5.4.44",
    &["Adi", "maDya", "anta", "pfzWa", "pArSva"],
)
.with_varttika("AdyAdiBya upasaMKyAnam");

/// *śaradādi-gaṇa* (202), first used in the following *sūtra*:
///
/// > 5.4.107 *avyayībhāvē śaratprabhr̥tibhyaḥ*
pub(crate) const SHARADADI: GanapathaEntry = GanapathaEntry::basic(
    "SaradAdiH",
    202,
    "5.4.107",
    &[
        "Sarad", "vipAS", "anas", "manas", "upAnah", "anaquh", "div", "himavat", "hiruk", "vid",
        "sad", "diS", "dfS", "viS", "catur", "tyad", "tad", "yad", "etad", "kiyat",
        // <<jarAyA jaras ca>>
        // <<pratiparasamanuByo'kzRaH>>
        "paTin",
    ],
);

/// *dvidaṇḍyādi-gaṇa* (203), first used in the following *sūtra*:
///
/// > 5.4.128 *dvidaṇḍyādibhyaśca*
pub(crate) const DVIDANDYADI: GanapathaEntry = GanapathaEntry::basic(
    "dvidaRqyAdiH",
    203,
    "5.4.128",
    &[
        "dvidaRqi",
        "dvimusali",
        "uBAYjali",
        "uBayAYjali",
        "uBAdanti",
        "uBayAdanti",
        "uBAhasti",
        "uBayAhasti",
        "uBAkarRi",
        "uBayAkarRi",
        "uBApARi",
        "uBayApARi",
        "uBAbAhu",
        "uBayAbAhu",
        "ekapadI",
        "prozWapadI",
        "AcyapadI",
        "AQayapadi",
        "sapati",
        "nikucyakarRi",
        "saMhatapucCi",
        "antekasI",
    ],
);

/// *hastyādi-gaṇa* (204), first used in the following *sūtra*:
///
/// > 5.4.138 *pādasya lōpō'hastyādibhyaḥ*
pub(crate) const HASTYADI: GanapathaEntry = GanapathaEntry::basic(
    "hastyAdiH",
    204,
    "5.4.138",
    &[
        "hastin",
        "kuddAla",
        "aSva",
        "kaSika",
        "kuruta",
        "kawola",
        "kawolaka",
        "gaRqola",
        "gaRqolaka",
        "kaRqola",
        "kaRqolaka",
        "aja",
        "kapota",
        "jAla",
        "gaRqa",
        "mahilA",
        "dAsI",
        "gaRikA",
        "kusUla",
    ],
);

/// *kumbhapadyādi-gaṇa* (205), first used in the following *sūtra*:
///
/// > 5.4.139 *kumbhapadīṣu ca*
pub(crate) const KUMBHAPADYADI: GanapathaEntry = GanapathaEntry::basic(
    "kumBapadyAdiH",
    205,
    "5.4.139",
    &[
        "kumBapadI",
        "ekapadI",
        "jAlapadI",
        "SUlapadI",
        "munipadI",
        "guRapadI",
        "SatapadI",
        "sUtrapadI",
        "goDApadI",
        "kalaSIpadI",
        "vipadI",
        "tfRapadI",
        "dvipadI",
        "tripadI",
        "zawpadI",
        "dAsIpadI",
        "SitipadI",
        "vizRupadI",
        "supadI",
        "nizpadI",
        "ArdrapadI",
        "kuRipadI",
        "kfzRapadI",
        "SucipadI",
        "droRIpadI",
        "droRapadI",
        "drupadI",
        "sUkarapadI",
        "SakftpadI",
        "azwApadI",
        "sTURapadI",
        "apadI",
        "sUcIpadI",
    ],
);

/// *uraḥprabhr̥taya-gaṇa* (206), first used in the following *sūtra*:
///
/// > 5.4.151 *uraḥprabhr̥tibhyaḥ kap*
pub(crate) const URAHPRABHRTAYAH: GanapathaEntry = GanapathaEntry::basic(
    "uraHpraBftayaH",
    206,
    "5.4.151",
    &[
        "uras", "sarpiz", "upAnaqa", "pumAn", "anaqvAn", "payaH", "nOH", "lakzmIH", "daDi", "maDu",
        "SAlI", "SAliH",
    ],
);

/// *śakandhvādi-gaṇa* (207), first used in the following *sūtra*:
///
/// > 6.1.94 *ēṅi pararūpam*
/// The *sūtra* has the following *vārttika*:
///
/// > *SakanDvAdizu pararUpaM vAcyam*
pub(crate) const SHAKANDHVADI: GanapathaEntry = GanapathaEntry::basic(
    "SakanDvAdiH",
    207,
    "6.1.94",
    &[
        "SakanDu",
        "karkanDu",
        "kulawA",
        "sImantaH",
        "keSaveSe",
        "halIzA",
        "manIzA",
        "lANgalIzA",
        "pataYjaliH",
        "sAraNgaH",
        "paSupakziRoH",
    ],
)
.with_varttika("SakanDvAdizu pararUpaM vAcyam");

/// *pāraskarādi-gaṇa* (208), first used in the following *sūtra*:
///
/// > 6.1.157 *pāraskaraprabhr̥tīni ca saṁjñāyām*
pub(crate) const PARASKARADI: GanapathaEntry = GanapathaEntry::basic(
    "pAraskarAdiH",
    208,
    "6.1.157",
    &[
    // <<pAraskaro deSaH>>
    // <<kAraskaro vfkzaH>>
    // <<raTasyA nadI>>
    // <<rayasTA nadI>>
    // <<kizkuH pramARam>>
    // <<kizkinDA guhA>>
    // <<tadbfhatoH karapatyoScoradevatayoH suw talopaSca>>
    // <<prAttumpatO gavi kartari>>
],
);

/// *uñchādi-gaṇa* (209), first used in the following *sūtra*:
///
/// > 6.1.160 *uñchādīnāṁ ca*
pub(crate) const UNCHADI: GanapathaEntry = GanapathaEntry::basic(
    "uYCAdiH",
    209,
    "6.1.160",
    &[
        "uYCa", "mlecCa", "jaYja", "talpa", "jalpa", "japa",
        "vaDa",
        // <<yuga kAlaviSeze raTAyadyupakaraRe ca>>
        // <<garo dUzye>>
        // <<garo qUzye>>
        // <<abantaH vedavegavezwabanDAH karaRe>>
        // <<stuTudruvaSCandasi>>
        // <<vartani stotre>>
        // <<SvaBre daraH>>
        // <<sAmbatApo BAvagarhAyAm>>
        // <<uttamaSaSvattamO sarvatra>>
        // <<BakzamanTaBogamanTAH dehAH>>
    ],
);

/// *vr̥ṣādi-gaṇa* (210), first used in the following *sūtra*:
///
/// > 6.1.203 *vr̥ṣādīnāṁ ca*
pub(crate) const VRSHADI: GanapathaEntry = GanapathaEntry::basic(
    "vfzAdiH",
    210,
    "6.1.203",
    &[
        "vfzaH", "janaH", "jvaraH", "grahaH", "hayaH", "mayaH", "gayaH", "tAyaH", "tayaH", "cayaH",
        "amaH", "vedaH", "sUdaH", "aMSaH", "guhA",
        // <<SamaraRO saMjYAyAM saMmatO BAvakarmaRoH>>
        "mantraH", "SAntiH", "kAmaH", "yAmaH", "ArA", "DArA", "kArA", "vahaH", "kalpaH",
        "pAdaH",
        // <<avihitalakzaRamAdyudAttatvaM vfzAdizu jYeyam>>
    ],
);

/// *vispaṣṭādi-gaṇa* (211), first used in the following *sūtra*:
///
/// > 6.2.24 *vispaṣṭādīni guṇavacanēṣu*
pub(crate) const VISPASHTADI: GanapathaEntry = GanapathaEntry::basic(
    "vispazwAdiH",
    211,
    "6.2.24",
    &[
        "vispazwa", "vicitra", "vicitta", "vyakta", "sampanna", "pawu", "paRqita", "kuSala",
        "capala", "nipuRa",
    ],
);

/// *kārtakaujapādi-gaṇa* (212), first used in the following *sūtra*:
///
/// > 6.2.37 *kārtakaujapādayaśca*
pub(crate) const KARTAKAUJAPADI: GanapathaEntry = GanapathaEntry::basic(
    "kArtakOjapAdiH",
    212,
    "6.2.37",
    &[
        "kArtakOjapO",
        "sAvarRimARqakeyO",
        "sAvarRimARqUkeyO",
        "avantyaSmakAH",
        "pElaSyAparReyAH",
        "kapiSyAparReyAH",
        "SEtikAkzapAYcAleyAH",
        "kawukavADUleyAH",
        "SAkalaSunakAH",
        "SAkalaSaRakAH",
        "SaRakabABravAH",
        "ArcABimOdgalAH",
        "kuntisurAzwrAH",
        "cintisurAzwrAH",
        "taRqavataRqAH",
        "avimattakAmavidDAH",
        "bABravaSAlaNkAyanAH",
        "bABravadAnacyutAH",
        "kaWakAlApAH",
        "kaWakOTumAH",
        "kOTumalOkAkzAH",
        "strIkumAram",
        "mOdapEppalAdAH",
        "vatsajarantaH",
        "sOSrutapArTavAH",
        "jarAmftyU",
        "yAjyAnuvAkye",
    ],
);

/// *kurugārhapatādi-gaṇa* (213), first used in the following *sūtra*:
///
/// > 6.2.42 *kurugārhapatariktagurvasūtajaratyaślīladr̥ḍharūpāpārēvaḍavātaitilakadrūḥpaṇyakambalō dāsībhārāṇāṁ ca*
pub(crate) const KURUGARHAPATADI: GanapathaEntry = GanapathaEntry::basic(
    "kurugArhapatAdiH",
    213,
    "6.2.42",
    &[
        "dAsIBAraH",
        "devahUtiH",
        "devaBItiH",
        "devalAtiH",
        "vasunItiH",
        "vasUnitiH",
        "OzaDiH",
        "candramAH",
    ],
);

/// *yuktārōhyādi-gaṇa* (214), first used in the following *sūtra*:
///
/// > 6.2.81 *yuktārōhyādayaśca*
pub(crate) const YUKTAROHYADI: GanapathaEntry = GanapathaEntry::akrti(
    "yuktArohyAdiH",
    214,
    "6.2.81",
    &[
        "yuktArohI",
        "AnatarohI",
        "AgatayoDI",
        "AgatavaYcI",
        "AgatanandI",
        "AgataprahArI",
        "AgatamatsyaH",
        "kzIrahotA",
        "BaginIBartA",
        "grAmagoDuk",
        "aSvatrirAtraH",
        "gargatrirAtraH",
        "vyuzwatrirAtraH",
        "gaRapAdaH",
        "ekaSitipAd",
        // <<pAtresamitAdayaSca>>
    ],
);

/// *ghōṣādi-gaṇa* (215), first used in the following *sūtra*:
///
/// > 6.2.85 *ghōṣādiṣu ca*
pub(crate) const GHOSHADI: GanapathaEntry = GanapathaEntry::basic(
    "GozAdiH",
    215,
    "6.2.85",
    &[
        "Goza", "Gawa", "kawa", "vallaBa", "hfda", "badari", "piNgala", "piNgalI", "piSaNga",
        "mAlA", "rakzA", "SAlA", "vfw", "kUwa", "SAlmalI", "aSvatTa", "tfRa", "SilpI", "muni",
        "prekzAka", "prekzA",
    ],
);

/// *chātryādi-gaṇa* (216), first used in the following *sūtra*:
///
/// > 6.2.86 *chātryādayaḥ śālāyām*
pub(crate) const CHATRYADI: GanapathaEntry = GanapathaEntry::basic(
    "CAtryAdiH",
    216,
    "6.2.86",
    &["CAtri", "peli", "BARqi", "vyAqi", "AKaRqi", "Awi", "gomi"],
);

/// *karkyādi-gaṇa* (217), first used in the following *sūtra*:
///
/// > 6.2.87 *prasthē'vr̥ddhamakarkyādīnām*
pub(crate) const KARKYADI: GanapathaEntry = GanapathaEntry::basic(
    "karkyAdiH",
    217,
    "6.2.87",
    &[
        "karki", "karkI", "makarI", "karkanDu", "SamI", "karIri", "karIra", "kanduka", "kuvala",
        "kavala", "badarI",
    ],
);

/// *mālādi-gaṇa* (218), first used in the following *sūtra*:
///
/// > 6.2.88 *mālādīnāṁ ca*
pub(crate) const MALADI: GanapathaEntry = GanapathaEntry::basic(
    "mAlAdiH",
    218,
    "6.2.88",
    &[
        "mAlA", "SAlA", "SoRA", "SoRa", "drAkzA", "srAkzA", "kzAmA", "kAYcI", "eka", "kAma",
    ],
);

/// *kratvādi-gaṇa* (219), first used in the following *sūtra*:
///
/// > 6.2.118 *kratvādayaśca*
pub(crate) const KRATVADI: GanapathaEntry = GanapathaEntry::basic(
    "kratvAdiH",
    219,
    "6.2.118",
    &[
        "kratu", "dfSIka", "pratIka", "pratUrti", "havya", "Bavya", "Baga",
    ],
);

/// *cihaṇādi-gaṇa* (220), first used in the following *sūtra*:
///
/// > 6.2.125 *ādiścihaṇādīnām*
pub(crate) const CIHANADI: GanapathaEntry = GanapathaEntry::basic(
    "cihaRAdiH",
    220,
    "6.2.125",
    &[
        "cihaRa",
        "madura",
        "madugura",
        "vEtula",
        "pawatka",
        "vEqAlikarRaka",
        "vEqAlikarRi",
        "kukkuwa",
        "cikkaRa",
        "citkaRa",
    ],
);

/// *vargyādi-gaṇa* (221), first used in the following *sūtra*:
///
/// > 6.2.131 *vargyādayaśca*
pub(crate) const VARGYADI: GanapathaEntry = GanapathaEntry::basic(
    "vargyAdiH",
    221,
    "6.2.131",
    &[
    // <<digAdizu vargyAdayasta eva kftayadantAH>>
],
);

/// *cūrṇādi-gaṇa* (222), first used in the following *sūtra*:
///
/// > 6.2.134 *cūrṇādīnyaprāṇiṣaṣṭhyāḥ*
pub(crate) const CURNADI: GanapathaEntry = GanapathaEntry::basic(
    "cUrRAdiH",
    222,
    "6.2.134",
    &[
        "cUrRa", "kariza", "karIza", "SAkina", "SAkawa", "drAkzA", "tUsta", "kundama", "dalapa",
        "camasI", "cakkana", "cOla",
    ],
);

/// *kāṇḍādi-gaṇa* (223), first used in the following *sūtra*:
///
/// > 6.2.135 *ṣaṭ ca kāṇḍādīni*
pub(crate) const KANDADI: GanapathaEntry = GanapathaEntry::basic(
    "kARqAdiH",
    223,
    "6.2.135",
    &["kARqa", "cIra", "palala", "sUza", "SAka", "kUla"],
);

/// *vanaspatyādi-gaṇa* (224), first used in the following *sūtra*:
///
/// > 6.2.140 *ubhē vanaspatyādiṣu yugapat*
pub(crate) const VANASPATYADI: GanapathaEntry = GanapathaEntry::basic(
    "vanaspatyAdiH",
    224,
    "6.2.140",
    &[
        "vanaspatiH",
        "bfhaspatiH",
        "SacIpatiH",
        "tanUnapAt",
        "narASaMsaH",
        "SunaHSepaH",
        "SaRqAmarkO",
        "tfzRAvarUtri",
        "lambAviSvavayasO",
        "marmftyuH",
    ],
);

/// *ācitādi-gaṇa* (225), first used in the following *sūtra*:
///
/// > 6.2.146 *saṁjñāyāmanācitādīnām*
pub(crate) const ACITADI: GanapathaEntry = GanapathaEntry::basic(
    "AcitAdiH",
    225,
    "6.2.146",
    &[
        "Acita",
        "paryAcita",
        "AsTApita",
        "parigfhIta",
        "nirukta",
        "pratipanna",
        "apaSlizwa",
        "praSlizwa",
        "upahita",
        "upasTita",
        "saMhitAgavi",
    ],
);

/// *pravr̥ddhādi-gaṇa* (226), first used in the following *sūtra*:
///
/// > 6.2.147 *pravr̥ddhādīnāṁ ca*
pub(crate) const PRAVRDDHADI: GanapathaEntry = GanapathaEntry::akrti(
    "pravfdDAdiH",
    226,
    "6.2.147",
    &[
        // <<pravfdDaM yAnam>>
        // <<pravfdDo vfzalaH>>
        // <<prayutA sazRavaH>>
        // <<Akarze avahitaH>>
        // <<avahito Bogezu>>
        "KawvArUQaH",
        "kaviSastaH",
    ],
);

/// *cārvādi-gaṇa* (227), first used in the following *sūtra*:
///
/// > 6.2.160 *kr̥tyōkēṣṇuccārvādayaśca*
pub(crate) const CARVADI: GanapathaEntry = GanapathaEntry::basic(
    "cArvAdiH",
    227,
    "6.2.160",
    &[
        "cAru",
        "sADu",
        "yODika",
        "yODaki",
        "anaNgamejaya",
        "vadAnya",
        "akasmAt",
        // <<vartamAnavarDamAnatvaramARaDriyamARakriyamARarocamAnaSoBamAnAH saMjYAyAm>>
        // <<vikArasadfSe vyastasamaste>>
        "gfhapati",
        "gfhapatika",
        // <<rAjAhnoSCandasi>>
    ],
);

/// *guṇādi-gaṇa* (228), first used in the following *sūtra*:
///
/// > 6.2.176 *na guṇādayō'vayavāḥ*
pub(crate) const GUNADI: GanapathaEntry = GanapathaEntry::akrti(
    "guRAdiH",
    228,
    "6.2.176",
    &["guRa", "akzara", "aDyAya", "sUkka", "CandomAna"],
);

/// *nirudakādi-gaṇa* (229), first used in the following *sūtra*:
///
/// > 6.2.184 *nirudakādīni ca*
pub(crate) const NIRUDAKADI: GanapathaEntry = GanapathaEntry::akrti(
    "nirudakAdiH",
    229,
    "6.2.184",
    &[
        "nirudaka",
        "nirupala",
        "nirmakzika",
        "nirmaSaka",
        "nizkAlaka",
        "nizkAlika",
        "nizpeza",
        "dustarIpa",
        "nistarIpa",
        "nistarIka",
        "nirajina",
        "udajina",
        "upAjina",
        "parihastapAdakeSakarzAH",
    ],
);

/// *aṁśvādi-gaṇa* (230), first used in the following *sūtra*:
///
/// > 6.2.193 *pratēraṁśvādayastatpuruṣē*
pub(crate) const AMSHVADI: GanapathaEntry = GanapathaEntry::basic(
    "aMSvAdiH",
    230,
    "6.2.193",
    &[
        "aMSu", "jana", "rAjan", "uzwra", "Kewaka", "ajira", "ArdrA", "SravaRa", "kfttikA",
        "arDapura",
    ],
);

/// *gaurādi-gaṇa* (231), first used in the following *sūtra*:
///
/// > 6.2.194 *upād dvyajajinamagaurādayaḥ*
pub(crate) const GAURADI_6: GanapathaEntry = GanapathaEntry::basic(
    "gOrAdiH",
    231,
    "6.2.194",
    &[
        "gOra", "tEza", "tEla", "lewa", "lowa", "jihvA", "kfzRA", "kanyA", "guDa", "kalya", "pAda",
    ],
);

/// *tricakrād-gaṇa* (232), first used in the following *sūtra*:
///
/// > 6.2.129 *kūlasūdasthalakarṣāḥ saṁjñāyām*
/// The *sūtra* has the following *vārttika*:
///
/// > *tricakrAdInAM CandasyupasaMKyAnam*
pub(crate) const TRICAKRADI: GanapathaEntry = GanapathaEntry::akrti(
    "tricakrAdi",
    232,
    "6.2.129",
    &["tricakra", "trivft", "trivaNkara"],
)
.with_varttika("tricakrAdInAM CandasyupasaMKyAnam");

/// *priyādi-gaṇa* (233), first used in the following *sūtra*:
///
/// > 6.3.34 *striyāḥ puṁvadbhāṣitapuṁskādanūṅ samānādhikaraṇē striyāmapūraṇīpriyā''diṣu*
pub(crate) const PRIYADI: GanapathaEntry = GanapathaEntry::basic(
    "priyAdiH",
    233,
    "6.3.34",
    &[
        "priyA", "manojYA", "kalyARI", "suBagA", "durBagA", "BaktiH", "sAcavA", "svasA", "svA",
        "kAntA", "kzAntA", "samA", "capalA", "duhitA", "vAmA", "abalA", "tanayA",
    ],
);

/// *tasilādi-gaṇa* (234), first used in the following *sūtra*:
///
/// > 6.3.35 *tasilādiṣvākr̥tvasucaḥ*
pub(crate) const TASILADI: GanapathaEntry = GanapathaEntry::basic(
    "tasilAdiH",
    234,
    "6.3.35",
    &[
        "tasil", "trat", "tarap", "tamap", "caraw", "jAtIyar", "kalpap", "deSIyar", "rUpap",
        "pASap", "Tal", "TAl", "dArhil", "til", "Tyan",
    ],
);

/// *kukkuṭyādi-gaṇa* (235), first used in the following *sūtra*:
///
/// > 6.3.42 *puṁvat karmadhārayajātīyadēśīyēṣu*
/// The *sūtra* has the following *vārttika*:
///
/// > *kukkuwyAdInAmaRqAdizu*
pub(crate) const KUKKUTYADI: GanapathaEntry = GanapathaEntry::basic(
    "kukkuwyAdiH",
    235,
    "6.3.42",
    &[
        "kukkuwI", "mfgI", "kAkI", "aRqa", "pada", "SAva", "BrukuMsa", "BrukuwI",
    ],
)
.with_varttika("kukkuwyAdInAmaRqAdizu");

/// *pr̥ṣōdarādi-gaṇa* (236), first used in the following *sūtra*:
///
/// > 6.3.109 *pr̥ṣōdarādīni yathōpadiṣṭam*
pub(crate) const PRSHODARADI: GanapathaEntry = GanapathaEntry::basic(
    "pfzodarAdiH",
    236,
    "6.3.109",
    &[
        "pfzodara",
        "pfzotTAna",
        "balAhaka",
        "jImUta",
        "SmaSAna",
        "ulUKala",
        "piSAca",
        "bfsI",
        "mayUra",
    ],
);

/// *kōṭarādi-gaṇa* (237), first used in the following *sūtra*:
///
/// > 6.3.117 *vanagiryōḥ saṁjñāyāṁ kōṭarakiṁśulakādīnām*
pub(crate) const KOTARADI: GanapathaEntry = GanapathaEntry::basic(
    "kowarAdiH",
    237,
    "6.3.117",
    &["kowara", "miSraka", "siGraka", "puraga", "sArika", "SArika"],
);

/// *kiṁśulukādi-gaṇa* (238), first used in the following *sūtra*:
///
/// > 6.3.117 *vanagiryōḥ saṁjñāyāṁ kōṭarakiṁśulakādīnām*
pub(crate) const KIMSHULUKADI: GanapathaEntry = GanapathaEntry::basic(
    "kiMSulukAdiH",
    238,
    "6.3.117",
    &[
        "kiMSuluka",
        "SAlva",
        "naqa",
        "aYjana",
        "BaYjana",
        "lohita",
        "kukkuwa",
    ],
);

/// *ajirādi-gaṇa* (239), first used in the following *sūtra*:
///
/// > 6.3.119 *matau bahvacō'najirādīnām*
pub(crate) const AJIRADI: GanapathaEntry = GanapathaEntry::basic(
    "ajirAdiH",
    239,
    "6.3.119",
    &[
        "ajira",
        "Kadira",
        "pulina",
        "haMsaka",
        "haMsa",
        "kAraRqa",
        "kAraRqava",
        "cakravAka",
    ],
);

/// *śarādi-gaṇa* (240), first used in the following *sūtra*:
///
/// > 6.3.120 *śarādīnāṁ ca*
pub(crate) const SHARADI_6: GanapathaEntry = GanapathaEntry::basic(
    "SarAdiH",
    240,
    "6.3.120",
    &[
        "Sara", "vaMSa", "DUma", "ahi", "kapi", "maRi", "muni", "Suci", "hanu",
    ],
);

/// *pīlvādi-gaṇa* (241), first used in the following *sūtra*:
///
/// > 6.3.121 *ikaḥ vahē'pīlōḥ*
/// The *sūtra* has the following *vārttika*:
///
/// > *apIlvAdInAmiti vaktavyam*
pub(crate) const PILVADI_6: GanapathaEntry = GanapathaEntry::basic(
    "pIlvAdiH",
    241,
    "6.3.121",
    &["pIlu", "dAru", "ruci", "cAru", "gam", "kam"],
)
.with_varttika("apIlvAdInAmiti vaktavyam");

/// *bilvakādi-gaṇa* (242), first used in the following *sūtra*:
///
/// > 6.4.153 *bilvakādibhyaśchasya luk*
pub(crate) const BILVAKADI: GanapathaEntry = GanapathaEntry::basic(
    "bilvakAdiH",
    242,
    "6.4.153",
    &[
        "bilvaka", "veRu", "vetra", "vetasa", "tfRa", "ikzu", "kAzWa", "kapota", "kruYcA",
        "takzan",
        // (naqAdyantargaRo bilvAdiH
        // CaviDAnArTaM ye naqAdayaste yadA CasaMniyoge kftakugAgamAste bilvakAdayaH )
    ],
);

/// *snātvyādi-gaṇa* (243), first used in the following *sūtra*:
///
/// > 7.1.49 *snātvyādayaśca*
pub(crate) const SNATVYADI: GanapathaEntry =
    GanapathaEntry::akrti("snAtvyAdiH", 243, "7.1.49", &["snAtvI", "pItvI"]);

/// *dvārādi-gaṇa* (244), first used in the following *sūtra*:
///
/// > 7.3.4 *dvārādīnāṁ ca*
pub(crate) const DVARADI: GanapathaEntry = GanapathaEntry::basic(
    "dvArAdiH",
    244,
    "7.3.4",
    &[
        "dvAra", "svara", "svagrAma", "svADyAya", "vyalkaSa", "svasti", "svar", "sPyakft", "svAdu",
        "mfdu", "Svas", "Svan", "sva",
    ],
);

/// *svāgatādi-gaṇa* (245), first used in the following *sūtra*:
///
/// > 7.3.7 *svāgatādīnāṁ ca*
pub(crate) const SVAGATADI: GanapathaEntry = GanapathaEntry::basic(
    "svAgatAdiH",
    245,
    "7.3.7",
    &[
        "svAgata",
        "svaDvara",
        "svaNga",
        "vyaNga",
        "vyaqa",
        "vyavahAra",
        "svapati",
        "svapiti",
    ],
);

/// *anuśatikādi-gaṇa* (246), first used in the following *sūtra*:
///
/// > 7.3.20 *anuśatikādīnāṁ ca*
pub(crate) const ANUSHATIKADI: GanapathaEntry = GanapathaEntry::akrti(
    "anuSatikAdiH",
    246,
    "7.3.20",
    &[
        "anuSatika",
        "anuhoqa",
        "anusaMvaraRa",
        "anusaMcaraRa",
        "anusaMvatsara",
        "aNgAraveRu",
        "asihatya",
        "syahat",
        "asyaheti",
        "vaDyoga",
        "puzkarasad",
        "anuharat",
        "kurukata",
        "kurupaYcAla",
        "udakaSudDa",
        "ihaloka",
        "paraloka",
        "sarvaloka",
        "sarvapuruza",
        "sarvaBUmi",
        "prayoga",
        "parastrI",
        // <<rAjapuruzAtzyaYi>>
        "sUtranaqa",
        "aBigama",
        "aDiBUta",
        "aDideva",
        "caturvidyA",
    ],
);

/// *kṣipakādi-gaṇa* (247), first used in the following *sūtra*:
///
/// > 7.3.45 *na yāsayōḥ*
/// The *sūtra* has the following *vārttika*:
///
/// > *kzipakAdInAM copasaMKyAnam*
pub(crate) const KSHIPAKADI: GanapathaEntry = GanapathaEntry::akrti(
    "kzipakAdiH",
    247,
    "7.3.45",
    &[
        "kzipakA", "DruvakA", "carakA", "sevakA", "karakA", "cawakA", "avakA", "lahakA", "alakA",
        "kanyakA", "DruvakA", "eqakA",
    ],
)
.with_varttika("kzipakAdInAM copasaMKyAnam");

/// *nyaṅkvādi-gaṇa* (248), first used in the following *sūtra*:
///
/// > 7.3.53 *nyaṅkvādīnāṁ ca*
pub(crate) const NYANKVADI: GanapathaEntry = GanapathaEntry::basic(
    "nyaNkvAdiH",
    248,
    "7.3.53",
    &[
        "nyaNku",
        "madgu",
        "Bfgu",
        "dUrepAka",
        "valepAka",
        "kzaRepAka",
        "dUrepAkA",
        "PalepAkA",
        "dUrepAku",
        "PalapAku",
        "takra",
        "tatra",
        "vakra",
        "cakra",
        "vyatizaNga",
        "anuzaNga",
        "avasarga",
        "upasarga",
        "SvapAka",
        "mAMsapAka",
        "mAsapAka",
        "kapotapAka",
        "ulUkapAka",
        "saMjYAyAM",
        // <<meGanidAGAvadAGArGAH>>
        "nyagroDa",
        "vIrut",
    ],
);

/// *kaṇādi-gaṇa* (249), first used in the following *sūtra*:
///
/// > 7.4.3 *bhrājabhāsabhāṣadīpajīvamīlapīḍāmanyatarasyām*
/// The *sūtra* has the following *vārttika*:
///
/// > *kARAdInAM ceti vaktavyam*
pub(crate) const KANADI: GanapathaEntry = GanapathaEntry::basic(
    "kaRAdiH",
    249,
    "7.4.3",
    &[
        "kaRa", "raRa", "BaRa", "SraRa", "lupa", "heWa", "vhAyi", "vARi", "cARi", "lowi", "loWi",
        "lopi",
    ],
)
.with_varttika("kARAdInAM ceti vaktavyam");

/// *gōtrādi-gaṇa* (250), first used in the following *sūtra*:
///
/// > 8.1.27 *tiṅō gōtrādīni kutsanābhīkṣṇyayōḥ*
pub(crate) const GOTRADI: GanapathaEntry = GanapathaEntry::basic(
    "gotrAdiH",
    250,
    "8.1.27",
    &[
        "gotra",
        "bruva",
        "pravacana",
        "prahasana",
        "prakaTana",
        "pratyayana",
        "prapaYca",
        "prAya",
        "nyAya",
        "pracakzaRa",
        "vicakzaRa",
        "avacakzaRa",
        "svADyAya",
        "BUyizWa",
        "BUyizwa",
        "vAnAma",
    ],
);

/// *kāṣṭhādi-gaṇa* (251), first used in the following *sūtra*:
///
/// > 8.1.67 *pūjanāt pūjitamanudāttaṁ kāṣṭhādibhyaḥ*
pub(crate) const KASHTHADI: GanapathaEntry = GanapathaEntry::basic(
    "kAzWAdiH",
    251,
    "8.1.67",
    &[
        "kAzWa",
        "dAruRa",
        "amAtAputra",
        "veSa",
        "anAjYAta",
        "anujYAta",
        "aputra",
        "ayuta",
        "adButa",
        "anukta",
        "BfSa",
        "Gora",
        "suKa",
        "parama",
        "su",
        "ati",
    ],
);

/// *yavādi-gaṇa* (252), first used in the following *sūtra*:
///
/// > 8.2.9 *mādupadhāyāśca matōrvō'yavādibhyaḥ*
pub(crate) const YAVADI_8: GanapathaEntry = GanapathaEntry::akrti(
    "yavAdiH",
    252,
    "8.2.9",
    &[
        "yava", "dalmi", "Urmi", "urmi", "BUmi", "kfmi", "kruYcA", "vaSA", "drAkzA", "DrAkzA",
        "Graji", "vraji", "Dvaji", "niji", "siji", "saYji", "harit", "kakuda", "marut", "garut",
        "ikzudru", "maDu",
    ],
);

/// *kapilakādi-gaṇa* (253), first used in the following *sūtra*:
///
/// > 8.2.18 *kr̥pō rō laḥ*
/// The *sūtra* has the following *vārttika*:
///
/// > *kapilakAdInAM saMjYACandasorveti vAcyam*
pub(crate) const KAPILAKADI: GanapathaEntry = GanapathaEntry::akrti(
    "kapilakAdiH",
    253,
    "8.2.18",
    &[
        "kapilaka",
        "nirvilIka",
        "lomAni",
        "pAMsula",
        "kalpa",
        "Sukla",
        "kapilakA",
        "tarpilikA",
        "tarpili",
    ],
)
.with_varttika("kapilakAdInAM saMjYACandasorveti vAcyam");

/// *aharādi-gaṇa* (254), first used in the following *sūtra*:
///
/// > 8.2.70 *amnarūdharavarityubhayathā chandasi*
/// The *sūtra* has the following *vārttika*:
///
/// > *aharAdInAM patyAdizu vA rePaH*
pub(crate) const AHARADI: GanapathaEntry =
    GanapathaEntry::basic("aharAdiH", 254, "8.2.70", &["ahar", "gIr", "Dur"])
        .with_varttika("aharAdInAM patyAdizu vA rePaH");

/// *patyādi-gaṇa* (255), first used in the following *sūtra*:
///
/// > 8.2.70 *amnarūdharavarityubhayathā chandasi*
/// The *sūtra* has the following *vārttika*:
///
/// > *aharAdInAM patyAdizu vA rePaH*
pub(crate) const PATYADI: GanapathaEntry =
    GanapathaEntry::basic("patyAdiH", 255, "8.2.70", &["pati", "gaRa", "putra"])
        .with_varttika("aharAdInAM patyAdizu vA rePaH");

/// *kaskādi-gaṇa* (256), first used in the following *sūtra*:
///
/// > 8.3.48 *kaskādiṣu ca*
pub(crate) const KASKADI: GanapathaEntry = GanapathaEntry::akrti(
    "kaskAdiH",
    256,
    "8.3.48",
    &[
        "kaskaH",
        "kOtaskutaH",
        "BrAtuzputraH",
        "SunaskarRaH",
        "sadyaskAlaH",
        "sadyaskIH",
        "sAdyaskaH",
        "kAMskAn",
        "sarpizkuRqikA",
        "DanuzkapAlam",
        "bahizpalam",
        "barhizpalam",
        "yajuzpAtram",
        "ayaskAntaH",
        "tamaskARqaH",
        "ayaskARqaH",
        "medaspiRqaH",
        "BAskaraH",
        "ahaskaraH",
    ],
);

/// *suṣāmādi-gaṇa* (257), first used in the following *sūtra*:
///
/// > 8.3.98 *suṣāmādiṣu ca*
pub(crate) const SUSHAMADI: GanapathaEntry = GanapathaEntry::akrti(
    "suzAmAdiH",
    257,
    "8.3.98",
    &[
        "suzAmA",
        "niHzAmA",
        "duHzAmA",
        "suzeDaH",
        "nizeDaH",
        "niHzeDaH",
        "duHzeDaH",
        "suzaMDiH",
        "niHzaMDiH",
        "duHzaMDiH",
        "suzWu",
        "duzWu",
        "gOrizakTaH",
        "saMjYAyAm",
        "pratizRikA",
        "jalAzAham",
        "jalAzAqam",
        "nOzecanam",
        "dunduBizevaRam",
        "dunduBizecanam",
        // <<eti saMjYAyAmagAt>>
        // <<nakzatrAdvA>>
        "harizeRaH",
        "rohiRIzeRaH",
    ],
);

/// *savanādi-gaṇa* (258), first used in the following *sūtra*:
///
/// > 8.3.110 *na raparasr̥pisr̥jispr̥śispr̥hisavanādīnām*
pub(crate) const SAVANADI: GanapathaEntry = GanapathaEntry::basic(
    "savanAdiH",
    258,
    "8.3.110",
    &[
        // savane savane
        // sUte sUte
        // some some
        // savanamuKe savanamuKe
        // kiMsam kiMsam
        // kiMsaH kiMsaH
        "anusavanamanusavanam",
        // gosaniM gosanim
        "aSvasanimaSvasanim",
        // <<saMjYAyAM bfhaspatisavaH>>
        "Sakunisavanam",
        // some some
        // sute sute
        // saMvatsare saMvatsare
        // bisam bisam
        // kisam kisam
        // musalam musalam
        "gosanimaSvasanim",
    ],
);

/// *irikādi-gaṇa* (259), first used in the following *sūtra*:
///
/// > 8.4.6 *vibhāṣauṣadhivanaspatibhyaḥ*
/// The *sūtra* has the following *vārttika*:
///
/// > *irikAdiByaH pratizeGo vaktavyaH*
pub(crate) const IRIKADI: GanapathaEntry =
    GanapathaEntry::akrti("irikAdiH", 259, "8.4.6", &["irikA", "mirikA", "timirA"])
        .with_varttika("irikAdiByaH pratizeGo vaktavyaH");

/// *girinadyādi-gaṇa* (260), first used in the following *sūtra*:
///
/// > 8.4.10 *vā bhāvakaraṇayōḥ*
/// The *sūtra* has the following *vārttika*:
///
/// > *vAprakaraRegirinadyAdInAM copasaMKyAnam*
pub(crate) const GIRINADYADI: GanapathaEntry = GanapathaEntry::akrti(
    "girinadyAdiH",
    260,
    "8.4.10",
    &[
        "girinadI",
        "girinaKa",
        "girinadDa",
        "girinitamba",
        "cakranadI",
        "cakranitamba",
        "taryamAna",
        "mAzona",
        "Argayana",
    ],
)
.with_varttika("vAprakaraRegirinadyAdInAM copasaMKyAnam");

/// *yuvādi-gaṇa* (261), first used in the following *sūtra*:
///
/// > 8.4.11 *prātipadikāntanumvibhaktiṣu ca*
/// The *sūtra* has the following *vārttika*:
///
/// > *yuvAderna*
pub(crate) const YUVADI_8: GanapathaEntry = GanapathaEntry::akrti(
    "yuvAdiH",
    261,
    "8.4.11",
    &[
        "yuvan",
        "pakva",
        "ahan",
        // AryayUnA kzatriyayunA prapakvAni
        "paripakvAni",
        "dIrGAhnI",
    ],
)
.with_varttika("yuvAderna");

/// *kṣubhnādi-gaṇa* (262), first used in the following *sūtra*:
///
/// > 8.4.39 *kṣubhnā''diṣu ca*
pub(crate) const KSHUBHNADI: GanapathaEntry = GanapathaEntry::akrti(
    "kzuBnAdiH",
    262,
    "8.4.39",
    &[
        // <<kzuBnA-tfpnu-nfgamana-nandin-nandana-nagara-etAnyuttarapadAni saMjYAyAM prayojayanti>>
        "harinandI",
        "harinandanaH",
        "girinagaram",
        // <<nftiryaNi prayojayanti>>
        "narInftyata",
        // <<nartana-gahana-nandana-niveSa-nivAsa-agni-anUpa-etAnyuttarapadAni prayojayanti>>
        "parinartanam",
        "parigahanam",
        "parinandanam",
        "SaraniveSaH",
        "SaranivAsaH",
        "SarAgniH",
        "darBAnUpa",
        // <<AcAryAdaRatvaM ca>>
        "AcAryaBogInaH",
        "caturhAyana",
        // <<irikAdIni vanottarapadAni saMjYAyAm>>
        "irikA",
        "timira",
        "samIra",
        "kubera",
        "hari",
        "karmAra",
    ],
);

/// Returns an ordered iterator over all *gaṇa-sūtra*s.
pub fn all_sutras() -> impl Iterator<Item = &'static GanapathaEntry> {
    const SUTRAS: &[GanapathaEntry] = &[
        SARVADI,
        SVARADI,
        CADI,
        PRADI,
        URYADI,
        SAKSHATPRABHRTI,
        TISHTHADGUPRABHRTI,
        SHAUNDADI,
        PATRESAMITADI,
        VYAGHRADI,
        SHRENYADI,
        KRTADI,
        SHAKAPARTHIVADI,
        SHRAMANADI,
        MAYURAVYAMSAKADI,
        YAJAKADI,
        RAJADANTADI,
        AHITAGNYADI,
        KADARADI,
        NAVADI,
        PRAKRTYADI,
        PRATYADI,
        GAVASHVADI,
        DADHIPAYASYADI,
        ARDHARCADI,
        PAILADI,
        TAULVALYADI,
        YASKADI,
        GOPAVANADI,
        TIKAKITAVADI,
        UPAKADI,
        BHRSHADI,
        LOHITADI,
        SUKHADI,
        KANDVADI,
        NANDYADI,
        GRAHYADI,
        PACADI,
        MULAVIBHUJADI,
        PARSHVADI,
        GAMYADI,
        BHIDADI,
        SAMPADADI,
        BHIMADI,
        AJADI,
        SVASRADI,
        SAMANADI,
        GAURADI,
        BAHVADI,
        KRODADI,
        SHARNGARAVADI,
        KRAUDYADI,
        ASHVAPATYADI,
        UTSADI,
        BAAHVADI,
        KUNJADI,
        NADADI,
        BIDADI,
        GARGADI,
        ASHVADI,
        SHIVADI,
        SHUBHRADI,
        KALYANYADI,
        GRSHTYADI,
        REVATYADI,
        KURVADI,
        TIKADI,
        VAKINADI,
        KAMBOJADI,
        BHARGADI,
        YAUDHEYADI,
        BHIKSHADI,
        KHANDIKADI,
        PASHADI,
        KHALADI,
        RAJANYADI,
        BHAURIKYADI,
        AISHUKARYADI,
        UKTHADI,
        KRAMADI,
        VASANTADI,
        SANKALADI,
        SUVASTVADI,
        ARIHANADI,
        KRSHASHVADI,
        RSHYADI,
        KUMUDADI,
        KASHADI,
        TRNADI,
        PREKSHADI,
        ASHMADI,
        SAKHYADI,
        SANKASHADI,
        BALADI,
        PAKSHADI,
        KARNADI,
        SUTANGAMADI,
        PRAGADYADI,
        VARAHADI,
        KUMUDADI_4,
        VARANADI,
        MADHVADI,
        UTKARADI,
        NADADI_4,
        KATRYADI,
        NADYADI,
        PALADYADI,
        KASHYADI,
        DHUMADI,
        KACCHADI,
        GAHADI,
        SANDHIVELADI,
        DIGADI,
        PARIMUKHADI,
        ADHYATMADI,
        RGAYANADI,
        SHUNDIKADI,
        SHANDIKADI,
        SINDHVADI,
        TAKSHASHILADI,
        SHAUNAKADI,
        KULALADI,
        RAIVATIKADI,
        BILVADI,
        PALASHADI,
        SHARADI,
        TALADI,
        RAJATADI,
        PLAKSHADI,
        HARITAKYADI,
        MASHABDADI,
        PRABHRTADI,
        SUSNATADI,
        PARADARADI,
        PARPADI,
        VETANADI,
        UTSANGADI,
        BHASTRADI,
        AKSHADYUTADI,
        MAHISHYADI,
        KISARADI,
        CHATRADI,
        PRATIJANADI,
        KATHADI,
        GUDADI,
        GAVADI,
        APUPADI,
        NISHKADI,
        ASHVADI_5,
        VAMSHADI,
        CHEDADI,
        DANDADI,
        MAHANAMNYADI,
        AVANTARADIKSHADI,
        VYUSHTADI,
        AGNIPADADI,
        SAMTAPADI,
        UPAVASTRADI,
        ANUPRAVACANADI,
        SVARGADI,
        PUNYAHAVACANADI,
        PRTHVADI,
        VARNADRDHADI,
        BRAHMANADI,
        CATURVEDADI,
        PUROHITADI,
        UDGATRADI,
        YUVADI,
        MANOJNADI,
        PILVADI,
        KARNADI_5,
        TARAKADI,
        VIMUKTADI,
        GOSHADADI,
        AKARSHADI,
        ISHTADI,
        RASADI,
        SIDHMADI,
        LOMADI,
        PAMADI,
        PICCHADI,
        JYOTSNADI,
        VRIHYADI,
        TUNDADI,
        ARSHAADI,
        SUKHADI_5,
        PUSHKARADI,
        BALADI_5,
        BHAVADADI,
        DEVAPATHADI,
        SHAKHADI,
        SHARKARADI,
        ANGULYADI,
        DAMANYADI,
        PARSHVADI_5,
        YAUDHEYADI_5,
        STHULADI,
        YAVADI,
        VINAYADI,
        PRAJNADI,
        ADYADI,
        SHARADADI,
        DVIDANDYADI,
        HASTYADI,
        KUMBHAPADYADI,
        URAHPRABHRTAYAH,
        SHAKANDHVADI,
        PARASKARADI,
        UNCHADI,
        VRSHADI,
        VISPASHTADI,
        KARTAKAUJAPADI,
        KURUGARHAPATADI,
        YUKTAROHYADI,
        GHOSHADI,
        CHATRYADI,
        KARKYADI,
        MALADI,
        KRATVADI,
        CIHANADI,
        VARGYADI,
        CURNADI,
        KANDADI,
        VANASPATYADI,
        ACITADI,
        PRAVRDDHADI,
        CARVADI,
        GUNADI,
        NIRUDAKADI,
        AMSHVADI,
        GAURADI_6,
        TRICAKRADI,
        PRIYADI,
        TASILADI,
        KUKKUTYADI,
        PRSHODARADI,
        KOTARADI,
        KIMSHULUKADI,
        AJIRADI,
        SHARADI_6,
        PILVADI_6,
        BILVAKADI,
        SNATVYADI,
        DVARADI,
        SVAGATADI,
        ANUSHATIKADI,
        KSHIPAKADI,
        NYANKVADI,
        KANADI,
        GOTRADI,
        KASHTHADI,
        YAVADI_8,
        KAPILAKADI,
        AHARADI,
        PATYADI,
        KASKADI,
        SUSHAMADI,
        SAVANADI,
        IRIKADI,
        GIRINADYADI,
        YUVADI_8,
        KSHUBHNADI,
    ];
    SUTRAS.iter()
}
